<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Tim College  <tcollege@interworx.com>
 * @date       Aug 5, 2008
 */

/**
 * IWorx Selenium helper class.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class IWorxSeleniumTestCase extends PHPUnit_Extensions_Selenium2TestCase {
  const NW_USER = NW_USER;
  const NW_PASS = NW_PASS;

  const SECONDARY_NICK  = SECONDARY_NICK;
  const SECONDARY_EMAIL = SECONDARY_EMAIL;
  const SECONDARY_PASS  = SECONDARY_PASS;

  const RSL_NICKNAME = RSL_NICKNAME;
  const RSL_EMAIL    = RSL_EMAIL;
  const RSL_PASSWORD = RSL_PASSWORD;
  const RSL_STATUS   = User::ACTIVE;

  const SW_DOMAIN   = SW_DOMAIN;
  const SW_UNIQNAME = SW_UNIQNAME;
  const SW_EMAIL    = SW_EMAIL;
  const SW_PASS     = SW_PASS;
  const SW_NICK     = SW_NICK;

  const SW_SECONDARY_DOMAIN = SW_SECONDARY_DOMAIN;

  const SW_SECONDARY_NICK  = SW_SECONDARY_NICK;
  const SW_SECONDARY_EMAIL = SW_SECONDARY_EMAIL;
  const SW_SECONDARY_PASS  = SW_SECONDARY_PASS;

  const SW_POINTER_DOMAIN = SW_POINTER_DOMAIN;

  const SW_PACKAGE = 'Package By Test';

  const TIMEOUT = 60; //seconds

  protected $captureScreenshotOnFailure = false;
  protected $screenshotPath = 'c:\\selenium-screenshots\\';
  protected $screenshotUrl = '';

  /**
   * Sets up the browser url.
   */
  public function setUp() {
    $this->setHost( PHPUNIT_TESTSUITE_EXTENSION_SELENIUM_HOST );
    $this->setPort( (int)PHPUNIT_TESTSUITE_EXTENSION_SELENIUM_PORT );
    $this->setBrowser( PHPUNIT_TESTSUITE_EXTENSION_SELENIUM_BROWSER );

    exec( '/bin/hostname', $result );
    $this->setBrowserUrl( "http://{$result[0]}:2080/" );
    $this->screenshotUrl = "http://{$result[0]}:2080/";

    Ini::set( Ini::IWORX, 'success_banner_timeout', '65000' );
    Ini::write();
    $this->setSeleniumServerRequestsTimeout( 120000 );
  }

  /**
   * Set browser url.
   *
   * @param string $url
   */
  public function setBrowserUrl( $url ) {
    parent::setBrowserUrl( $url );
    $this->_browser_url = $url;
  }

  /**
   * Get browser url.
   *
   * @return string
   */
  public function getBrowserUrl() {
    return $this->_browser_url;
  }

  /**
   * Shortcut for making a fail message.
   *
   * @param string $message
   * @return string
   */
  public function makeFailMessage( $message ) {
    // All aboard the FAILBOAT!
    $location = $this->getLocation();

    return 'Page ' . $location . ' ' . $message;
  }

  /**
   * Logs the Selenium browser into NodeWorx.
   *
   * @param string $user     User email address to log in as
   * @param string $password User password
   */
  public function loginToNWAsUser( $user     = self::NW_USER,
                                   $password = self::NW_PASS ) {
    $this->typeInto( 'email', $user );
    $this->typeInto( 'password', $password );
    $this->clickAndWait( 'submit' );
  }

  /**
   * Logs the Selenium browser into SiteWorx.
   *
   * @param string $user   User email address to log in as
   * @param string $pass   Password
   * @param string $domain Domain
   */
  public function loginToSWAsUser( $user   = self::SW_EMAIL,
                                   $pass   = self::SW_PASS,
                                   $domain = self::SW_DOMAIN ) {
    $this->typeInto( 'email', $user );
    $this->typeInto( 'password', $pass );
    $this->typeInto( 'domain', $domain );
    $this->clickAndWait( 'submit' );
  }

  /**
   * Logs in to NodeWorx and verifies that redirect worked as expected.
   *
   * @param string $page
   * @param string $user
   * @param string $pass
   */
  public function loginToNWAndGoTo( $page,
                                    $user = self::NW_USER,
                                    $pass = self::NW_PASS ) {

    $this->open( $page );
    $this->waitForPageToLoad( '60000' );

    // if we've been redirected to the login page...login.
    if( strstr( $this->getLocation(), '/nodeworx/index' ) ) {
      $this->waitForElement( 'email' );
      $this->loginToNWAsUser( $user, $pass );
    }
    $this->waitForElement( 'iw-ft' );
    $this->assertContains( $page, $this->getLocation() );
    $this->assertPageRenderedWithoutErrors();
  }

  /**
   * Logs you in to SiteWorx and redirects.
   *
   * @param string $page
   * @param string $user
   * @param string $pass
   * @param string $domain
   */
  public function loginToSWAndGoTo( $page,
                                    $user = self::SW_EMAIL,
                                    $pass = self::SW_PASS,
                                    $domain = self::SW_DOMAIN ) {

    $this->open( $page );
    $this->waitForPageToLoad( '60000' );

    // if we've been redirected to the login page...login.
    if( strstr( $this->getLocation(), '/siteworx/index' ) !== false ) {
      $this->waitForElement( 'email' );
      $this->loginToSWAsUser( $user, $pass, $domain );
    }
    /* This is because SiteWorx always logs a user into the main page
     * instead of where the (then unauthenticated) user was trying to go.
     */
    if( $page != $this->getLocation() ) {
      $this->open( $page );
    }
    $this->waitForElement( 'iw-ft' );
    $this->assertContains( $page, $this->getLocation() );
    $this->assertPageRenderedWithoutErrors();
  }

  /**
   * Gets the siteworx id of the account that was created.
   *
   * @param string $domain
   *
   * @return integer
   */
  public function getSWAccountId( $domain = self::SW_DOMAIN ) {
    $SiteWorx = new SiteWorx( $domain );
    return intval( $SiteWorx->getId() );
  }

  /**
   * Gets the user id of an email in nodeworx.
   *
   * @param string $email
   * @return integer
   */
  public function getNWUserId( $email ) {
    if( IWorxValidate::falsey( $email ) === true ) {
      return false;
    }

    $NodeWorx = new NodeWorx( $email );

    $user = $NodeWorx->getWorkingUser();

    if( IWorxValidate::falsey( $user ) === true ) {
      return false;
    }

    $user_ID = $NodeWorx->getWorkingUser()->getId();

    $this->assertTrue( IWorxValidate::integer( $user_ID, 2 ),
                       'Unable to deterimine NodeWorx ID' );

    return intval( $user_ID );
  }

  /**
   * Non-selenium wrapper to get the master nodeworx account.
   *
   * @return NodeWorx
   */
  protected function _fwGetMasterNodeWorx() {
    return IWorx_Framework_TestCase::getMasterNodeWorx();
  }

  /**
   * Non-selenium wrapper to add a siteworx account.
   *
   * @return boolean
   */
  protected function _fwAddSWAccount() {
    return IWorx_Framework_TestCase::addSWAccount();
  }

  /**
   * Non-selenium wrapper to delete a siteworx account.
   */
  protected function _fwDeleteSWAccount() {
    IWorx_Framework_TestCase::deleteSWAccountStatic();
  }

  /**
   * Non-selenium wrapper to delete a siteworx account if needed.
   */
  protected function _fwDeleteSWAccountIfNecessary() {
    IWorx_Framework_TestCase::deleteSWAccountIfNecessaryStatic();
  }

  /**
   * Non-selenium wrapper to add a siteworx account.
   *
   * @return boolean
   */
  protected function _fwAddSWAccountIfNecessary() {
    return IWorx_Framework_TestCase::addSWAccountIfNecessary();
  }

  /**
   * Creates a default SiteWorx account for tests.
   */
  public function addSWAccount() {
    $this->loginToNWAndGoTo( '/nodeworx/siteworx' );

    $this->assertEquals( -1, $this->getSWAccountId(),
                         'Default SW Account already exists, cannot add!' );

    $link = 'link=' . IWorxTranslate::tr( '##LG_ADDSITEWORXACCOUNT##' );
    $this->clickAndWaitForPopupForm( $link );

    $this->fillInAddSWAccountForm();

    $this->click( 'submitbutton' );

    $this->waitForMessageBanner();
    $this->assertTranslatedTextPresent( '##LG_SITEWORX_ADDSUCCESS##' );

    $loc = "//a[contains(@href, '" . self::SW_DOMAIN .
      "')][contains(@href, 'action=edit')]";

    $this->waitForElement( $loc );
  }

  /**
   * Checks for default SiteWorx account and creates if necessary.
   */
  public function addSWAccountIfNecessary() {
    IWorx_Framework_TestCase::addSWAccountIfNecessary();
  }

  /**
   * Adds a secondary user.
   *
   * @param string $email
   */
  public function addSWSecondaryUserIfNecessary( $email=self::SW_SECONDARY_EMAIL ) {
    if( $this->isSWSecondaryUserAlreadyCreated( $email ) === true ) {
      return;
    }

    $temp = SiteWorx::getPermsList();

    $perms = [];

    foreach( $temp as $perm ) {
      $perms[] = $perm['short_name'];
    }

    $input = array( 'nickname' => self::SW_SECONDARY_NICK,
                    'email'    => $email,
                    'password' => self::SW_SECONDARY_PASS,
                    'confirm_password' => self::SW_SECONDARY_PASS,
                    'perms'    =>  $perms
                    );

    $SW = new SiteWorx( self::SW_DOMAIN );

    $Reply = IW::FC()->routeFromPHP( 'Ctrl_Siteworx_Users',
                                     'add',
                                     $input,
                                     $SW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Asserts that the reply was successful.
   *
   * @param Reply  $Reply
   * @param string $error_message
   */
  public function assertReplySuccessful( Reply $Reply, $error_message = null ) {
    if( is_null( $error_message ) ) {
      $error_message = $Reply->getErrorsAsString();
    }
    $this->assertTrue( $Reply->wasSuccessful(), 'Error message: ' . $error_message );
  }

  /**
   * Gets the SiteWorx user ID for the email.
   *
   * @param string $domain
   * @param string $email
   * @return integer
   */
  public function getSWUserId( $domain, $email ) {
    return IWorx_Framework_TestCase::getSWUserId( $domain, $email );
  }

  /**
   * Checks if the secondary user exists already.
   *
   * @param string $email
   * @return boolean
   */
  public function isSWSecondaryUserAlreadyCreated( $email =
                                                   self::SW_SECONDARY_EMAIL ) {
    $user_ID = $this->getSWUserId( self::SW_DOMAIN, $email );

    return ( $user_ID > 0 );
  }

  /**
   * Fills in the form to add a SW user.
   */
  public function fillInAddSecondarySWUserForm() {
    $this->typeInto( 'nickname',  self::SW_SECONDARY_NICK );
    $this->typeInto( 'email',    self::SW_SECONDARY_EMAIL );
    $this->typeInto( 'password', self::SW_SECONDARY_PASS );
    $this->typeInto( 'confirm_password', self::SW_SECONDARY_PASS );
    $this->click( 'link=' . IWorxTranslate::tr( '##LG_CHECKALL##' ) );
  }

  /**
   * Clicks the save button of a FormBuilder form.
   */
  public function clickSave() {
    $this->click( "//input[@value='" . IWorxTranslate::tr( '##LG_SAVE##' ) . "']" );
  }

  /**
   * Deletes the default SiteWorx account.
   */
  public function deleteSWAccount() {
    $this->loginToNWAndGoTo( '/nodeworx/siteworx' );

    // Click the checkbox next to the domain
    //$loc = "//a[contains(@href, '" . self::SW_DOMAIN .
    //  "')][contains(@href, 'action=del')]";

    $loc = "//input[contains(@value,'" . self::SW_DOMAIN . "')]" .
      "[contains(@type,'checkbox')]";

    $this->click( $loc );

    $this->select( 'iw_action_name', 'value=delete' );

    $loc = "//input[@id='nodeworx_siteworx-submit']";

    $this->clickAndWaitForPopupForm( $loc );
    $deleteConfirm = '##LG_CONFIRM_SITEWORX_ACCOUNT_DELETION##';
    $this->assertTextPresent( IWorxTranslate::tr( $deleteConfirm ) );
    $this->assertTextPresent( self::SW_DOMAIN );

    // Gotta click the confirm button
    $this->click( 'confirm_action[]' );

    $this->click( "//input[@value='" .
                  IWorxTranslate::tr( '##LG_DELETE##' ) . "']" );

    $this->waitForMessageBanner();
    $this->assertTranslatedTextPresent( '##LG_SITEWORX_DELETESUCCESS##' );

    $this->waitForElementToDisappear( $loc );
  }

  /**
   * Checks if default SiteWorx Account exists and deletes if it does.
   */
  public function deleteSWAccountIfNecessary() {
    if( IWorx_Framework_TestCase::getSWAccountExists() === true ) {
      $this->deleteSWAccount();
    }
  }

  /**
   * Fills in default values for creating a siteworx account in NodeWorx.
   */
  public function fillInAddSWAccountForm() {
    $this->assertContains( 'nodeworx/siteworx', $this->getLocation() );
    $this->typeInto( 'master_domain', self::SW_DOMAIN );
    $this->keys( "\t" );
    sleep( 4 ); // give the uniqname a change to populate.
    $this->typeInto( 'uniqname', self::SW_UNIQNAME );
    $this->select( 'billing_day', 'label=12' );
    $this->typeInto( 'nickname', self::SW_NICK );
    $this->typeInto( 'email', self::SW_EMAIL );
    $this->typeInto( 'password', self::SW_PASS );
    $this->typeInto( 'confirm_password', self::SW_PASS );
    $this->select( 'theme', 'value=interworx' );

    $options = Option::getOptionList();

    foreach( $options as $opt ) {
      if( $opt['displayGroup'] !== '1' ) {
        continue;
      }
      $radio_button_name = substr( $opt['name'], 3 ) . '_j_check_j_0';
      $this->click( $radio_button_name );
    }

    foreach( $options as $opt ) {
      $opt['name'] = substr( $opt['name'], 3 );
      if( $opt['displayGroup'] !== '2' ) {
        continue;
      }
      if( $opt['name'] != 'OPT_SUPHP' && $opt['name'] != 'OPT_SSI' ) {
        $this->select( $opt['name'],
                       'label=' . IWorxTranslate::tr( '##LG_YES##' ) );
      }
    }
  }

  /**
   * Verifies that the page did not whitescreen.
   *
   * @param string $body
   */
  public function assertIsNotAWhitescreen( $body ) {
    $this->assertNotEquals( '', $body, $this->makeFailMessage( 'whitescreened' ) );
  }

  /**
   * Searches through the string given (bodyText).
   *
   * Verifies that Smarty didn't hock a wobbly
   *
   * @param string $body
   */
  public function assertNotContainsSmartyBarf( $body ) {
    $msg = 'contains Smarty barf or maybe just lang entry with no translation';
    $this->assertNotContains( '##LG_',
                              $body,
                              $this->makeFailMessage( $msg ) );
  }

  /**
   * Searches through the string given (bodyText).
   *
   * Verifies that Smarty didn't hock a wobbly
   *
   * @param string $body
   */
  public function assertNotContainsUntranslatedString( $body ) {
    $msg = 'contains untranslated string';
    $this->assertNotContains( 'LG_',
                              $body,
                              $this->makeFailMessage( $msg ) );
  }

  /**
   * Verifies that the page contains no stacktraces.
   *
   * @param string $body
   */
  public function assertNotContainsStacktrace( $body ) {
    // Make sure there's no errors on the page
    $this->assertNotContains( 'InterWorx Error Report',
                              $body,
                              $this->makeFailMessage( 'contains a stacktrace' ) );
  }

  /**
   * Searches the string given to verify no PHP errors are present.
   *
   * @param string $body
   */
  public function assertNotContainsPHPErrors( $body ) {
    if( strstr( $body, 'Parse error' ) &&
        strstr( $body, 'on line' ) ) {
      $this->fail( $this->makeFailMessage( 'contains PHP Parse Errors' ) );
    }
  }

  /**
   * Shell function to call several other assertions.
   *
   * @param boolean $check_translation
   */
  public function assertPageRenderedWithoutErrors( $check_translation = false ) {
    $body = $this->getBodyText();

    $this->assertIsNotAWhitescreen( $body );
    $this->assertNotContainsStacktrace( $body );
    $this->assertNotContainsPHPErrors( $body );
    $this->assertNotContainsSmartyBarf( $body );
    if( $check_translation === true ) {
      $this->assertNotContainsUntranslatedString( $body );
    }
  }

  /**
   * Verifies that a given regex is present in the bodyText.
   *
   * @param string $regex
   * @param string $message
   */
  public function assertRegExpPresent( $regex, $message = '' ) {
    $body = $this->getBodyText();
    $this->assertRegExp( $regex, $body, $message );
  }

  /**
   * Verifies that a regex is NOT in the bodyText.
   *
   * @param string $regex
   * @param string $message
   */
  public function assertRegExpNotPresent( $regex, $message = '' ) {
    $body = $this->getBodyText();
    $this->assertNotRegExp( $regex, $body, $message );
  }

  /**
   * Asserts that a given language entry is present in translated form.
   *
   * @param string $lang_key
   * @param string $message
   */
  public function assertTranslatedTextPresent( $lang_key, $message = '' ) {
    $this->assertTextPresent( IWorxTranslate::tr( $lang_key ), $message );
  }

  /**
   * Checks if the translated value of a key is present.
   *
   * @param string $lang_key
   * @return boolean
   */
  public function isTranslatedTextPresent( $lang_key ) {
    return $this->isTextPresent( IWorxTranslate::tr( $lang_key ) );
  }

  /**
   * Asserts that a given language entry is not present in translated form.
   *
   * @param string $lang_key
   * @param string $message
   */
  public function assertTranslatedTextNotPresent( $lang_key, $message = '' ) {
    $this->assertTextNotPresent( IWorxTranslate::tr( $lang_key ), $message );
  }

  /**
   * Deletes the default SiteWorx package.
   */
  public function deleteSWPackage() {
    $this->loginToNWAndGoTo( '/nodeworx/packages' );

    $NW = NodeWorx::getDefaultNodeWorx();
    $temps = $NW->getPackageTemplates();
    $id = 0;
    foreach( $temps as $temp ) {
      if( $temp['name'] === self::SW_PACKAGE ) {
        $id = $temp['id'];
        break;
      }
    }

    $loc = 'css=a[href="/nodeworx/packages?action=delete&id=' . $id . '"]';
    $this->clickAndWaitForPopupForm( $loc );

    $deleteConfirm = '##LG_CONFIRMDELETIONOF##';
    $this->assertTextPresent( IWorxTranslate::tr( $deleteConfirm ) );
    $this->assertTextPresent( self::SW_PACKAGE );
    $this->click( "//input[@value='" .
                  IWorxTranslate::tr( '##LG_DELETE##' ) . "']" );

    $this->waitForMessageBanner();
    $this->assertTranslatedTextPresent( '##LG_PACKAGE_DELETESUCCESS##' );
    $this->waitForElementToDisappear( $loc );
  }

  /**
   * Checks if default SiteWorx package exists and deletes if it does.
   */
  public function deleteSWPackageIfNecessary() {
    if( $this->packageTemplateExists() ) {
      $this->deleteSWPackage();
    }
  }

  /**
   * Creates a default SiteWorx package for tests.
   */
  public function addSWPackage() {
    $this->loginToNWAndGoTo( '/nodeworx/packages' );

    $this->assertFalse( $this->packageTemplateExists(),
                        'Default SW Package already exists, cannot add!' );

    $this->clickAndWaitForPopupForm( 'link=' . IWorxTranslate::tr( '##LG_ADD##' ) );

    $this->fillInAddSWPackageForm( $json = true );

    $this->click( "//input[@value='Add']" );

    $this->waitForMessageBanner();
    $this->assertTranslatedTextPresent( '##LG_PACKAGE_ADDSUCCESS##' );
  }

  /**
   * Checks for default SiteWorx package and creates if necessary.
   */
  public function addSWPackageIfNecessary() {
    $exists = $this->packageTemplateExists();
    if( !$exists ) {
      $this->addSWPackage();
    }
  }

  /**
   * Checks to see if a package exists.
   *
   * @param string $package_name
   * @return boolean
   */
  public function packageTemplateExists( $package_name =  self::SW_PACKAGE ) {
    $NodeWorx = NodeWorx::getDefaultNodeWorx();
    return $NodeWorx->packageTemplateExists( $package_name );
  }

  /**
   * Fills in default values for creating a siteworx package in NodeWorx.
   *
   * @param boolean $json
   */
  public function fillInAddSWPackageForm( $json = false ) {
    $this->assertContains( 'nodeworx/packages', $this->getLocation() );

    $this->typeInto( 'package_name', self::SW_PACKAGE );

    $options = Option::getOptionList();

    foreach( $options as $opt ) {
      $opt['name'] = substr( $opt['name'], 3 );
      if( $opt['displayGroup'] !== '1' ) {
        continue;
      }
      if( $json ) {
        $radio_button_label = $opt['name'] . '_j_check_j_0';
      } else {
        $radio_button_label = $opt['name'] . '_check_0';
      }
      $this->click( $radio_button_label );
    }

    foreach( $options as $opt ) {
      $opt['name'] = substr( $opt['name'], 3 );
      if( $opt['displayGroup'] !== '2' ) {
        continue;
      }
      $this->select( $opt['name'], 'label=' . IWorxTranslate::tr( '##LG_YES##' ) );
    }
  }

  /**
   * Waits for an element and ensures it has the correct text.
   *
   * @param string  $locator
   * @param string  $text
   * @param integer $timeout
   */
  public function waitForTextInElement( $locator, $text, $timeout = self::TIMEOUT ) {
    $this->_waitWithTimeout( 'isElementPresent', $locator, $timeout );
    $this->assertElementContainsText( $locator, $text );
  }

  /**
   * Waits for an element to appear before continuing.
   *
   * @param string  $locator
   * @param integer $timeout seconds
   */
  public function waitForElement( $locator, $timeout = self::TIMEOUT ) {
    $this->_waitWithTimeout( 'isElementPresent', $locator, $timeout, $sleep = 1 );
  }

  /**
   * Waits for an element to disappear from the dom before continuing.
   *
   * @param string  $locator
   * @param integer $timeout
   */
  public function waitForElementToDisappear( $locator, $timeout = self::TIMEOUT ) {
    $this->_waitWithTimeout( 'isElementPresent',
                             $locator,
                             $timeout,
                             $sleep = 1,
                             true );
  }

  /**
   * Click $locator and wait for a popup iw_form to show up.
   *
   * @param string $locator
   * @param string $form_name
   */
  public function clickAndWaitForPopupForm( $locator, $form_name = null ) {
    $this->click( $locator );
    if( $form_name === null ) {
      $this->waitForElement( 'css=#iw-modal-popup .iw-form-shell' );
    } else {
      $this->waitForElement( 'css=#iw-modal-popup .iw-form-shell .iw-form[name="' .
                             $form_name . '"]' );
    }
  }

  /**
   * Waits for the message banner.
   *
   * @param integer $timeout How long before the function times out (in seconds)
   */
  public function waitForMessageBanner( $timeout = self::TIMEOUT ) {
    $this->_waitWithTimeout( 'isElementVisable', 'iw-mb', $timeout );
  }

  /**
   * Returns true if the element is visable, and false otherwise.
   *
   * Also if the visibility is set to 'inherit' (as it would is some browsers)
   * then we'll continue look at each parent until we can figure out the
   * visibility of the element.
   *
   * @param string $id
   * @return boolean
   */
  public function isElementVisable( $id ) {
    $element = "window.document.getElementById( '{$id}' )";
    $style   = '.style.visibility;';

    $js         = $element . $style;
    $visibility = $this->getEval( $js );

    $count     = 0;
    $max_loops = 100;  // to prevent a shitty DOM Tree from putting us in a
                       // never ending loop.
    while( $visibility == 'inherit' ) {
      $element    = $element . '.parentNode';
      $js         = $element . $style;
      $visibility = $this->getEval( $js );

      $count++;
      if( $count >= $max_loops ) {
        break;
      }
    }
    return $visibility == 'visible';
  }

  /**
   * Gets the Message Banner inner-HTML.
   *
   * @param string  $id
   * @param integer $timeout
   * @return string
   */
  public function getMessageBannerHTML( $id = 'iw-mb-content',
                                        $timeout = self::TIMEOUT ) {
    $this->_waitWithTimeout( 'isElementVisable', 'iw-mb', $timeout );

    return $this->getIWMessageHTML( $id );
  }

  /**
   * Gets the innerHTML of any iw_message, by id.
   *
   * @param string $id
   * @return string
   */
  public function getIWMessageHTML( $id ) {
    $js_script  = "window.document.getElementById( '$id' ).";
    $js_script .= "getElementsByTagName( 'div' )[2].innerHTML";

    return $this->getEval( $js_script );
  }

  /**
   * Gets an element attribute.
   *
   * @param string $id
   * @param string $attribute
   * @return string
   */
  public function getElementAttributeById( $id, $attribute ) {
    $js = "window.document.getElementById( '{$id}' )" .
      ".getAttribute( '{$attribute}' );";

    return $this->getEval( $js );
  }

  /**
   * Closes the iw-mg banner.
   */
  public function closeMessageBanner() {
    $this->click( '//*[@id="iw-mb"]/a' );
  }

  /**
   * Click wrapper, fixes unexplainable random failures.  Maybe. :(.
   *
   * @param string $locator
   */
  public function click( $locator ) {
    $this->getEl( $locator )->click();
  }

  /**
   * Waits until the callback function returns a success.
   *
   * @param string       $callback
   * @param string|array $args
   * @param float        $timeout
   * @param float        $sleep_time
   * @param boolean      $inverse
   */
  private function _waitWithTimeout( $callback,
                                     $args,
                                     $timeout,
                                     $sleep_time=0.10,
                                     $inverse = false ) {

    $start_time = microtime( true );
    $sleep_time = $sleep_time * 1000000;

    $success = $this->{$callback}( $args );
    while( $success !== true ) {
      usleep( $sleep_time );
      $elapsed_time = microtime( true ) - $start_time;

      if( $elapsed_time >= $timeout ) {
        IWorxLog::log( $this->getHtmlSource() );
        $this->fail( "$callback timed out" );
        return;
      }
      $success = $this->{$callback}( $args );
      if( $inverse ) {
        $success = !$success;
      }
    }
    return;
  }

  /*******************************************************************************/
  // NOTE: BELOW THIS POINT ARE SELENIUM1/RC COMPATIBILITY FUNCTIONS, WRITTEN WHEN
  // MIGRATING TO SELENIUM2/WEBDRIVER.
  /*******************************************************************************/

  /**
   * Type into element identified by $locator.
   *
   * This replaces the selenium1/RC type() method.  I had to rename it because
   * type() exists in selenium2/webdriver but doesn't do what we want.
   *
   * @param string $locator
   * @param string $value
   */
  public function typeInto( $locator, $value ) {
    $this->getEl( $locator )->clear();
    if( $value !== '' ) {
      $this->getEl( $locator )->value( $value );
    }
  }

  /**
   * Click and wait for page to load.
   *
   * As of selenium2/webdriver, the "wait" component is no longer needed, so
   * this is just a wrapper to click() now.
   *
   * @param string $locator
   */
  public function clickAndWait( $locator ) {
    $this->click( $locator );
  }

  /**
   * Open $url in browser.
   *
   * @param string $url
   */
  public function open( $url ) {
    $this->url( $url );
  }

  /**
   * Wait for page to load.
   *
   * As of selenium2/webdriver, this is no longer needed.  Do nothing!
   */
  public function waitForPageToLoad() {
    //noop
  }

  /**
   * Get the current page location/uri.
   *
   * @return string
   */
  public function getLocation() {
    return $this->url();
  }

  /**
   * Returns true if element identified by $locator is "selected" (like a checkbox).
   *
   * @param string $locator
   */
  public function isChecked( $locator ) {
    return $this->getEl( $locator )->selected();
  }

  /**
   * Returns true if element identified by $locator is present on the page.
   *
   * @param string $locator
   * @return boolean
   */
  public function isElementPresent( $locator ) {
    return count( $this->findMultiple( $locator ) ) != 0;
  }

  /**
   * Returns an array of elements based on $locator.
   *
   * Understands xpath, link text, css selectors, name, id.
   *
   * @param string $locator
   * @return array
   */
  public function findMultiple( $locator ) {
    $elements = [];
    if( strpos( $locator, '//' ) === 0 ) {
      $elements = $this->elements( $this->using( 'xpath' )
                                   ->value( $locator ) );
    } else if( substr( $locator, 0, 6 ) === 'xpath=' ) {
      $elements = $this->elements( $this->using( 'xpath' )
                                   ->value( substr( $locator, 6 ) ) );
    } else if( substr( $locator, 0, 5 ) === 'link=' ) {
      $elements = $this->elements( $this->using( 'link text' )
                                   ->value( substr( $locator, 5 ) ) );
    } else if( substr( $locator, 0, 4 ) === 'css=' ) {
      $elements = $this->elements( $this->using( 'css selector' )
                                   ->value( substr( $locator, 4 ) ) );
    } else {
      $elements = $this->elements( $this->using( 'name' )
                                   ->value( $locator ) );
      if( empty( $elements ) ) {
        $elements = $this->elements( $this->using( 'id' )
                                     ->value( $locator ) );
      }
    }
    return $elements;
  }

  /**
   * Make select box identified by $select choose option identified by $option.
   *
   * @param string $select Select locator.
   * @param string $option Option identifier.
   */
  public function select( $select, $option ) {
    $El = $this->getEl( $select );
    if( substr( $option, 0, 6 ) === 'value=' ) {
      parent::select( $El )->selectOptionByValue( substr( $option, 6 ) );
    } else if( substr( $option, 0, 6 ) === 'label=' ) {
      parent::select( $El )->selectOptionByLabel( substr( $option, 6 ) );
    }
  }

  /**
   * Assert element value matches $text.
   *
   * @param string $locator
   * @param string $text
   * @param string $message
   */
  public function assertElementValueEquals( $locator, $text, $message = '' ) {
    $this->assertEquals( $text, $this->getValue( $locator ), $message );
  }

  /**
   * Assert that the select box identified by $locator has $text selected.
   *
   * @param string $locator
   * @param string $text
   * @param string $message
   */
  public function assertSelected( $locator, $text, $message = '' ) {
    $this->assertEquals( $text,
                         parent::select( $this->getEl( $locator ) )
                         ->selectedLabel(),
                         $message );
  }

  /**
   * Assert element identified by $locator is present on the page.
   *
   * @param string $locator
   */
  public function assertElementPresent( $locator ) {
    $this->assertTrue( $this->isElementPresent( $locator ),
                       'Element not present: ' . $locator );
  }

  /**
   * Aseert element identified by $locator is not present on the page.
   *
   * @param string $locator
   */
  public function assertElementNotPresent( $locator ) {
    $this->assertFalse( $this->isElementPresent( $locator ),
                        'Element present: ' . $locator );
  }

  /**
   * Assert the element identified by $locator's value contain's $text.
   *
   * @param string $locator
   * @param string $text
   */
  public function assertElementContainsText( $locator, $text ) {
    $this->assertContains( $text, $this->getEl( $locator )->text() );
  }

  /**
   * Assert the element identified by $locator's value does not contain $text.
   *
   * @param string $locator
   * @param string $text
   */
  public function assertElementNotContainsText( $locator, $text ) {
    $this->assertNotContains( $text, $this->getEl( $locator )->text() );
  }

  /**
   * Get page title.
   *
   * @return string
   */
  public function getTitle() {
    return $this->title();
  }

  /**
   * Execute arbitrary javascript in the brower.
   *
   * If $js is a single line, the result will be returned.
   *
   * @param string $js
   * @return string
   */
  public function getEval( $js ) {
    $prefix = '';
    if( strpos( $js, "\n" ) === false ) {
      $prefix = 'return ';
    }
    return $this->execute( [ 'script' => $prefix . $js, 'args' => [] ] );
  }

  /**
   * Assert $text is present on the page.
   *
   * @param string $text
   */
  public function assertTextPresent( $text ) {
    $this->assertTrue( strpos( $this->getBodyText(), $text ) !== false,
                       'assertTextPresent failed: Could not find ' . $text );
  }

  /**
   * Assert $text is not present on the page.
   *
   * @param string $text
   */
  public function assertTextNotPresent( $text ) {
    $this->assertTrue( strpos( $this->getBodyText(), $text ) === false,
                       'assertTextNotPresent failed: Found ' . $text );
  }

  /**
   * Return true if $text is present on the page.
   *
   * @param string $text
   * @return boolean
   */
  public function isTextPresent( $text ) {
    return strpos( strtoupper( $this->getBodyText() ),
                   strtoupper( $text ) ) !== false;
  }

  /**
   * Get the value of the element identified by $locator.
   *
   * @param string $locator
   */
  public function getValue( $locator ) {
    return $this->getEl( $locator )->value();
  }

  /**
   * Get an element based on $locator.
   *
   * $locator can be xpath, link text, css selector, id, or name.
   *
   * @param string $locator
   * @return object
   */
  public function getEl( $locator ) {
    $El = null;
    if( strpos( $locator, '//' ) === 0 ) {
      $El = $this->byXPath( $locator );
    } else if( substr( $locator, 0, 6 ) === 'xpath=' ) {
      $El = $this->byXPath( substr( $locator, 6 ) );
    } else if( substr( $locator, 0, 5 ) === 'link=' ) {
      $El = $this->byLinkText( substr( $locator, 5 ) );
    } else if( substr( $locator, 0, 4 ) === 'css=' ) {
      $El = $this->byCssSelector( substr( $locator, 4 ) );
    } else {
      try {
        $El = $this->byId( $locator );
      } catch( PHPUnit_Extensions_Selenium2TestCase_WebDriverException $e ) {
        $El = $this->byName( $locator );
      }
    }
    return $El;
  }

  /**
   * Gets an array of all ids of all links on the page.
   *
   * Compatible with selenium1/RC version.
   *
   * @return array
   */
  public function getAllLinks() {
    $a_tags = $this->elements( $this->using( 'tag name' )->value( 'a' ) );
    $ids = [];
    foreach( $a_tags as $tag ) {
      $ids[] = $tag->attribute( 'id' );
    }
    return $ids;
  }

  /**
   * Get text within the body tag (not html tags).
   *
   * @return string
   */
  public function getBodyText() {
    return $this->byTag( 'body' )->text();
  }

  /**
   * Get the raw html source of the page.
   */
  public function getHtmlSource() {
    return $this->source();
  }
}
