<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Jonathan Daniel <jdaniel@interworx.com>
 * @date    Mar 22, 2010
 */

/**
 * Class that stores all the config data.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class TestConfig {

  static private $_Instance = null;

  /**
   * Get instance.
   *
   * @return TestConfig
   */
  static public function getInstance() {
    if( is_null( self::$_Instance ) ) {
      self::$_Instance = new self();
    }
    return self::$_Instance;
  }

  /**
   * Constructor.
   */
  private function __construct() {
    $path = Ini::get( Ini::IWORX_DIR, 'base' ) . '/TESTS/_include/testconfig.yaml';
    if( !IWorxFileSys::fileExists( $path ) ) {
      $path = Ini::get( Ini::IWORX_DIR, 'base' ) . '/TESTS/_include/testconfig.yaml-dist';
      if( !IWorxFileSys::fileExists( $path ) ) {
        $msg = 'testconfig.yaml is missing, you need to create it. ' .
          'Check testconfig.yaml-dist';
        throw new IWorx_Exception( $msg );
      }
    }
    $this->_config = Spyc::YAMLLoad( $path );
  }

  /**
   * Gets the config array.
   *
   * @return array
   */
  public function getConfig() {
    return $this->_config;
  }

  /**
   * Gets a section of the config file.
   *
   * @return array|string
   */
  static public function get() {
    $args = func_get_args();
    if( count( $args ) == 0 ) {
      return null;
    }

    $tmp = self::getInstance()->getConfig();
    foreach( $args as $arg ) {
      if( !array_key_exists( $arg, $tmp ) ) {
        $msg = 'Unknown config parameter: ' . implode( ',', $args );
        throw new IWorx_Exception( $msg );
      }
      $tmp = $tmp[$arg];
    }

    return $tmp;
  }
}
