<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Paul Oehler <poehler@interworx.com>
 * @id $Id$
 */

/**
 * Tests all of packages.php page in NodeWorx.
 *
 * @group NodeWorx
 * @group NW_SW_Package
 *
 * @package    InterWorx
 * @subpackage Tests
 * @group      selenium
 */
class PageNWPackageTest extends IWorxSeleniumTestCase {

  /**
   * Tests that the required data is required.
   *
   * @group NW_User_Secondary
   */
  public function testAddingSWPackageRequiredData() {
    $this->loginToNWAndGoTo( '/nodeworx/packages?action=add&do=show' );

    // Blank user name
    $this->fillInAddSWPackageForm();
    $this->typeInto( 'package_name', '' );

    $this->click( "//input[@value='" . IWorxTranslate::tr( '##LG_ADD##' ) . "']" );

    $package_name_id = 'package_name_status_span';
    $class = $this->getElementAttributeById( $package_name_id, 'class' );
    $this->assertEquals( 'iw-i-alert iw-form-img iw-i', $class );

    foreach( Option::getOptionList() as $opt ) {
      if( $opt['displayGroup'] !== '1' ) {
        continue;
      }
      $element_id = substr( $opt['name'], 3 ) . '_status_span';
      $class      = $this->getElementAttributeById( $element_id, 'class' );

      $this->assertEquals( 'iw-i-positive iw-form-img iw-i', $class );
    }

    // Bad user name
    $this->typeInto( 'package_name', 'Bad#$!@Chars!' );
    $this->click( "//input[@value='" . IWorxTranslate::tr( '##LG_ADD##' ) . "']" );

    $this->waitForMessageBanner();
    $this->assertTranslatedTextPresent( '##LG_INPUT_FORM_DID_NOT_VALIDATE##' );
  }

  /**
   * Tests that adding a SiteWorx package works.
   */
  public function testAddingSWPackage() {
    $this->deleteSWPackageIfNecessary();
    $this->addSWPackage();
  }

  /**
   * Verifies the package information of the added package.
   */
  public function testVerifySWPackageInformation() {
    $this->addSWPackageIfNecessary();
    $this->loginToNWAndGoTo( '/nodeworx/packages' );

    $NW = NodeWorx::getDefaultNodeWorx();
    $temps = $NW->getPackageTemplates();
    $id = 0;
    foreach( $temps as $temp ) {
      if( $temp['name'] === self::SW_PACKAGE ) {
        $id = $temp['id'];
        break;
      }
    }

    $loc = 'css=a[href="/nodeworx/packages?action=edit&id=' . $id . '"]';
    $this->clickAndWaitForPopupForm( $loc );

    $this->assertElementValueEquals( 'package_name', self::SW_PACKAGE );

    $options = Option::getOptionList();

    foreach( $options as $opt ) {
      $opt['name'] = substr( $opt['name'], 3 );
      if( $opt['displayGroup'] !== '1' ) {
        continue;
      }
      $radio_button_name = $opt['name'] . '_j_check_j_0';
      $this->assertTrue( $this->isChecked( $radio_button_name ) );
    }

    foreach( $options as $opt ) {
      $opt['name'] = substr( $opt['name'], 3 );
      if( $opt['displayGroup'] !== '2' ) {
        continue;
      }
      $this->assertSelected( $opt['name'], IWorxTranslate::tr( '##LG_YES##' ) );
    }
  }

  /**
   * Tests deleting the SW package.
   */
  public function testDeletingSWPackage() {
    $this->addSWPackageIfNecessary();
    $this->deleteSWPackage();
    $this->assertFalse( $this->packageTemplateExists(),
                        'SiteWorx package template was not deleted correctly' );
  }
}
