<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright ( c ) 2000-2008 InterWorx L.L.C., All Rights Reserved.     |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | ( INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE   |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @id $Id$
 * @package    InterWorx
 * @subpackage Tests
 * @author     Tim College <tcollege@interworx.com>
 * @date       Sep 2, 2008
 */

/**
 * Tests resellers.php in NodeWorx.
 *
 * @group NodeWorx
 *
 * @package    InterWorx
 * @subpackage Tests
 * @group      selenium
 */
class PageNWResellersTest extends IWorxSeleniumTestCase {

  const NW_RESELLER_NICK  = 'Reseller';
  const NW_RESELLER_EMAIL = 'reseller@interworx.co';
  const NW_RESELLER_PASS  = 'reseller_password';

  /**
   * TestAddingResellerNWUser.
   */
  public function testAddingResellerNWUser() {
    $NodeWorx = new NodeWorx( self::NW_RESELLER_EMAIL );
    if( $NodeWorx->isReseller() ) {
      $this->testDeletingResellerNWUser();
    }

    $this->loginToNWAndGoTo( '/nodeworx/reseller' );
    $this->clickAndWaitForPopupForm( 'link=' .
                                     IWorxTranslate::tr( '##LG_ADDRESELLER##' ) );

    $this->typeInto( 'nickname', self::NW_RESELLER_NICK );
    $this->typeInto( 'email', self::NW_RESELLER_EMAIL );
    $this->typeInto( 'password', self::NW_RESELLER_PASS );
    $this->typeInto( 'confirm_password', self::NW_RESELLER_PASS );
    $this->typeInto( 'RSL_OPT_STORAGE', '15' );
    $this->typeInto( 'RSL_OPT_BANDWIDTH', '15' );
    $this->typeInto( 'RSL_OPT_EMAIL_ALIASES', '15' );
    $this->typeInto( 'RSL_OPT_EMAIL_AUTORESPONDERS', '15' );
    $this->typeInto( 'RSL_OPT_EMAIL_BOXES', '15' );
    $this->typeInto( 'RSL_OPT_EMAIL_GROUPS', '15' );
    $this->typeInto( 'RSL_OPT_FTP_ACCOUNTS', '15' );
    $this->typeInto( 'RSL_OPT_MYSQL_DBS', '15' );
    $this->typeInto( 'RSL_OPT_MYSQL_DB_USERS', '15' );
    $this->typeInto( 'RSL_OPT_POINTER_DOMAINS', '15' );
    $this->typeInto( 'RSL_OPT_SITEWORX_ACCOUNTS', '15' );
    $this->typeInto( 'RSL_OPT_SLAVE_DOMAINS', '15' );
    $this->typeInto( 'RSL_OPT_SUBDOMAINS', '15' );
    $this->select( 'RSL_OPT_OVERSELL_STORAGE', 'label=' .
                   IWorxTranslate::tr( '##LG_YES##' ) );
    $this->select( 'RSL_OPT_OVERSELL_BANDWIDTH', 'label=' .
                   IWorxTranslate::tr( '##LG_YES##' ) );

    $this->click( 'ipv4_j_0' );

    $default_server = Ini::get( Ini::MYSQL, 'default_server' );
    $DatabaseServer = new DatabaseServer( $default_server );
    if( !$DatabaseServer->exists() ) {
      $this->click( 'database_servers_j_0' );
    }

    $this->clickSave();
    $this->waitForMessageBanner();
    $this->assertTranslatedTextPresent( '##LG_RESELLERADDED##' );
  }

  /**
   * Logs in as the newly created user, just to be sure.
   *
   * Also does initial reseller setup, to ensure it works.
   */
  public function testResellerNWUserLoginWorks() {
    $this->loginToNWAndGoTo( '/nodeworx/dns?setup=true',
                             self::NW_RESELLER_EMAIL,
                             self::NW_RESELLER_PASS );

    $this->typeInto( 'ns1', 'ns1.interworx.com' );
    $this->typeInto( 'ns2', 'ns2.interworx.com' );
    $this->click( 'submitbutton' );

    $this->waitForMessageBanner();
    $this->assertTranslatedTextPresent( '##LG_SETTINGSUPDATEDSUCCESSFULLY##' );
    $this->waitForPageToLoad();
    $this->click( 'iw-menu-home' );
    $this->waitForPageToLoad();
    $this->assertTextPresent( self::NW_RESELLER_NICK );
  }

  /**
   * Tests deleting the reseller account.
   */
  public function testDeletingResellerNWUser() {
    $NodeWorx = new NodeWorx( self::NW_RESELLER_EMAIL );
    if( $NodeWorx->userExists( self::NW_RESELLER_EMAIL ) !== true ) {
      $this->assertTrue( false, 'Reseller doesn\'t exist! Something bad happened.' );
    }

    $url = '/nodeworx/reseller?action=delete&reseller_id=' . $NodeWorx->getId();
    $this->loginToNWAndGoTo( $url );

    $this->assertTextPresent( 'Please confirm the following action' );
    $this->assertTextPresent( 'Reseller (reseller@interworx.co)' );

    // Gotta click the confirm button
    //$this->click( 'confirm_action_0' );

    $this->click( "//input[@value='" .
                  IWorxTranslate::tr( '##LG_DELETE##' ) . "']" );

    $this->waitForMessageBanner();
    $this->assertTranslatedTextPresent( '##LG_RESELLERDELETED##',
                                        'Reseller delete sucess message missing' );
  }

}
