<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @id $Id$
 * @package    InterWorx
 * @subpackage Tests
 * @author     Tim College <tcollege@interworx.com>
 */

/**
 * Tests all of siteworx.php page in NodeWorx.
 *
 * @group NodeWorx
 * @group NW_SW_Account
 *
 * @package    InterWorx
 * @subpackage Tests
 * @group      selenium
 */
class PageNWSiteworxTest extends IWorxSeleniumTestCase {

  /**
   * Url to the phpunit coverage start script.
   *
   * @var string
   */
  //  protected $coverageScriptUrl = 'http://dev.interworx.com:2080/nodeworx/phpunit_coverage.php';

  /**
   * Tests adding the siteworx account.
   */
  public function testAddingSWAccount() {
    $this->deleteSWAccountIfNecessary();
    $this->addSWAccount();
    $this->assertGreaterThan( 0, $this->getSWAccountId( self::SW_DOMAIN ),
                              'SiteWorx account was not added correctly' );
  }

  /**
   * Test to verify added siteworx account information.
   */
  public function testVerifySWAccountInformation() {
    $this->addSWAccountIfNecessary();
    $this->loginToNWAndGoTo( '/nodeworx/siteworx?action=edit&domain=' .
                             self::SW_DOMAIN .
                             '&reseller=1' );

    $this->assertTextPresent( self::SW_DOMAIN );
    $this->assertElementValueEquals( 'nickname', self::SW_NICK );
    $this->assertElementValueEquals( 'email', self::SW_EMAIL );

    $options = Option::getOptionList();

    foreach( $options as $opt ) {
      $opt['name'] = substr( $opt['name'], 3 );
      if( $opt['displayGroup'] !== '1' ) {
        continue;
      }
      $radio_button_name = $opt['name'] . '_check_0';
      $this->assertTrue( $this->isChecked( $radio_button_name ) );
    }

    foreach( $options as $opt ) {
      $opt['name'] = substr( $opt['name'], 3 );
      if( $opt['displayGroup'] !== '2' ) {
        continue;
      }
      $this->assertSelected( $opt['name'], IWorxTranslate::tr( '##LG_YES##' ) );
    }
  }

  /**
   * Tests deleting the siteworx account.
   */
  public function testDeletingSWAccount() {
    $this->addSWAccountIfNecessary();
    $this->deleteSWAccount();
    $this->assertEquals( -1, $this->getSWAccountId( self::SW_DOMAIN ),
                         'SiteWorx account was not deleted correctly' );
  }
}
