<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Tim College <tcollege@interworx.com>
 */

/**
 * Tests for the Updates page.
 *
 * @package    InterWorx
 * @subpackage Tests
 * @group      selenium
 */
class PageNWUpdatesTest extends IWorxSeleniumTestCase {
  private $_rpms = array( 'cos4x-i386'    => 'http://updates.interworx.com/_internal/legacy-files/RPMS/cos4x/legacy/i386/spamassassin-3.2.4-100.rhe4x.iworx.i386.rpm',
                          'cos4x-x86_64'  => 'http://updates.interworx.com/_internal/legacy-files/RPMS/cos4x/legacy/x86_64/spamassassin-3.2.4-100.rhe4x.iworx.x86_64.rpm',
                          'cos5x-i386'    => 'http://updates.interworx.com/_internal/legacy-files/RPMS/rhe5x/legacy/i386/spamassassin-3.2.4-100.rhe5x.iworx.i386.rpm',
                          'cos5x-x86_64'  => 'http://updates.interworx.com/_internal/legacy-files/RPMS/rhe5x/legacy/x86_64/spamassassin-3.2.4-100.rhe5x.iworx.x86_64.rpm',
                          'cos6x-i386'    => 'http://updates.interworx.com/_internal/legacy-files/RPMS/rhe6x/legacy/i386/spamassassin-3.3.1-2011.rhe6x.iworx.i386.rpm',
                          'cos6x-x86_64'  => 'http://updates.interworx.com/_internal/legacy-files/RPMS/rhe6x/legacy/x86_64/spamassassin-3.3.1-2011.rhe6x.iworx.x86_64.rpm',
                          );


  /**
   * Force installs an old package.
   */
  private function _forceInstallOldPackage() {
    $sysinfo = new IWorxSysInfo();

    $distro = $sysinfo->distro();

    $short_dist = false;

    $arch = trim( `uname -i` );
    if( strstr( $distro, 'CentOS release 6' ) ) {
      $short_dist = "cos6x-{$arch}";
    } else if( strstr( $distro, 'CentOS release 5' ) ) {
      $short_dist = "cos5x-{$arch}";
    } else if( strstr( $distro, 'CentOS release 4' ) ) {
      $short_dist = "cos4x-{$arch}";
    }

    $this->assertNotEquals( false, $short_dist, 'Unable to determine distro' );

    $cmd = 'rpm -Uvh --force ' . $this->_rpms[ $short_dist ];

    IWorxExec::exec( $cmd );
  }

  /**
   * Checks if updates are available.
   *
   * @return boolean
   */
  private function _isUpdatesAvailable() {
    return !( $this->isTranslatedTextPresent( '##LG_NOUPDATESPENDING##' ) );
  }

  /**
   * Checks if YUM is still running.
   *
   * @return boolean
   */
  private function _yumIsStillRunning() {
    $js     = 'window.yumLogHandler.isStillRunning';
    $result =  $this->getEval( $js );

    return $result === 'true';
  }

  /**
   * Sets up the old package, if necessary.
   */
  public function testUpdatesAvailable() {
    $this->loginToNWAndGoTo( 'nodeworx/updates' );

    if( $this->_isUpdatesAvailable() === false ) {
      $this->_forceInstallOldPackage();
      $NW = IWorx_Framework_TestCase::getMasterNodeWorx();

      $Reply = IW::FC()->routeFromPHP( 'Ctrl_Nodeworx_Updates',
                                       'refresh',
                                       array(),
                                       $NW );

      $this->assertReplySuccessful( $Reply );
    }
  }

  /**
   * Doublechecks that the force install worked.
   */
  public function testUpdatesAvailableFollowup() {
    $this->loginToNWAndGoTo( 'nodeworx/updates' );

    $this->assertTranslatedTextNotPresent( '##LG_NOUPDATESPENDING##',
                                           'Force install of old RPM failed' );
  }

  /**
   * Installs the updates.
   */
  public function testInstallingUpdates() {
    $this->loginToNWAndGoTo( 'nodeworx/updates' );
    $xpath = "xpath=(//div[@id='available_updates-shell']" .
      "//input[@type='checkbox'])[1]";

    if( $this->_isUpdatesAvailable() === true ) {
      $this->click( $xpath );
      $this->select( 'iw_action_name4', 'label=' .
                     IWorxTranslate::tr( '##LG_INSTALL_UPDATES##' ) );
      $this->click( 'available_updates-submit' );

      $this->waitForElement( 'iw-file-output_h', 5 );

      while( $this->_yumIsStillRunning() ) {
        // Sleep a bit to wait for the install to complete
        sleep( 5 );
      }
    }
  }

  /**
   * Checks that updates applied.
   */
  public function testInstallWorked() {
    $this->loginToNWAndGoTo( 'nodeworx/updates' );

    $this->assertFalse( $this->_isUpdatesAvailable() );
  }
}