<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Tim College <tcollege@interworx.com>
 * @date       Sep 2, 2008
 */

/**
 * Tests for the xhr.php page.
 *
 * @package    InterWorx
 * @subpackage Tests
 * @group      selenium
 */
class PageErrorTest extends IWorxSeleniumTestCase {

  /**
   * Url to the phpunit coverage start script.
   *
   * @var string
   */
  //  protected $coverageScriptUrl = 'http://dev.interworx.com:2080/html/phpunit_coverage.php';

  /**
   * Test.
   *
   * @group errorHandling
   */
  public function testError() {
    TestDebug::turnDebugOn();
    $this->open( '/test/error_tests.php?mode=error' );
    $this->assertTextPresent( 'InterWorx Error Report', 'Missing error Report' );
    $this->assertTextPresent( 'FILE: /usr/local/interworx/html/test/error_tests.php',
                              'FILE was not in error report' );
    $this->assertRegExpPresent( '/LINE: [0-9]+/', 'LINE number missing' );
    $this->assertTextPresent( 'I triggered an error!', 'Error Message Missing' );
    TestDebug::restoreDebug();
  }

  /**
   * Test.
   *
   * @group errorHandling
   */
  public function testErrorWithDebugOff() {
    TestDebug::turnDebugOff();
    $this->open( '/test/error_tests.php?mode=error' );
    $this->assertTextNotPresent( 'InterWorx Error Report',
                                 'Error report is in output' );
    $this->assertTextPresent( 'A system error has occurred. Please try your ' .
                              'request again in a few minutes.',
                              'Generic error notification missing' );
    TestDebug::restoreDebug();
  }

  /**
   * Test.
   *
   * @group errorHandling
   */
  public function testErrorFatal() {
    TestDebug::turnDebugOn();
    $this->open( '/test/error_tests.php?mode=fatal' );
    $this->assertTextPresent( 'InterWorx Error Report', 'Missing error Report' );
    $this->assertTextPresent( 'FILE: /usr/local/interworx/html/test/error_tests.php',
                              'FILE was not in error report' );
    $this->assertRegExpPresent( '/LINE: [0-9]+/', 'LINE number missing' );
    try {
      $this->assertTextPresent( 'callUnknownFunction()', 'Error Message Missing' );
    } catch( PHPUnit_Framework_ExpectationFailedException $e ) {
      $this->assertTextPresent( 'callunknownfunction()', 'Error Message Missing' );
    }
    TestDebug::restoreDebug();
  }

  /**
   * Test.
   *
   * @group errorHandling
   */
  public function testErrorFatalWithFatalErrorHandlingStopped() {
    TestDebug::turnDebugOn();

    $this->open( '/test/error_tests.php?mode=fatal-off' );
    $this->assertTextNotPresent( 'InterWorx Error Report',
                                 'Found InterWorx Error Report' );
    try {
      $this->assertTextPresent( 'callUnknownFunction()', 'Error Message Missing' );
    } catch( PHPUnit_Framework_ExpectationFailedException $e ) {
      $this->assertTextPresent( 'callunknownfunction()', 'Error Message Missing' );
    }
    TestDebug::restoreDebug();
  }

  /**
   * Test.
   *
   * @group errorHandling
   */
  public function testErrorFatalWithDebugOff() {
    TestDebug::turnDebugOff();
    $this->open( '/test/error_tests.php?mode=fatal' );
    $this->assertTextNotPresent( 'InterWorx Error Report',
                                 'Error report is in output' );
    $this->assertTextPresent( 'A system error has occurred. Please try your ' .
                              'request again in a few minutes.',
                              'Generic error notification missing' );
    TestDebug::restoreDebug();
  }

  /**
   * Test.
   *
   * @group errorHandling
   */
  public function testException() {
    TestDebug::turnDebugOn();
    $this->open( '/test/error_tests.php?mode=exception' );
    $this->assertTextPresent( 'InterWorx Error Report', 'Missing error Report' );
    $this->assertTextPresent( 'FILE: /usr/local/interworx/html/test/error_tests.php',
                              'FILE was not in error report' );
    $this->assertRegExpPresent( '/LINE: [0-9]+/', 'LINE number missing' );
    $this->assertTextPresent( 'BARF!', 'Error Message Missing' );
    TestDebug::restoreDebug();
  }

  /**
   * Test.
   *
   * @group errorHandling
   */
  public function testExceptionWithDebugOff() {
    TestDebug::turnDebugOff();
    $this->open( '/test/error_tests.php?mode=exception' );
    $this->assertTextNotPresent( 'InterWorx Error Report',
                                 'Error report is in output' );
    $this->assertTextPresent( 'A system error has occurred. Please try your ' .
                              'request again in a few minutes.',
                              'Generic error notification missing' );
    TestDebug::restoreDebug();
  }
}
