<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Tim College <tcollege@interworx.com>
 * @date       Sep 2, 2008
 */

/**
 * Tests for the xhr.php page.
 *
 * @package    InterWorx
 * @subpackage Tests
 * @group      selenium
 */
class PageXhrTest extends IWorxSeleniumTestCase {

  /**
   * Url to the phpunit coverage start script.
   *
   * @var string
   */
  //  protected $coverageScriptUrl = 'http://dev..interworx.com:2080/nodeworx/phpunit_coverage.php';

  /**
   * Test.
   *
   * @group errorHandling
   */
  public function testXhrError() {
    TestDebug::turnDebugOn();
    $this->loginToNWAndGoTo( '/nodeworx/themes' );
    $this->open( '/xhr.php?i={"r":"Error"}' );
    $this->assertTextPresent( '"c":700' );
    TestDebug::restoreDebug();
  }

  /**
   * Test.
   *
   * @group errorHandling
   */
  public function testXhrErrorWithDebugOff() {
    TestDebug::turnDebugOff();
    $this->loginToNWAndGoTo( '/nodeworx/themes' );
    $this->open( '/xhr.php?i={"r":"Error"}' );
    $this->assertTextPresent( '"c":500' );
    TestDebug::restoreDebug();
  }

  /**
   * Test.
   *
   * @group errorHandling
   */
  public function testXhrErrorFatal() {
    TestDebug::turnDebugOn();
    $this->loginToNWAndGoTo( '/nodeworx/themes' );
    $this->open( '/xhr.php?i={"r":"ErrorFatal"}' );
    $this->assertTextPresent( '"c":700,"t":"Call to undefined function' );
    TestDebug::restoreDebug();
  }

  /**
   * Test.
   *
   * @group errorHandling
   */
  public function testXhrErrorFatalWithDebugOff() {
    TestDebug::turnDebugOff();
    $this->loginToNWAndGoTo( '/nodeworx/themes' );
    $this->open( '/xhr.php?i={"r":"ErrorFatal"}' );
    $this->assertTextPresent( '"c":500' );
    TestDebug::restoreDebug();
  }

  /**
   * Test.
   *
   * @group errorHandling
   */
  public function testXhrException() {
    TestDebug::turnDebugOn();
    $this->loginToNWAndGoTo( '/nodeworx/themes' );
    $this->open( '/xhr.php?i={"r":"ExceptionThrower"}' );
    $this->assertTextPresent( '"c":700,"t":"Uncaught' );
    TestDebug::restoreDebug();
  }

  /**
   * Test.
   *
   * @group errorHandling
   */
  public function testXhrExceptionWithDebugOff() {
    TestDebug::turnDebugOff();
    $this->loginToNWAndGoTo( '/nodeworx/themes' );
    $this->open( '/xhr.php?i={"r":"ExceptionThrower"}' );
    $this->assertTextPresent( '"c":500' );
    TestDebug::restoreDebug();
  }
}
