<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Jonathan Daniel <jdaniel@interworx.com>
 * @date    Oct 24, 2008
 * @id $Id$
 */

/**
 * SiteWorx backup test for Selenium.
 *
 * @package    Interworx
 * @subpackage Tests
 * @group      selenium
 */
class PageSWBackupTest extends IWorxSeleniumTestCase {

  static private $_first_setup = false;

  /**
   * Make sure the SiteWorx account has been deleted.
   */
  public function setUp() {
    parent::setUp();

    if( self::$_first_setup === false ) {
      $this->_fwDeleteSWAccountIfNecessary();
      $this->_fwAddSWAccountIfNecessary();
      self::$_first_setup = true;
    }//end if
  }

  /**
   * Tests the backup index page, makes sure no error appear.
   */
  public function testBackupIndex() {
    $this->deleteSWAccountIfNecessary();
    $this->addSWAccount();

    $this->loginToSWAndGoTo( '/siteworx/backup' );

    $this->assertPageRenderedWithoutErrors();
  }

  /**
   * TestCreateFullBackup function.
   */
  public function testCreateFullBackup() {
    $this->addSWAccountIfNecessary();

    $this->loginToSWAndGoTo( '/siteworx/backup' );

    $this->clickAndWaitForPopupForm( 'link=Create Backup' );

    $this->clickAndWaitForPopupForm( 'submitbutton', 'create_backup' );

    $this->click( 'submitbutton' );

    $this->waitForMessageBanner( 60 * 5 );
    $this->assertTranslatedTextPresent( '##LG_BACKUP_SUCCESS##' );
  }

  /**
   * TestCreatePartialWebBackup function.
   */
  public function testCreatePartialWebBackup() {
    $this->addSWAccountIfNecessary();

    $this->loginToSWAndGoTo( '/siteworx/backup' );

    $this->clickAndWaitForPopupForm( 'link=Create Backup' );

    $this->select( 'type', 'label=Partial Backup' );
    $this->clickAndWaitForPopupForm( 'submitbutton', 'create_backup' );

    $this->click( 'options_j_0' );
    $this->click( 'submitbutton' );

    $this->waitForMessageBanner( 60 * 5 );
    $this->assertTranslatedTextPresent( '##LG_BACKUP_SUCCESS##' );
  }

  /**
   * Tests a restoring of a partial backup.
   */
  public function testRestorePartialBackup() {
    $this->_deleteAllBackups();
    $this->_createPartialBackup();

    $SiteWorx = new SiteWorx( self::SW_DOMAIN );
    $domain   = $SiteWorx->getMasterDomainName();

    $url = '/siteworx/backup';

    $this->loginToSWAndGoTo( $url );

    $this->click( 'link=RESTORE' );
    $this->waitForPageToLoad();
    $this->click( 'submitbutton' );
    $this->waitForMessageBanner( 60 * 5 );

    $this->assertTranslatedTextPresent( '##LG_RESTORE_SUCCESS##' );
  }

  /**
   * TestLoginAfterRestore function.
   */
  public function testLoginAfterRestore() {
    $this->loginToSWAndGoTo( '/siteworx/overview' );

    $this->assertTrue( $this->isElementPresent( "//div[@id='iw-quickmenu']" ),
                       'Unable to Locate menu because I suck' );
  }

  /**
   * TestBackupDeletion.
   */
  public function testBackupDeletion() {
    $this->loginToSWAndGoTo( '/siteworx/backup' );

    $this->click( 'css=.iw-check-all' );

    $this->select( 'iw_action_name2', 'label=Delete' );

    $this->clickAndWaitForPopupForm( 'submitbutton' );
    $this->click( 'submitbutton' );

    $this->waitForMessageBanner();
    $this->assertTranslatedTextPresent( '##LG_BACKUP_DELETED##' );
  }

  /**
   * Creates a backup using routeFromPHP.
   */
  private function _createPartialBackup() {
    $CTRL     = 'Ctrl_Siteworx_Backup';
    $SiteWorx = new SiteWorx( self::SW_DOMAIN );

    $input = array( 'type'     => 'partial',
                    'options'  => array( 'web', 'mail', 'dbs' ),
                    'location' => 'siteworx',
                    'email'    => $SiteWorx->getWorkingUser()->getEmail()
                    );

    IW::FC()->routeFromPHP( $CTRL, 'create', $input, $SiteWorx );
  }

  /**
   * Deletes all backups using routeFromPHP.
   */
  private function _deleteAllBackups() {
    $CTRL     = 'Ctrl_Siteworx_Backup';
    $SiteWorx = new SiteWorx( self::SW_DOMAIN );

    $DS_Backups = new DataSource_Backup_Files( self::SW_DOMAIN );
    $input      = array( 'backups' => $DS_Backups->getValues() );

    if( !empty( $input['backups'] )  ) {
      IW::FC()->routeFromPHP( $CTRL, 'delete', $input, $SiteWorx );
    }
  }

}
