<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Jonathan Daniel <jdaniel@interworx.com>
 * @date    Oct 16, 2008
 * @id $Id$
 */

/**
 * Tests for the mysql pages of SiteWorx.
 *
 * @package    InterWorx
 * @subpackage Tests
 * @group      selenium
 */
class PageSWMySqlTest extends IWorxSeleniumTestCase {

  const MYSQL_PREFIX   = 'addedbyt_';
  const MYSQL_USERNAME = 'db_test';
  const MYSQL_PASSWORD = 'drizzle';
  const MYSQL_DBNAME   = 'testdb';

  static private $_first_setup = false;

  /**
   * Make sure the SiteWorx account has been deleted.
   */
  public function setUp() {
    parent::setUp();

    if( self::$_first_setup === false ) {
      $this->_fwDeleteSWAccountIfNecessary();
      $this->_fwAddSWAccountIfNecessary();
      self::$_first_setup = true;
    }//end if
  }

  /**
   * FillInMySqlUserForm function.
   */
  public function fillInMySqlUserForm() {
    $this->typeInto( 'name',  self::MYSQL_USERNAME );
    $this->typeInto( 'password', self::MYSQL_PASSWORD );
    $this->typeInto( 'confirm_password', self::MYSQL_PASSWORD );
  }
  /**
   * FillInMySqlDBForm function.
   */
  public function fillInMySqlDBForm() {
    $this->typeInto( 'name', self::MYSQL_DBNAME );
  }
  /**
   * ClearMySQLUserForm function.
   */
  public function clearMySQLUserForm() {
    $this->typeInto( 'name',  '' );
    $this->typeInto( 'password', '' );
    $this->typeInto( 'confirm_password', '' );
  }

  /**
   * TestAddingDBRequiredData function.
   */
  public function testAddingDBRequiredData() {
    $this->loginToSWAndGoTo( '/siteworx/mysql/db' );
    $this->typeInto( 'name', '' );

    $this->click( "//input[@value='" . IWorxTranslate::tr( '##LG_ADD##' ) . "']" );

    sleep( 1 );

    $title = $this->getElementAttributeById( 'name_status_span', 'title' );
    $this->assertEquals( IWorxTranslate::tr( '##LG_INPUT_IS_REQUIRED##' ),
                         $title );
  }

  /**
   * TestAddedMySQLDatabase function.
   */
  public function testAddingMySQLDatabase() {
    $this->loginToSWAndGoTo( '/siteworx/mysql/db' );

    $this->fillInMySqlDBForm();
    $this->click( "//input[@value='" . IWorxTranslate::tr( '##LG_ADD##' ) . "']" );

    $this->waitForMessageBanner();
    $this->assertTextPresent( 'Database created successfully',
                              'Missing success notice on ' .
                              'database creation.' );
  }

  /**
   * TestAddingMySqlUserRequiredData function.
   */
  public function testAddingMySqlUserReqiredData() {
    $this->loginToSWAndGoTo( '/siteworx/mysql/user' );

    // blank username
    $this->fillInMySqlUserForm();
    $this->typeInto( 'name', '' );
    $this->click( "//input[@value='" . IWorxTranslate::tr( '##LG_ADD##' ) . "']" );

    $title = $this->getElementAttributeById( 'name_status_span', 'title' );
    $this->assertEquals(
                        IWorxTranslate::tr( '##LG_INPUT_IS_REQUIRED##' ),
                        $title );

    $this->clearMySQLUserForm();

    // blank password 1
    $this->fillInMySqlUserForm();
    $this->typeInto( 'password', '' );
    $this->keys( "\t" );
    sleep( 1 );
    $this->click( "//input[@value='" . IWorxTranslate::tr( '##LG_ADD##' ) . "']" );

    $title = $this->getElementAttributeById(
                                            'password_status_span',
                                            'title' );
    $this->assertEquals(
                        IWorxTranslate::tr( '##LG_INPUT_IS_REQUIRED##' ),
                        $title );

    $this->clearMySQLUserForm();

    // blank password 2
    $this->fillInMySqlUserForm();
    $this->typeInto( 'confirm_password', '' );
    $this->click( "//input[@value='" . IWorxTranslate::tr( '##LG_ADD##' ) . "']" );

    $title = $this->getElementAttributeById( 'confirm_password_status_span',
                                             'title' );
    $this->assertEquals( IWorxTranslate::tr( '##LG_INPUT_IS_REQUIRED##' ),
                         $title );

    $this->clearMySQLUserForm();

    // mismatched passwords
    $this->fillInMySqlUserForm();
    $this->typeInto( 'confirm_password', 'darnbobbytables' );
    $this->click( "//input[@value='" . IWorxTranslate::tr( '##LG_ADD##' ) . "']" );

    sleep( 3 );

    $title = $this->getElementAttributeById( 'confirm_password_status_span',
                                             'title' );
    $this->assertEquals( IWorxTranslate::tr( '##LG_PASSWORDSDONOTMATCH##' ),
                         $title );

    $this->clearMySQLUserForm();
  }

  /**
   * TestAddingMySqlUser function.
   */
  public function testAddingMySqlUser() {
    $this->loginToSWAndGoTo( '/siteworx/mysql/user' );

    $this->fillInMySqlUserForm();
    $this->click( "//input[@value='" . IWorxTranslate::tr( '##LG_ADD##' ) . "']" );

    $this->waitForMessageBanner();
    $this->assertTextPresent( 'User added successfully',
                              'Error message missing ' .
                              'for successfully adding a user' );

    $this->assertTranslatedTextPresent( '##LG_DATABASE_USER_ADDEDSUCCESSFULLY##',
                                        'Error message missing ' .
                                        'for successfully adding a user' );
  }
}
