<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Jonathan Daniel <jdaniel@interworx.com>
 * @date    Jul 1, 2009
 */

/**
 * Tests the PhpMyAdmin Single-Signon.
 *
 * @package    InterWorx
 * @subpackage Tests
 * @group      selenium
 */
class PageSWPhpMyAdminTest extends IWorxSeleniumTestCase {

  static private $_first_setup = false;

  /**
   * Setup func.
   */
  public function setUp() {
    parent::setUp();

    $msg = 'Tests temporarily skipped until firefox/selenium issue resolved.';
    $this->markTestSkipped( $msg );

    if( self::$_first_setup === false ) {
      $this->_fwDeleteSWAccountIfNecessary();
      $this->_fwAddSWAccountIfNecessary();
      self::$_first_setup = true;
    }//end if
  }

  /**
   * TestPhpMyAdminSignleSignOn function.
   */
  public function testPhpMyAdminSingleSignOn() {
    $SiteWorx = new SiteWorx( self::SW_DOMAIN );
    $Driver   = $SiteWorx->getMySQLDriver();

    if( $Driver->databaseExists( 'dummy' ) === false ) {
      $Driver->addDatabase( 'dummy' );
    }

    $this->loginToSWAndGoTo( '/siteworx/overview' );
    $this->click( 'iw-menu-features-mysql-pma' );
    $this->waitForPopUp( '_blank' );
    $this->selectWindow( '_blank' );

    sleep( 6 );

    $this->selectFrame( 'frame_navigation' );
    $this->click( "//*[@id='leftframelinks']/a[2]" );
    $this->waitForPageToLoad();

    $this->assertPageRenderedWithoutErrors();
  }

  /**
   * Tests that logging into PHPMyAdmin won't work if the user doesn't have any dbs.
   */
  public function testPhpMyAdminWontLoginWithNoDbs() {
    $SiteWorx = new SiteWorx( self::SW_DOMAIN );
    $Driver   = $SiteWorx->getMySQLDriver();

    $dbs = $Driver->getDatabases();
    foreach( $dbs as $db ) {
      $Driver->deleteDatabase( $db->name );
    }

    $this->loginToSWAndGoTo( '/siteworx/overview' );
    $this->click( 'iw-menu-features-mysql-pma' );
    $this->waitForPopUp( '_blank' );
    $this->selectWindow( '_blank' );

    sleep( 3 );

    $text = 'You must have at least one database to use phpMyAdmin';

    $this->assertTextPresent( $text );
  }

}
