<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Jonathan Daniel <jdaniel@interworx.com>
 * @date    Oct 16, 2008
 */

/**
 * Tests for the users page of SiteWorx.
 *
 * @package    InterWorx
 * @subpackage Tests
 * @group      selenium
 */
class PageSWUsersTest extends IWorxSeleniumTestCase {

  static private $_first_setup = false;

  /**
   * Make sure the SiteWorx account has been deleted.
   */
  public function setUp() {
    parent::setUp();

    if( self::$_first_setup === false ) {
      $this->_fwDeleteSWAccountIfNecessary();
      $this->_fwAddSWAccountIfNecessary();
      self::$_first_setup = true;
    }//end if
  }

  /**
   * Gets all the permissions for SiteWorx.
   *
   * @param boolean $json
   * @return array
   */
  public static function getAllSWPerms( $json = false ) {
    $perms = SiteWorx::getPermsList();
    $ids   = [];
    $i     = 0;

    foreach( $perms as $perm ) {
      if( $json ) {
        $ids[] = array( 'name' => $perm['short_name'],
                        'short_name' => "perms_j_{$i}" );
      } else {
        $ids[] = array( 'name' => $perm['short_name'],
                        'short_name' => "perms_{$i}" );
      }
      $i++;
    }
    return $ids;
  }

  /**
   * Clears the form to add a SW User.
   */
  public function clearSecondarySWUserForm() {
    $this->typeInto( 'nickname', '' );
    $this->typeInto( 'email', '' );
    $this->typeInto( 'password', '' );
    $this->typeInto( 'confirm_password', '' );
    $this->click( 'link=' . IWorxTranslate::tr( '##LG_CHECKALL##' ) );
  }

  /**
   * Deletes the test users if they already exist.
   *
   * The setup function is called before EVERY test, so I had to put this here.
   */
  public function testDeleteSecondaryTestUser() {
    // Clear out the Secondary SiteWorx Test User if exists
    $user_ID = $this->getSWUserId( self::SW_DOMAIN, self::SW_SECONDARY_EMAIL );

    if( !empty( $user_ID ) ) {
      $SW    = new SiteWorx( self::SW_DOMAIN );
      $Reply = IW::FC()->routeFromPHP( 'Ctrl_Siteworx_Users',
                                       'delete',
                                       array( 'user' => self::SW_SECONDARY_EMAIL ),
                                       $SW );

      $this->assertReplySuccessful( $Reply );
    }
  }

  /**
   * Verifies default is unchecked, tests check all and check none.
   */
  public function testAddingSecondarySWUserCheckAllNone() {
    $this->loginToSWAndGoTo( '/siteworx/users?action=add' );

    $perms = self::getAllSWPerms();

    foreach( $perms as $perm ) {
      if( $perm['name'] === 'LOGIN' ) {
        continue;
      }

      $this->assertFalse( $this->isChecked( $perm['short_name'] ),
                          'Permission ' . $perm['short_name'] . ' was checked' );
    }

    $this->click( 'link=check all' );

    foreach( $perms as $perm ) {
      $this->assertTrue( $this->isChecked( $perm['short_name'] ),
                         'Permission ' .
                         $perm['short_name'] .
                         ' was not checked after "check all"' );
    }

    $this->click( 'link=check none' );

    foreach( $perms as $perm ) {
      $this->assertFalse( $this->isChecked( $perm['short_name'] ),
                          'Permission ' .
                          $perm['short_name'] .
                          ' was still checked after "check none"' );
    }
  }

  /**
   * Tests required data for secondary users.
   */
  public function testAddingSecondarySWUserRequiredData() {
    $this->loginToSWAndGoTo( '/siteworx/users?action=add' );

    // blank email
    $this->fillInAddSecondarySWUserForm();
    $this->typeInto( 'email', '' );
    $this->clickSave();

    $title = $this->getElementAttributeById( 'email_status_span', 'title' );
    $this->assertEquals( IWorxTranslate::tr( '##LG_INPUT_IS_REQUIRED##' ), $title );

    $this->clearSecondarySWUserForm();

    // bad email
    $this->fillInAddSecondarySWUserForm();
    $this->typeInto( 'email', 'tcollege@interworx' );
    /*
      $title = $this->getElementAttributeById( 'email_status_span', 'title' );
      $this->assertEquals( IWorxTranslate::tr( '##LG_INPUT_EMAIL_INVALID##' ),
      $title );
    */
    $this->clearSecondarySWUserForm();

    // blank passwords
    $this->fillInAddSecondarySWUserForm();
    $this->typeInto( 'password', '' );
    $this->typeInto( 'confirm_password', '' );
    $this->clickSave();

    $title = $this->getElementAttributeById( 'password_status_span', 'title' );
    $this->assertEquals( IWorxTranslate::tr( '##LG_INPUT_IS_REQUIRED##' ), $title );
    $this->clearSecondarySWUserForm();

    // blank password 1
    $this->fillInAddSecondarySWUserForm();
    $this->typeInto( 'password', '' );
    $this->keys( "\t" );
    sleep( 1 );
    $this->clickSave();

    $title = $this->getElementAttributeById( 'password_status_span', 'title' );
    $this->assertEquals( IWorxTranslate::tr( '##LG_INPUT_IS_REQUIRED##' ), $title );
    $this->clearSecondarySWUserForm();

    // blank password 2
    $this->fillInAddSecondarySWUserForm();
    $this->typeInto( 'confirm_password', '' );
    $this->clickSave();
    $title =
      $this->getElementAttributeById( 'confirm_password_status_span', 'title' );
    $this->assertEquals( IWorxTranslate::tr( '##LG_INPUT_IS_REQUIRED##' ), $title );
    $this->clearSecondarySWUserForm();

    // mismatched passwords
    $this->fillInAddSecondarySWUserForm();
    $this->typeInto( 'password', self::SW_SECONDARY_EMAIL . 'mismatch' );
    $this->clickSave();

    $this->waitForMessageBanner();

    $title =
      $this->getElementAttributeById( 'confirm_password_status_span', 'title' );
    $this->assertEquals( IWorxTranslate::tr( '##LG_PASSWORDSDONOTMATCH##' ),
                         $title );
  }

  /**
   * Adds a SiteWorx User.
   */
  public function testAddingSecondarySWUser() {
    $this->loginToSWAndGoTo( '/siteworx/users?action=add' );

    if( $this->getSWUserId(  self::SW_DOMAIN, self::SW_SECONDARY_EMAIL  ) ) {
      $this->fail( 'User already exists, cannot add!' );
    }

    $this->fillInAddSecondarySWUserForm();

    $this->clickSave();

    $this->waitForMessageBanner();
    $this->assertTextPresent( 'SiteWorx user successfully added' );

    $primary_ID = $this->getSWUserId( self::SW_DOMAIN, self::SW_EMAIL );
    $user_ID    = $this->getSWUserId( self::SW_DOMAIN, self::SW_SECONDARY_EMAIL  );
    $this->assertGreaterThan( $primary_ID, $user_ID, 'User was not added properly' );
  }

  /**
   * Verifies setup from testAddingSecondarySWUser().
   */
  public function testSecondarySWUserInformationIsCorrect() {
    $this->loginToSWAndGoTo( '/siteworx/users' );

    $user = self::SW_SECONDARY_EMAIL;

    $path = "//a[contains(@href, 'action=edit')][contains(@href, 'user=$user')]";
    $this->clickAndWaitForPopupForm( $path );
    $this->assertEquals( self::SW_SECONDARY_NICK, $this->getValue( 'nickname' ) );
    $this->assertEquals( self::SW_SECONDARY_EMAIL, $this->getValue( 'email' ) );

    foreach( self::getAllSWPerms( $json = true ) as $perm ) {
      $this->assertTrue( $this->isChecked( $perm['short_name'] ) );
    }
  }

  /**
   * Logs in as the newly created user, just to be sure.
   */
  public function testSecondarySWUserLoginWorks() {
    $this->loginToSWAndGoTo( '/siteworx/overview',
                             self::SW_SECONDARY_EMAIL,
                             self::SW_SECONDARY_PASS,
                             self::SW_DOMAIN );
    $this->assertTextPresent( self::SW_SECONDARY_EMAIL );
  }

  /**
   * Deletes the SiteWorx User.
   */
  public function testDeletingSecondarySWUser() {
    $this->loginToSWAndGoTo( '/siteworx/users' );

    $user = self::SW_SECONDARY_EMAIL;

    $path = "//a[contains(@href, 'action=del')][contains(@href, '$user')]";
    $this->clickAndWaitForPopupForm( $path );
    $this->assertTextPresent( 'Confirm' );
    $this->assertTextPresent( self::SW_SECONDARY_EMAIL );

    $path = "//input[@value='" . IWorxTranslate::tr( '##LG_DELETE##' ) . "']";
    $this->click( $path );

    $this->waitForMessageBanner();
    $this->assertTranslatedTextPresent( '##LG_SITEWORX_USER_DELETE_SUCCESS##' );
  }
}
