<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Jonathan Daniel <jdaniel@interworx.com>
 * @date    Jan 15, 2009
 */

/**
 * Tests to ensure Bug #1085 was corrected.
 *
 * @package    InterWorx
 * @subpackage Tests
 * @group      selenium
 */
class Bug_1085_FixTest extends IWorxSeleniumTestCase {

  /**
   * AddIpToNodeWorx.
   *
   * @param string $ip_range_start
   * @param string $ip_range_end
   * @param string $netmask
   * @param array  $args
   */
  private function _AddIpToNodeWorx( $ip_range_start,
                                     $ip_range_end,
                                     $netmask = '255.255.255.0',
                                     array $args = [] ) {

    $this->loginToNWAndGoTo( '/nodeworx/ip' );

    $link = 'link=' . IWorxTranslate::tr( '##LG_ADD_IP_ADDRESSES##' );
    $this->clickAndWaitForPopupForm( $link );

    $this->typeInto( 'ip_start', $ip_range_start );
    $this->typeInto( 'ip_end', $ip_range_end );
    $this->typeInto( 'netmask', $netmask );

    if( !empty( $args ) ) {
      foreach( $args as $key => $label ) {
        $this->select( $key, "label=$label" );
      }
    }
    $this->click( 'submitbutton' );

    $result = $this->getMessageBannerHTML();

    $this->assertTrue( !empty( $result ) );
  }

  /**
   * TestAddIPWithResellerAccess.
   */
  public function testAddIPWithResellerAccess() {
    $ip_start = '10.10.10.10';
    $ip_end   = '10.10.10.10';
    $netmask  = '255.255.255.0';
    $args     = array( 'reseller' => 'No' );

    $this->_AddIpToNodeWorx( $ip_start, $ip_end, $netmask, $args );
  }

  /**
   * TestCheckIfIPAvailableToResellersAndRemove.
   */
  public function testCheckIfIPAvailableToResellersAndRemove() {
    $xpath = "//a[@href='/nodeworx/ip?action=edit&ip=10.10.10.10']";

    $this->loginToNWAndGoTo( '/nodeworx/ip' );
    $this->clickAndWaitForPopupForm( $xpath );
    $this->assertElementPresent( 'is_reseller' );
    $this->assertElementValueEquals( 'is_reseller', 0 );
    $this->click( 'cancel' );

    $xpath = "//a[@href='/nodeworx/ip?action=delete&ip=10.10.10.10']";
    $this->clickAndWaitForPopupForm( $xpath );
    $this->click( 'confirm_action[]' );
    $this->click( 'submitbutton' );
    $result = $this->getMessageBannerHTML();

    $this->assertTrue( !empty( $result ) );
  }
}
