<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Jonathan Daniel <jdaniel@interworx.com>
 * @date    Jan 15, 2009
 */

/**
 * Tests to ensure Bug #884 was corrected.
 *
 * @package    InterWorx
 * @subpackage Tests
 * @group      selenium
 */
class Bug_884_FixTest extends IWorxSeleniumTestCase {

  /**
   * Sets the NodeWorx INI settings from the web interface.
   *
   * @param mixed $settings
   */
  public function setNodeWorxSettings( $settings = [] ) {
    $this->loginToNWAndGoTo( '/nodeworx/settings' );

    $this->assertPageRenderedWithoutErrors();
    foreach( $settings as $id => $setting ) {
      if( $setting == true ) {
        $status = 'Enabled';
      } else if( $setting == false ) {
        $status = 'Disabled';
      } else {
        $status = $setting;
      }
      $this->select( $id, "label=$status" );
    }
    $this->click( 'submitbutton' );

    $this->waitForMessageBanner();
    $this->assertTranslatedTextPresent( '##LG_SETTINGSUPDATEDSUCCESSFULLY##' );
  }

  /**
   * Checks menu items based on their DOM Id.
   *
   * @param mixed $present
   * @param mixed $not_present
   */
  private function _checkMenuItems( $present, $not_present=array() ) {
    if( !empty( $present ) ) {
      foreach( $present as $item ) {
        $this->assertElementPresent( $item );
      }
    }

    if( !empty( $not_present ) ) {
      foreach( $not_present as $item ) {
        $this->assertElementNotPresent( $item );
      }
    }
  }

  /**
   * Tests each stats setting.
   */
  public function testEachSetting() {
    // AW STATS
    $settings = array( 'stats.awstats_enabled'   => false,
                       'stats.analog_enabled'    => true,
                       'stats.webalizer_enabled' => true );

    $this->setNodeWorxSettings( $settings );

    $this->addSWAccountIfNecessary();
    $this->loginToSWAndGoTo( '/siteworx/overview' );

    $present = array( 'iw-menu-stats-analog',
                      'iw-menu-stats-webalizer' );

    $not_present = array( 'iw-menu-stats-awstats' );

    $this->_checkMenuItems( $present, $not_present );

    // ANALOG
    $settings = array( 'stats.awstats_enabled'   => true,
                       'stats.analog_enabled'    => false,
                       'stats.webalizer_enabled' => true );

    $this->setNodeWorxSettings( $settings );

    $this->addSWAccountIfNecessary();
    $this->loginToSWAndGoTo( '/siteworx/overview' );

    $present     = array( 'iw-menu-stats-awstats',
                          'iw-menu-stats-webalizer' );

    $not_present = array( 'iw-menu-stats-analog' );

    $this->_checkMenuItems( $present, $not_present );

    // Webalizer
    $settings = array( 'stats.awstats_enabled'   => true,
                       'stats.analog_enabled'    => true,
                       'stats.webalizer_enabled' => false );

    $this->setNodeWorxSettings( $settings );

    $this->addSWAccountIfNecessary();
    $this->loginToSWAndGoTo( '/siteworx/overview' );

    $present     = array( 'iw-menu-stats-awstats',
                          'iw-menu-stats-analog' );

    $not_present = array( 'iw-menu-stats-webalizer' );

    $this->_checkMenuItems( $present, $not_present );
  }

  /**
   * Disables Stat settings.
   */
  public function testDisableStatsSettings() {

    $settings = array( 'stats.awstats_enabled'   => false,
                       'stats.analog_enabled'    => false,
                       'stats.webalizer_enabled' => false );

    $this->setNodeWorxSettings( $settings );

    $this->addSWAccountIfNecessary();
    $this->loginToSWAndGoTo( '/siteworx/overview' );

    $present     = [];
    $not_present = array( 'iw-menu-stats-webalizer',
                          'iw-menu-stats-awstats',
                          'iw-menu-stats-analog' );

    $this->_checkMenuItems( $present, $not_present );
  }

  /**
   * Enables Stat settings.
   */
  public function testEnableStatusSettings() {
    $settings = array( 'stats.awstats_enabled'   => true,
                       'stats.analog_enabled'    => true,
                       'stats.webalizer_enabled' => true );

    $this->setNodeWorxSettings( $settings );

    $this->addSWAccountIfNecessary();
    $this->loginToSWAndGoTo( '/siteworx/overview' );

    $present = array( 'iw-menu-stats-webalizer',
                      'iw-menu-stats-awstats',
                      'iw-menu-stats-analog' );

    $this->_checkMenuItems( $present );
  }
}
