<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Jonathan Daniel <jdaniel@interworx.com>
 * @date    Feb 22, 2010
 */

/**
 * Test that domainkeys are properly restored on backup/restore.
 *
 * @package    InterWorx
 * @subpackage Tests
 * @group      slow-tests
 */
class BackupRestore_DomainkeyTest extends IWorx_Framework_TestCase {

  /**
   * Test that the restore still works if no domain keys are present.
   */
  public function testRestoreStillWorksWithNoDomainKeys() {
    $this->deleteSWAccountIfNecessary();

    $dkdir  = Ini::get( Ini::DIR, 'domainkeys' );
    $domain = self::SW_DOMAIN;
    if( IWorxFileSys::fileExists( "{$dkdir}/{$domain}" ) ) {
      IWorxFileSys::rm( "{$dkdir}/{$domain}" );
    }

    self::addSWAccountIfNecessary();

    $this->assertTrue( $this->doBackupAndImport() );
  }

  /**
   * Tests the domain key backup and restore.
   */
  public function testDomainKeyRestore() {
    $domain = self::SW_DOMAIN;
    $input  = array( 'domain' => $domain );

    $this->deleteSWAccountIfNecessary();
    self::addSWAccountIfNecessary();

    IW::FC()->routeFromPHP( 'Ctrl_Siteworx_EmailDomainkeys',
                            'add',
                            $input,
                            $this->getSWAccount() );

    $this->doBackupAndImport();

    $this->_checkForDomainkeyFiles();
    $this->_chekForCorrectDNSEntries();
  }

  /**
   * Checks to make sure the correct DNS entries are present.
   */
  private function _chekForCorrectDNSEntries() {
    $this->_checkForCorrectDNSDomainKey();
    $this->_checkForCorrectDNSDefaultDomainKey();
  }

  /**
   * Checks to make sure the correct _domainkey.domain DNS entry is present.
   */
  private function _checkForCorrectDNSDomainKey() {
    $domain = self::SW_DOMAIN;

    $Zone = new Dns_Zone( 'domain = ' . q( $domain ) );
    $zone_id = $Zone->getId();

    $host    = "_domainkey.{$domain}";
    $target  = 'v=DKIM1; k=rsa; t=y;';
    $where   = 'host = ' . q( $host ) . ' AND zone_id = ' . q( $zone_id );

    $this->assertTrue( Dns_Record::exists( $where ) );

    $Record = new Dns_Record( $where );

    $this->assertEquals( $target, $Record->getRdata() );
  }

  /**
   * Check to make sure the correct default._domainkey DNS entry is present.
   */
  private function _checkForCorrectDNSDefaultDomainKey() {
    $domain = self::SW_DOMAIN;

    $Zone    = new Dns_Zone( 'domain = ' . q( $domain ) );
    $zone_id = $Zone->getId();

    $host   = "default._domainkey.{$domain}";
    $target = $this->_getDomainKey();
    $where  = 'host = ' . q( $host ) . ' AND zone_id = ' . q( $zone_id );

    $this->assertTrue( Dns_Record::exists( $where ) );

    $Record = new Dns_Record( $where );

    $this->assertEquals( $target, $Record->getRdata() );
  }

  /**
   * Gets the actual domainkey value.
   *
   * @return string
   */
  private function _getDomainKey() {
    $domain = self::SW_DOMAIN;
    $dkdir  = Ini::get( Ini::DIR, 'domainkeys' );

    $cwd = getcwd();
    chdir( "{$dkdir}/{$domain}" );
    $result = shell_exec( 'grep -v ^- rsa.public | ' .
                          'perl -e \'while(<>){chop;$l.=$_;}' .
                          'print "v=DKIM1; k=rsa; t=y; p=$l;\n";\'' );
    chdir( $cwd );
    return trim( $result );
  }

  /**
   * Checks to see if the domainkey files physically exist.
   */
  private function _checkForDomainkeyFiles() {
    $domain = self::SW_DOMAIN;
    $dkdir  = Ini::get( Ini::DIR, 'domainkeys' );
    $path   = "{$dkdir}/{$domain}";

    $this->assertTrue( IWorxFileSys::fileExists( "{$path}/default" ) );
    $this->assertTrue( IWorxFileSys::fileExists( "{$path}/rsa.public" ) );
  }
}
