<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Jonathan Daniel <jdaniel@interworx.com>
 * @date    Feb 23, 2010
 */

/**
 * Test various aspects about partial backups and their restoration.
 *
 * @package    InterWorx
 * @subpackage Tests
 * @group      slow-tests
 */
class BackupRestore_PartialRestoreTest extends IWorx_Framework_TestCase {

  /**
   * Tests that a partial backup will properly restore.
   */
  public function testPartialWillRestore() {
    $this->deleteSWAccountIfNecessary(); // start fresh for this one
    $this->addSWAccountIfNecessary();

    $this->_doPartialBackup();
    $this->_doPartialRestore( '_verifyRestoreSucceeded' );
  }

  /**
   * Tests that a partial backup will *NOT* restore (gracefully).
   */
  public function testPartialWontRestoreIfDomainNotPresent() {
    $this->addSWAccountIfNecessary();

    $this->_doPartialBackup();
    $this->deleteSWAccount();

    $this->_doPartialRestore( '_verifyRestoreFailed' );
  }

  /**
   * Verify that the restore was successful.
   *
   * @param array   $result
   * @param integer $retval
   */
  private function _verifyRestoreSucceeded( array $result, $retval ) {
    $this->assertEquals( 0, $retval, implode( "\n", $result ) );
  }

  /**
   * Verifies that the restore failed.
   *
   * @param array   $result
   * @param integer $retval
   */
  private function _verifyRestoreFailed( array $result, $retval ) {
    $this->assertNotEquals( 0, $retval, implode( "\n", $result ) );
  }

  /**
   * Do the actual partial restore.
   *
   * Accepts a callback to do additional testing against the $result and
   * $retval arguments return from the exec.
   *
   * @param string $callback
   * @return boolean
   */
  private function _doPartialRestore( $callback=null ) {
    $import = Ini::get( Ini::IWORX_BIN, 'import' );
    $tmp    = Ini::get( Ini::DIR, 'tmp' );

    $cmd = $import  .
      " --archive {$tmp}/partial_backupforunittest.tgz" .
      ' --control-panel siteworx' .
      ' --master-domain ' . self::SW_DOMAIN .
      ' --ip-address ' . self::_getFirstSharedIP() .
      ' --force' .
      ' --reseller-id=1';

    IWorxExec::exec( $cmd, $result, $retval );

    if( !is_null( $callback ) ) {
      $this->{$callback}( $result, $retval );
    }

    return $retval == 0;
  }

  /**
   * Create a partial backup of the test SiteWorx account.
   *
   * @return boolean
   */
  private function _doPartialBackup() {
    $SiteWorx = $this->getSWAccount();
    $tmp      = Ini::get( Ini::DIR, 'tmp' );
    $filename = 'partial_backupforunittest';
    $options  = ' mail db';

    $backup = Ini::get( Ini::IWORX_BIN, 'backup' );

    $cmd = $backup .
      " --domains {$SiteWorx->getMasterDomainName()}" .
      " --backup-options {$options}" .
      " --output-dir {$tmp}" .
      " --filename-format {$filename}";

    IWorxExec::exec( $cmd, $result, $retval );

    return $retval == 0;
  }
}
