<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Matt Parmelee <mparmelee@interworx.com>
 * @date    May 21, 2012
 */

/**
 * Test that redirect domains are properly restored on backup/restore.
 *
 * @package    InterWorx
 * @subpackage Tests
 * @group      slow-tests
 */
class BackupRestore_RedirectDomainTest extends IWorx_Framework_TestCase {

  /**
   * Test that redirect domains are correctly restored.
   */
  public function testRedirectDomainRestore() {
    $this->deleteSWAccountIfNecessary();
    self::addSWAccountIfNecessary();

    $SiteWorx       = new SiteWorx( self::SW_DOMAIN );
    $working_domain = $SiteWorx->getWorkingDomainName();

    $name_301 = 'redir1.com';
    $name_302 = 'redir2.com';

    // add 301 redirect
    $input = array( 'domain' => $name_301,
                    'points_to' => "{$working_domain}",
                    'redir_type' => Domain::PTR_301 );

    $Reply = IW::FC()->routeFromPHP( 'Ctrl_Siteworx_DomainsPointer',
                                     'add',
                                     $input,
                                     $this->getSWAccount() );

    $this->assertReplySuccessful( $Reply );

    // add 302 redirect
    $input = array( 'domain' => $name_302,
                    'points_to' => "{$working_domain}",
                    'redir_type' => Domain::PTR_302 );

    $Reply = IW::FC()->routeFromPHP( 'Ctrl_Siteworx_DomainsPointer',
                                     'add',
                                     $input,
                                     $this->getSWAccount() );

    $this->assertReplySuccessful( $Reply );

    $redirects = $SiteWorx->getDomainNameList( Domain::REDIRECT );

    $this->assertTrue( in_array( $name_301, $redirects ) );
    $this->assertTrue( in_array( $name_302, $redirects ) );

    $zones = Payload_Factory_SW_Dns::zones( $SiteWorx )->getData();

    $zone_301_id = 0;
    foreach( $zones as $zone ) {
      if( $zone->domain === $name_301 ) {
        $zone_301_id = $zone->zone_id;
        break;
      }
    }

    $zone_302_id = 0;
    foreach( $zones as $zone ) {
      if( $zone->domain === $name_302 ) {
        $zone_302_id = $zone->zone_id;
        break;
      }
    }

    $this->assertNotEquals( 0, $zone_301_id,
                            "Could not find zone_id for {$name_301}" );
    $this->assertNotEquals( 0, $zone_302_id,
                            "Could not find zone_id for {$name_302}" );

    $Reply = IW::FC()->routeFromPHP( 'Ctrl_Siteworx_Dns',
                                     'addA',
                                     ['host'      => "home.{$name_301}",
                                      'ipaddress' => '1.2.3.4',
                                      'zone_id'   => $zone_301_id],
                                     $SiteWorx );
    $this->assertReplySuccessful( $Reply );
    $Reply = IW::FC()->routeFromPHP( 'Ctrl_Siteworx_Dns',
                                     'addA',
                                     ['host'      => "home.{$name_302}",
                                      'ipaddress' => '1.2.3.4',
                                      'zone_id'   => $zone_302_id],
                                     $SiteWorx );
    $this->assertReplySuccessful( $Reply );

    $Reply = IW::FC()->routeFromPHP( 'Ctrl_Siteworx_Dns',
                                     'queryDnsRecords',
                                     ['zone_id' => $zone_301_id],
                                     $SiteWorx );
    $this->assertReplySuccessful( $Reply );

    $this->doBackupAndImport();
    $SiteWorx = new SiteWorx( self::SW_DOMAIN );

    $redirects = $SiteWorx->getDomainNameList( Domain::REDIRECT );
    $this->assertTrue( in_array( $name_301, $redirects ) );
    $this->assertTrue( in_array( $name_302, $redirects ) );

    $zones = Payload_Factory_SW_Dns::zones( $SiteWorx )->getData();

    $zone_301_id = 0;
    foreach( $zones as $zone ) {
      if( $zone->domain === $name_301 ) {
        $zone_301_id = $zone->zone_id;
        break;
      }
    }

    $zone_302_id = 0;
    foreach( $zones as $zone ) {
      if( $zone->domain === $name_302 ) {
        $zone_302_id = $zone->zone_id;
        break;
      }
    }

    $this->assertNotEquals( 0, $zone_301_id,
                            "Could not find zone_id for {$name_301}" );
    $this->assertNotEquals( 0, $zone_302_id,
                            "Could not find zone_id for {$name_302}" );

    $Reply = IW::FC()->routeFromPHP( 'Ctrl_Siteworx_Dns',
                                     'queryDnsRecords',
                                     ['zone_id' => $zone_301_id],
                                     $SiteWorx );
    $this->assertReplySuccessful( $Reply );
    $found = false;
    foreach( $Reply->getPayloadData() as $record ) {
      if( $record->host === "home.{$name_301}" ) {
        $found = true;
        break;
      }
    }
    var_export( $Reply->getPayloadData() );
    $this->assertTrue( $found, 'Could not find dns record after import: ' .
                       "home.{$name_301}" );

    $Reply = IW::FC()->routeFromPHP( 'Ctrl_Siteworx_Dns',
                                     'queryDnsRecords',
                                     ['zone_id' => $zone_302_id],
                                     $SiteWorx );
    $this->assertReplySuccessful( $Reply );
    $found = false;
    foreach( $Reply->getPayloadData() as $record ) {
      if( $record->host === "home.{$name_302}" ) {
        $found = true;
        break;
      }
    }
    $this->assertTrue( $found, 'Could not find dns record after import: ' .
                       "home.{$name_302}" );

  }
}
