<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Jonathan Daniel <jdaniel@interworx.com>
 * @date    Feb 22, 2010
 */

/**
 * Test that vpopmail settings are properly restored on backup/restore.
 *
 * @package    InterWorx
 * @subpackage Tests
 * @group      slow-tests
 */
class BackupRestore_VpopmailTest extends IWorx_Framework_TestCase {

  /**
   * Tests the vpopmail directory hasing restore operation.
   */
  public function testVpopmailDirectoryHashingRestore() {
    $this->deleteSWAccountIfNecessary();
    self::addSWAccountIfNecessary();
    $this->_add300Mailboxes();
    $this->_delete300Mailboxes();
    $this->_addOneMailbox();
    $this->_addADummyMessageFileToInbox();

    $this->doBackupAndImport();

    $this->_checkMailboxLocation();
  }

  /**
   * Helper.
   */
  private function _add300Mailboxes() {
    $range  = range( 1, 300 );
    $domain = self::SW_DOMAIN;
    foreach( $range as $num ) {
      $cmd = "/var/vpopmail/bin/vadduser user{$num}@{$domain} password";
      IWorxExec::exec( $cmd );
    }
  }

  /**
   * Helper.
   */
  private function _delete300Mailboxes() {
    $range  = range( 1, 300 );
    $domain = self::SW_DOMAIN;
    foreach( $range as $num ) {
      $cmd = "/var/vpopmail/bin/vdeluser user{$num}@{$domain}";
      IWorxExec::exec( $cmd );
    }
  }

  /**
   * Helper.
   */
  private function _addOneMailbox() {
    $domain = self::SW_DOMAIN;
    $cmd = "/var/vpopmail/bin/vadduser testuser@{$domain} password";
    IWorxExec::exec( $cmd );
  }

  /**
   * Helper.
   */
  private function _addADummyMessageFileToInbox() {
    $domain = self::SW_DOMAIN;
    $cmd = "/bin/echo TESTING | mail testuser@{$domain}";
    IWorxExec::exec( $cmd, $result, $retval, IWorxExec::ALLOW_PIPE );
  }

  /**
   * Helper.
   */
  private function _checkMailboxLocation() {
    $SW       = $this->getSWAccount();
    $dir      = $SW->getIncomingMailDriver()->getUserDir( 'testuser' );
    $hostname = IWorxSysInfo::hostname();
    $array    = glob( "{$dir}/Maildir/new/*{$hostname}*" );
    $this->assertTrue( IWorxFileSys::isFile( $array[ 0 ] ),
                       'mail message not found' );
  }
}
