<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Paul Oehler <poehler@interworx.com>
 * @date       Sept 30, 2008
 */

/**
 * Unit tests IWorx cron scripts.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class BasicIWorxCronJobsTest extends PHPUnit_Framework_TestCase {

  /**
   * Tear down function, reloads the ini to default state.
   */
  public function tearDown() {
    Ini::reload();
  }

  /**
   * Cut/pasted from the iworx.php cron.  probably should refactor.
   *
   * @return boolean
   */
  public function iworxBandwidth() {
    $chunksize = Ini::get( Ini::IWORX, 'chunksize' );
    $ids       = SiteWorx::getIdList();

    $chunks = array_chunk( $ids, $chunksize );

    $i       = 1;
    $success = true;
    foreach( $chunks as $chunk ) {
      IWorxExec::exec( Ini::get( Ini::IWORX_CRON, 'bandwidth' ) .
                       ' --overage ' .
                       " --chunksize=$chunksize " .
                       " --chunk=$i ",
                       $result,
                       $retval,
                       IWorxExec::NO_OUTPUT );
      $i++;
      if( $retval !== 0 ) {
        $success = false;
      }
    }
    return $success;
  }

  /**
   * Cut/pasted from the iworx.php cron.  probably should refactor.
   *
   * @return boolean
   */
  public function iworxLocalhostCheck() {
    $hosts_file  = Ini::get( Ini::FILE, 'hosts' );
    $contents    = file_get_contents( $hosts_file );
    $check_regex = '^127\.0\.0\.1(.+)localhost(\s*)\n^';

    if( preg_match( $check_regex, $contents ) == 0 ) {
      $localhost_line = "\n127.0.0.1 localhost\n";
      file_put_contents( $hosts_file, $localhost_line, FILE_APPEND );
      IWorxLog::log( "localhost was missing from {$hosts_file}," .
                     ' it has been added' );
    }
    return true;
  }

  /**
   * Cut/pasted from the iworx.php cron.  probably should refactor.
   *
   * @return boolean
   */
  public function iworxStorage() {
    // we just need to run this on regular servers
    // and/or cluster managers
    if( Node::isInCluster() && !Node::isManager() ) {
      IWorxLog::log( 'skipping storage on this clustered servers',
                     IWorxLog::INFO );
      return 1;
    }

    $chunksize = Ini::get( Ini::IWORX, 'chunksize' );
    $ids       = SiteWorx::getIdList();

    $chunks = array_chunk( $ids, $chunksize );

    $i       = 1;
    $success = true;
    foreach( $chunks as $chunk ) {
      IWorxExec::exec( Ini::get( Ini::IWORX_CRON, 'storage' ) .
                       " --chunksize=$chunksize " .
                       " --chunk=$i ",
                       $result,
                       $retval,
                       IWorxExec::NO_OUTPUT );
      $i++;
      if( $retval !== 0 ) {
        $success = false;
      }
    }
    return $success;
  }

  /**
   * Cut/pasted from the iworx.php cron.  probably should refactor.
   *
   * @return boolean
   */
  public function iworxStorageSyncQuota() {
    $chunksize = Ini::get( Ini::IWORX, 'chunksize' );
    $ids       = SiteWorx::getIdList();

    $chunks = array_chunk( $ids, $chunksize );

    $i       = 1;
    $success = true;
    foreach( $chunks as $chunk ) {
      IWorxExec::exec( Ini::get( Ini::IWORX_CRON, 'storage' ) .
                       ' --syncquota ' .
                       " --chunksize=$chunksize " .
                       " --chunk=$i ",
                       $result,
                       $retval,
                       IWorxExec::NO_OUTPUT );
      $i++;
      if( $retval !== 0 ) {
        $success = false;
      }
    }
    return $success;
  }

  /**
   * Just run the bandwidth cron and see if it executes successfully or not.
   */
  public function testBandwidthCronExecution() {
    $this->assertTrue( $this->iworxBandwidth() );
  }

  /**
   * Just run the storage cron and see if it executes successfully or not.
   */
  public function testStorageCronExecution() {
    $this->assertTrue( $this->iworxStorage() );
  }

  /**
   * Test the storage cron syncquota mechanism (just looking for bin failures).
   */
  public function testStorageCronSyncQuota() {
    $this->assertTrue( $this->iworxStorageSyncQuota() );
  }

  /**
   * Tests the iworx_localhostcheck function under present conditions.
   */
  public function testLocalhostCheck() {
    $this->assertTrue( $this->iworxLocalhostCheck() );
  }

  /**
   * Tests iworx_localhostcheck if we're sure the 127.0.0.1. entry is not missing.
   *
   * For the test we are going to use a temp file, just so we aren't messing
   * with the real /etc/hosts file.
   */
  public function testLocalhostCheckWhenNotMissing() {
    $tmp        = Ini::get( Ini::DIR, 'tmp' );
    $test_hosts = "{$tmp}/hosts_test";
    $contents   = "1.2.3.4 somedomain.com\n127.0.0.1 localhost\n";

    file_put_contents( $test_hosts, $contents );
    $this->iworxLocalhostCheck();

    $expected_contents = $contents;
    $actual_contents   = file_get_contents( $test_hosts );

    $this->assertEquals( $expected_contents,
                         $actual_contents,
                         "Expected contents of {$test_hosts} do not match " .
                         'actual contents.' );
  }

  /**
   * Tests iworx_localhostcheck if we're sure the 127.0.0.1. entry is missing.
   *
   * For the test we are going to use a temp file, just so we aren't messing
   * with the real /etc/hosts file.
   */
  public function testLocalhostCheckWhenMissing() {
    $tmp        = Ini::get( Ini::DIR, 'tmp' );
    $test_hosts = "{$tmp}/hosts_test";
    $contents   = "1.2.3.4 somedomain.com\n";

    file_put_contents( $test_hosts, $contents );
    Ini::set( 'file', 'hosts', $test_hosts );

    $this->iworxLocalhostCheck();

    $expected_contents = "{$contents}\n127.0.0.1 localhost\n";
    $actual_contents   = file_get_contents( $test_hosts );

    $this->assertEquals( $expected_contents,
                         $actual_contents,
                         "Expected contents of {$test_hosts} do not match " .
                         'actual contents.' );
  }

  /**
   * Makes sure additional lines aren't added.
   */
  public function testLocalhostCheckWhenLocahostIsPresentIn127() {
    $tmp        = Ini::get( Ini::DIR, 'tmp' );
    $test_hosts = "{$tmp}/hosts_test";
    $contents   = "1.2.3.4 somedomain.com\n" .
      "127.0.0.1 xdev.interworx.com xdev localhost.localdomain localhost\n";

    file_put_contents( $test_hosts, $contents );
    Ini::set( 'file', 'hosts', $test_hosts );

    $this->iworxLocalhostCheck();

    $expected_contents = $contents;
    $actual_contents   = file_get_contents( $test_hosts );

    $this->assertEquals( $expected_contents,
                         $actual_contents,
                         "Expected contents of {$test_hosts} do not match " .
                         'actual contents.' );
  }

  /**
   * Test.
   */
  public function testLocalhostCheckWhenLocahostIsNotPresentIn127() {
    $tmp        = Ini::get( Ini::DIR, 'tmp' );
    $test_hosts = "{$tmp}/hosts_test";
    $contents   = "1.2.3.4 somedomain.com\n" .
      "127.0.0.1 xdev.interworx.com xdev localhost.localdomain\n";

    file_put_contents( $test_hosts, $contents );
    Ini::set( 'file', 'hosts', $test_hosts );

    $this->iworxLocalhostCheck();

    $expected_contents = "{$contents}\n127.0.0.1 localhost\n";
    $actual_contents   = file_get_contents( $test_hosts );

    $this->assertEquals( $expected_contents,
                         $actual_contents,
                         "Expected contents of {$test_hosts} do not match " .
                         'actual contents.' );
  }

  /**
   * Test the rrd update cron.
   */
  public function testRrdUpdateCronExecution() {
    IWorxExec::exec( Ini::get( Ini::IWORX_CRON, 'rrd' ) .
                     ' --update ' .
                     ' --services all ',
                     $result,
                     $retval,
                     IWorxExec::NO_OUTPUT );
    $this->assertEquals( 0, $retval );
  }

  /**
   * Test the dns cron.
   */
  public function testDnsCronExecution() {
    IWorxExec::exec( Ini::get( Ini::IWORX_CRON, 'dns' ) .
                     ' --sync'    .
                     ' --export ' .
                     ' --cdb ',
                     $result,
                     $retval );
    $this->assertEquals( 0, $retval );
  }

  /**
   * Test the clear password reset cron.
   */
  public function testClearPasswordResetCronExecution() {
    IWorxExec::exec( Ini::get( Ini::IWORX_CRON, 'clearpasswordreset' ),
                     $result,
                     $retval );
    $this->assertEquals( 0, $retval );
  }

  /**
   * Test the license sync cron.
   */
  public function testLicenseSyncCronExecution() {
    IWorxExec::exec( Ini::get( Ini::IWORX_CRON, 'license' ),
                     $result,
                     $retval );
    $this->assertEquals( 0, $retval );
  }
}
