<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Paul Oehler <poehler@interworx.com>
 * @date       Nov 17, 2008
 */

/**
 * Test class for CLI_APF.
 * Generated by PHPUnit on 2008-11-17 at 22:18:27.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class CLI_APFTest extends CLITestCase {

  /**
   * Gets the name of the command line script we're testing.
   *
   * @return string
   */
  protected function _getScriptToTest() {
    return Ini::get( Ini::APF_BIN, 'apf' );
  }

  /**
   * Gets the command line parameters for testing.  Is a dataprovider.
   *
   * @return array
   */
  public function getCommandlineTestInfo() {
    $get_config1 = 'DEVEL_MODE=>"0"
IFACE_TRUSTED=>""
IFACE_UNTRUSTED=>"eth0"
BLK_MCATNET=>"1"
BLK_PRVNET=>"1"
SYSCTL_TCP=>"0"
SYSCTL_ROUTE=>"1"
SYSCTL_CONNTRACK=>"34577"
TCP_STOP=>"RESET"
UDP_STOP=>"RESET"
TOS_DEF=>"8"
IG_TCP_CPORTS=>"21,22,23,25,80,110,143,443,993,995,2080,2443,3306,50000_51000,3313"
EG_TCP_CPORTS=>"22,23,25,80,443,2080,2443,3306,3313"
IG_UDP_CPORTS=>"20,21,53,3313"
EG_UDP_CPORTS=>"20,21,53,3313"
ALLOW_HOSTS=>"5.5.5.5,6.6.6.6"
DENY_HOSTS=>"7.7.7.7,8.8.8.8"';

    $get_config2 = 'DEVEL_MODE=>"0"
IFACE_TRUSTED=>""
IFACE_UNTRUSTED=>"eth0"
BLK_MCATNET=>"1"
BLK_PRVNET=>"0"
SYSCTL_TCP=>"1"
SYSCTL_ROUTE=>"0"
SYSCTL_CONNTRACK=>"34576"
TCP_STOP=>"DROP"
UDP_STOP=>"DROP"
TOS_DEF=>"4"
IG_TCP_CPORTS=>"21,22,23,25,80,110,143,443,993,995,2080,2443,3306,50000_51000"
EG_TCP_CPORTS=>"22,23,25,80,443,2080,2443,3306,4444"
IG_UDP_CPORTS=>"20,21,53"
EG_UDP_CPORTS=>"20,21,53"
ALLOW_HOSTS=>"1.1.1.1,2.2.2.2"
DENY_HOSTS=>"3.3.3.3,4.4.4.4"';

    $usage = 'Usage: apf.php [ options ]
--get-config|-g                    Get APF config file
--get-blocked-ports                Get explicitly disabled ports
--set-blocked-ports <string>       Set explicitly disabled ports (11,222,...)
--version|-v                       Get APF version
--set-allowed-ips <string>         Set list of allowed IPs (1.1.1.1,2.2.2.2,)
--set-denied-ips <string>          Set list of denied IPs (1.1.1.1,2.2.2.2)
--set-inbound-tcp <string>         Set inbound open TCP ports (1,2,...)
--set-outbound-tcp <string>        Set outbound open TCP ports (1,2,...)
--set-inbound-udp <string>         Set inbound UDP ports (1,2,...)
--set-outbound-udp <string>        Set outbound UDP ports (1,2,...)
--set-debug-mode <integer>         Set APF debug mode on/off (1|0)
--set-block-multicast <integer>    Set block multicast on/off (1|0)
--set-block-privnet <integer>      Set block private network on/off (1|0)
--set-max-sessions <integer>       Set max sessions to n
--set-sysctl-tcp <integer>         Set Sysctl TCP on/off (1|0)
--set-sysctl-route <integer>       Set Sysctl route on/off (1|0)
--set-default-tos <integer>        Set default type of service
--set-tcp-drop-policy <string>     Set TCP drop policy
--set-udp-drop-policy <string>     Set UDP drop policy
--set-trusted-interfaces <string>  Sets the trusted interfaces
--set-untrusted-interface <string> Sets the untrusted interface
--start-on-boot <string>           Check status or start/stop on boot
--isrunning|-r                     Checks of APF is running
--start                            Start APF service
--stop                             Stop APF service
--restart                          Restart APF service
--block                            Wait for restart to finish before exit
--help                             Print this help message
--verbose                          Provide verbose output if possible
--no-color                         Suppress colorized output
--force-color                      Forces colorized output.  Overrides --no-color';

    return( array( array( array( '' ),
                          $usage,
                          CLI::NO_ACTION_FOUND,
                          '_saveRunningStatus' ),
                   array( array( '--version' ),
                          IWorxRpm::getRpmVersion( 'apf' ),
                          0 ),
                   array( array( '--set-debug-mode=0',
                                 '--set-block-multicast=1',
                                 '--set-block-privnet=1',
                                 '--set-sysctl-tcp=0',
                                 '--set-sysctl-route=1',
                                 '--set-max-sessions=34577',
                                 '--set-tcp-drop-policy=RESET',
                                 '--set-udp-drop-policy=RESET',
                                 '--set-default-tos=8',
                                 '--set-inbound-tcp=21,22,23,25,80,110,143,443,993,995,2080,2443,3306,50000_51000,3313',
                                 '--set-outbound-tcp=22,23,25,80,443,2080,' .
                                 '2443,3306,3313',
                                 '--set-inbound-udp=20,21,53,3313',
                                 '--set-outbound-udp=20,21,53,3313',
                                 '--set-allowed-ips=5.5.5.5,6.6.6.6',
                                 '--set-denied-ips=7.7.7.7,8.8.8.8' ),
                          '',
                          0 ),
                   array( array( '--get-config' ),
                          $get_config1,
                          0 ),
                   array( array( '--set-allowed-ips=1.1.1.1,2.2.2.2' ),
                          '',
                          0 ),
                   array( array( '--set-denied-ips=3.3.3.3,4.4.4.4' ),
                          '',
                          0 ),
                   array( array( '--set-inbound-tcp=21,22,23,25,80,110,143,443,993,995,2080,2443,3306,50000_51000' ),
                          '',
                          0 ),
                   array( array( '--set-outbound-tcp=22,23,25,80,443,2080,2443,3306,'
                                 . '4444' ),
                          '',
                          0 ),
                   array( array( '--set-inbound-udp=20,21,53' ),
                          '',
                          0 ),
                   array( array( '--set-outbound-udp=20,21,53' ),
                          '',
                          0 ),
                   array( array( '--set-default-tos=4' ),
                          '',
                          0 ),
                   array( array( '--set-udp-drop-policy=DROP' ),
                          '',
                          0 ),
                   array( array( '--set-tcp-drop-policy=DROP' ),
                          '',
                          0 ),
                   array( array( '--set-max-sessions=34576' ),
                          '',
                          0 ),
                   array( array( '--set-sysctl-route=0' ),
                          '',
                          0 ),
                   array( array( '--set-sysctl-tcp=1' ),
                          '',
                          0 ),
                   array( array( '--set-block-privnet=0' ),
                          '',
                          0 ),
                   array( array( '--set-debug-mode=0' ),
                          '',
                          0 ),
                   array( array( '--set-sysctl-route=0' ),
                          '',
                          0 ),
                   array( array( '--get-config' ),
                          $get_config2,
                          0 ),
                   array( array( '--stop' ),
                          '',
                          0 ),
                   array( array( '--isrunning' ),
                          '',
                          1 ),
                   array( array( '--start' ),
                          '',
                          0 ),
                   array( array( '--isrunning' ),
                          '',
                          0 ),
                   array( array( '--restart --block' ),
                          '',
                          0 ),
                   array( array( '--isrunning' ),
                          '',
                          0,
                          '_restoreRunningStatus' ),
                   ));
  }

  /**
   * Saves the firewall's status for the test session.
   */
  protected function _saveRunningStatus() {
    //    $cmd = $this->_getScriptToTest();
    //    IWorxExec::exec( "$cmd --isrunning", $result, $retval );
    //    if( $retval == CLI::NOT_OK ) {
    //      $this->_firewallWasOff = true;
    //    }
  }

  /**
   * Restores the firewall's status after the test session.
   */
  protected function _restoreRunningStatus() {
    //    if( $this->_firewallWasOff ) {
    //      $cmd = $this->_getScriptToTest();
    //      IWorxExec::exec( "$cmd --stop", $result, $retval );
    //    }
  }
}

