<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Jonathan Daniel <jdaniel@interworx.com>
 * @date    Jan 22, 2009
 */

/**
 * CLI_IWorxDb_BackupTest.
 *
 * @package    InterWorx
 * @subpackage Tests
 * @group      slow-tests
 *
 */
class CLI_IWorxDb_BackupTest extends CLITestCase {

  /**
   * Gets the name of the command line script we're testing.
   *
   * @return string
   */
  protected function _getScriptToTest() {
    return Ini::get( Ini::IWORX_BIN, 'iworxdb-backup' );
  }

  /**
   * Gets the command line parameters for testing.  Is a dataprovider.
   *
   * @return array
   */
  public function getCommandlineTestInfo() {
    return [];
  }

  /**
   * Does nothing.
   */
  public function testCommandlineScript() {
    // does nothing
  }

  /**
   * Tests to make sure the --get-backup-dir CLI options is working properly.
   */
  public function testGetBackupDir() {
    $actual   = $this->_getBackupDir();
    $expected = '/home/interworx/var/backups/iworxdb';

    $this->assertEquals( $expected, $actual );
  }

  /**
   * TestIWorxDBBackup.
   */
  public function testIWorxDBBackup() {
    $backup_dir = $this->_getBackupDir();
    IWorxExec::quick_exec( $this->_getScriptToTest() );

    $files = scandir( $backup_dir );
    $this->assertTrue( count( $files ) > 2 );

    $this->_runXTimes( 6 );

    $files = scandir( $backup_dir );
    $this->assertTrue( count( $files ) >= 7 );
  }

  /**
   * Duchebag style testing of the auto file delete feature.
   */
  public function testAutodelete() {
    $test_filenames =
      array( 'dontDeleteMe',
             'testfile1.sql.gz',
             'testfile1.sql',
             'iworxdb-12345678901234.txt.bz',
             'iworxdb-12345678901234.txt.gx',
             'iworxdb-12345678901234.txt.gz',
             '.iworxdb-12345678901234.sql.gz',
             '1worxdb-12345678901234.sql.gz', //that's a 1 (one)
             'iworxdb-12345678901234.sql2.gz',
             'iworxdb-12345678901234.sq1.gz', //that's a 1 (one)
             'iworxdb-12345678901234.gz.sql',
             'iworxdb-00000000000X00.sql.gz',
             'iworxdb-00000000000X00.sql',
             'iworxdb-0000100000000.sql.gz',
             'iworxdb-12345678901X34.sql.gz' );

    $this->_doAutodeleteTest( $test_filenames );
  }

  /**
   * DoAutodeleteTest.
   *
   * @param array $test_filenames
   */
  private function _doAutodeleteTest( array $test_filenames ) {
    $backup_dir = $this->_getBackupDir();
    $content    = 'Should never be deleted';

    foreach( $test_filenames as $test_filename ) {
      $testfile_path = "$backup_dir/$test_filename";

      IWorxFileSys::smartBarf( $testfile_path, $content );

      $validate = new IWorx_Validate_File_Local();
      // Do an assert here for good measure.
      $this->assertTrue( $validate->isValid( $testfile_path ) );
    }

    // Run the backup code 6 times.  If the backup puring is working properly
    // the the test file created will not be deleted.
    $this->_runXTimes( 6 );

    foreach( $test_filenames as $test_filename ) {
      $testfile_path = "$backup_dir/$test_filename";
      $this->assertTrue( $validate->isValid( $testfile_path ),
                         "$testfile_path is gone" );

      // Now that we're done testing we can delete this file.
      IWorxFileSys::rm( $testfile_path );
    }
  }


  /**
   * Returns the backup dir.
   *
   * @return string
   */
  private function _getBackupDir() {
    $iworxdb_backup = $this->_getScriptToTest();

    $cmd = "$iworxdb_backup --get-backup-dir";
    IWorxExec::exec( $cmd, $result, $retval );
    return implode( '', $result );
  }

  /**
   * Runs the script X times.
   *
   * @param integer $num_times
   */
  private function _runXTimes( $num_times ) {
    $count = 0;
    while( $num_times >= $count ) {
      IWorxExec::quick_exec( $this->_getScriptToTest() );
      sleep( 1 );

      $count++;
    }
  }
}
