<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2014 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Upgrade
 * @author     Tim College <tcollege@interworx.com>
 * @date       Jul 17, 2014
 */

/**
 * Class CLI_IniTest
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class CLI_IniTest extends IWorx_Framework_TestCase {
  /**
   * Setup function.
   */
  public function setUp() {
    Ini::remove( Ini::IWORX, 'test' );
    Ini::write();
  }

  /**
   * Tests getting a value via CLI.
   *
   * @throws IWorx_Exception_ExecNoCommand If Command is missing.
   */
  public function testGetIniValue() {
    Ini::set( Ini::IWORX, 'test', 'test_value' );
    Ini::write();
    $Cmd = new Command();
    $Cmd->setCmd( '/home/interworx/bin/ini.pex --get --section iworx --index test' )
      ->exec();

    $output = $Cmd->getOutput();

    $this->assertEquals( 'test_value', $output[0] );
  }

  /**
   * Tests setting an ini value through CLI.
   *
   * @throws IWorx_Exception_ExecNoCommand If no command set.
   */
  public function testSetIniValue() {
    $Cmd = new Command();
    $cmd = '/home/interworx/bin/ini.pex --set --section iworx --index test ' .
      '--value 5';
    $Cmd->setCmd( $cmd )->exec();

    $Cmd = new Command();
    $Cmd->setCmd( '/home/interworx/bin/ini.pex --get --section iworx --index test' )
      ->exec();

    $output = $Cmd->getOutput();

    $this->assertEquals( '5', $output[0] );
  }
}
