<?php
/**
 * InterWorx Hosting Control Panel
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2015 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Upgrade
 * @author     Tim College <tcollege@interworx.com>
 * @date       Nov 30, 2015
 */

/**
 * Class CLI_LogTest
 *
 * @package    InterWorx
 * @subpackage TESTS
 */
class CLI_LogTest extends IWorx_Framework_TestCase {
  /**
   * Test that the log.pex works.
   */
  public function testLogMessageWithDefaultLevel() {
    IWorxLog::log( 'logging to get a request id set' );

    $message = 'Test message ' . __FUNCTION__;
    $log     = Ini::get( Ini::IWORX_BIN, 'log' );
    $Cmd     = new Command( "{$log} --message '{$message}'" );
    $Cmd->allowQuoteSingle();
    $Cmd->exec();

    $log_id = 'logpex-' . IW::Env()->getLogRequestId();
    $this->assertLogEntryExists( $message, $log_id, null, IWorxLog::INFO );
  }

  /**
   * Test that the log.pex works with custom levels.
   */
  public function testLogMessageWithCustomLevel() {
    IWorxLog::log( 'logging to get a request id set' );

    $message = 'Test message ' . __FUNCTION__;
    $log     = Ini::get( Ini::IWORX_BIN, 'log' );
    $Cmd     = new Command( "{$log} --message '{$message}' --level ALERT" );
    $Cmd->allowQuoteSingle();
    $Cmd->exec();

    $log_id = 'logpex-' . IW::Env()->getLogRequestId();
    $this->assertLogEntryExists( $message, $log_id, null, IWorxLog::ALERT );
  }

  /**
   * Test that the log.pex works with custom request ids.
   */
  public function testLogMessageWithCustomRequestId() {
    IWorxLog::log( 'logging to get a request id set' );

    $message = 'Test message ' . __FUNCTION__;
    $log     = Ini::get( Ini::IWORX_BIN, 'log' );
    $cmd     = "{$log} --message '{$message}' --level ALERT --request_id test";
    $Cmd     = new Command( $cmd );
    $Cmd->allowQuoteSingle();
    $Cmd->exec();

    $log_id = 'logpex-test';
    $this->assertLogEntryExists( $message, $log_id, null, IWorxLog::ALERT );
  }
}
