<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Jonathan Daniel <jdaniel@interworx.com>
 * @date    Dec 3, 2010
 */

/**
 * Test case for scheduled backups.
 *
 * @package    InterWorx
 * @subpackage Tests
 * @group      slow-tests
 */
class CLI_ScheduledBackupTest extends CLITestCase {

  static private $_Scheduled = null;

  /**
   * Setup.
   */
  public static function setUpBeforeClass() {
    $SW = new SiteWorx( IWorx_Framework_TestCase::SW_DOMAIN );
    if( $SW->exists() ) {
      $NW = IWorx_Framework_TestCase::getMasterNodeWorx();
      $NW->deleteAccount( $SW );
    }//end if

    IWorx_Framework_TestCase::addSWAccountIfNecessary();

    Ini::set( Ini::IWORX_BACKUP, 'enable_daily', '1' );

    $SiteWorx = new SiteWorx( IWorx_Framework_TestCase::SW_DOMAIN );
    $input    = array( 'frequency' => Model_Scheduled_Backup::FREQUENCY_DAILY,
                       'type'      => Model_Scheduled_Backup::TYPE_STRUCTURE,
                       'location'  => Model_Scheduled_Backup::LOCATION_SITEWORX,
                       'hour'      => '1',
                       'rotate'    => '2' );

    $Reply = IW::FC()->routeFromPHP( 'Ctrl_Siteworx_BackupSchedule',
                                     'create',
                                     $input,
                                     $SiteWorx );

    $find_args = array( 'siteworx_id' => $SiteWorx->getId() );
    $s         = Model_Scheduled_Backup::find( $find_args );
    $Scheduled = array_shift( $s );

    self::addVariable( 'id', $Scheduled->getPrimaryId() );

    self::$_Scheduled = $Scheduled;
  }

  /**
   * Gets the name of the script to test.
   *
   * @return array
   */
  protected function _getScriptToTest() {
    return Ini::get( Ini::IWORX_BIN, 'scheduled-backup' );
  }

  /**
   * Gets the command line test info.
   *
   * @return array
   */
  public function getCommandlineTestInfo() {
    // We're executing multiple times to make sure the rotation is working.
    return array( $this->_getSyncCronTestcase(),
                  $this->_getExecuteTestCase(),
                  $this->_getExecuteTestCase(),
                  $this->_getExecuteTestCase() );
  }

  /**
   * Gets the test case info.
   *
   * @return array
   */
  private function _getExecuteTestCase() {
    $Scheduled       = self::$_Scheduled;

    $params          = array( '--execute=%%id%%' );
    $expected_result = '';
    $expected_retval = CLI::OK;
    $callback        = '_checkExecute';

    $case = array( $params,
                   $expected_result,
                   $expected_retval,
                   $callback );

    return $case;
  }

  /**
   * Gets the test case info.
   *
   * @return array
   */
  private function _getSyncCronTestcase() {
    $params          = array( '--sync-cron' );
    $expected_result = '';
    $expected_retval = CLI::OK;
    $callback        = '_checkCron';

    $case = array( $params,
                   $expected_result,
                   $expected_retval,
                   $callback );

    return $case;
  }

  /**
   * Checks some conditions after execute is run.
   */
  protected function _checkExecute() {
    $Scheduled = self::$_Scheduled;
    $backups = $Scheduled->getBackups();

    $this->assertTrue( count( $backups ) <= $Scheduled->rotate );
  }

  /**
   * Checks some conditions after cron-sync is running.
   */
  protected function _checkCron() {
    $cron = IWorxCron::getUserCrontab( 'iworx-backup' );

    $this->assertTrue( count( $cron ) > 1 );
  }
}

