<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2015 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Upgrade
 * @author     Tim College <tcollege@interworx.com>
 * @date       Jul 02, 2015
 */

/**
 * Class CLI_Support_NodeworxUsersTest
 *
 * @package    InterWorx
 * @subpackage TESTS
 */
class CLI_Support_NodeworxUsersTest extends IWorx_Framework_TestCase {
  /**
   * Confirm that the --dry-run option finds the secondary user.
   */
  public function testDryRunFindsNodeworxUsers() {
    $input = $this->_getInput();

    $NW = new NodeWorx( NodeWorx::MASTER_ID );

    $Reply = IW::FC()->routeFromPHP( 'Ctrl_Nodeworx_Users',
                                     'deleteCommit',
                                     $input,
                                     $NW );

    $Reply = IW::FC()->routeFromPHP( 'Ctrl_Nodeworx_Users',
                                     'addCommit',
                                     $input,
                                     $NW );

    $this->assertReplySuccessful( $Reply );

    $Cmd = new Command_WithArgs( Ini::get( Ini::IWORX_BIN, 'support' ) );
    $Cmd->addOption( 'cleanup' )
      ->addOption( 'verbose' )
      ->addOption( 'dry-run' )
      ->exec();

    $needle = 'Deleting NodeWorx Secondary User ' . $input['nickname'] . ' ' .
      $input['email'];
    $this->assertCliOutputContains( $needle, $Cmd->getOutput() );
  }

  /**
   * Make sure the user is actually deleted.
   */
  public function testCleanupRemovesNodeworxUser() {
    $Cmd = new Command_WithArgs( Ini::get( Ini::IWORX_BIN, 'support' ) );
    $Cmd->addOption( 'cleanup' )
      ->addOption( 'verbose' )
      ->exec();

    $input = $this->_getInput();

    $Search = new IWorx_Search_NwUsers();
    $Search->setOption( 'email', $input['email'] );
    $Search->addSelectableField( 'email' );
    $Search->search();

    $this->assertEquals( 0, $Search->foundRows() );
  }

  /**
   * Gets input for both adding and deleting, hence the 'email' and 'user'.
   *
   * @return array
   */
  protected function _getInput() {
    $input = ['nickname'         => 'Support User Account',
              'email'            => 'whatever@interworx.support',
              'user'             => 'whatever@interworx.support',
              'password'         => 'asdf0897asdf98787sdfsdf98878d7fdfdfdfd',
              'confirm_password' => 'asdf0897asdf98787sdfsdf98878d7fdfdfdfd'];
    return $input;
  }
}
