<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2015 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Upgrade
 * @author     Tim College <tcollege@interworx.com>
 * @date       Jul 02, 2015
 */

/**
 * Class CLI_Support_ResellerAccountTest
 *
 * @package    InterWorx
 * @subpackage TESTS
 */
class CLI_Support_ResellerAccountTest extends IWorx_Framework_TestCase {
  /**
   * Verifies that the cleanup function finds the reseller.
   */
  public function testDryRunFindsResellerAccount() {
    $email = 'unittestreseller@interworx.support';
    $this->deleteResellerAccountIfNecessary( $email );

    $input          = $this->_getResellerInput();
    $input['email'] = $email;

    $this->_addResellerFromInput( $input );

    $Cmd = new Command_WithArgs( Ini::get( Ini::IWORX_BIN, 'support' ) );
    $Cmd->addOption( 'cleanup' )
      ->addOption( 'verbose' )
      ->addOption( 'dry-run' )
      ->exec();

    $this->assertCliOutputContains( "Deleting Reseller Account {$email}",
                                    $Cmd->getOutput() );
  }

  /**
   * Tests that cleanup removes the reseller account appropriately.
   *
   * @depends testDryRunFindsResellerAccount
   */
  public function testCleaningUpResellerAccount() {
    $Cmd = new Command_WithArgs( Ini::get( Ini::IWORX_BIN, 'support' ) );
    $Cmd->addOption( 'cleanup' )
      ->addOption( 'verbose' )
      ->exec();

    $NW = new NodeWorx( 'unittestreseller@interworx.support' );

    $this->assertEquals( -1, $NW->getId() );
  }
}
