<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2015 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Upgrade
 * @author     Tim College <tcollege@interworx.com>
 * @date       Jul 01, 2015
 */

/**
 * Class SupportTest
 *
 * @package    InterWorx
 * @subpackage TESTS
 */
class CLI_Support_SiteworxAccountTest extends IWorx_Framework_TestCase {
  const DOMAIN   = 'clisupporttest.com';
  const UNIQNAME = 'clisuppo';
  const EMAIL    = 'unittest@interworx.support';

  /**
   * Checks that the SiteWorx account will be deleted.
   */
  public function testDryRunFindsSiteworxAccount() {
    self::deleteSWAccountIfNecessaryStatic( self::DOMAIN );

    $input                  = $this->_getSWAccountInput();
    $input['master_domain'] = self::DOMAIN;
    $input['uniqname']      = self::UNIQNAME;
    $input['email']         = self::EMAIL;

    $this->addSWAccountFromInput( $input );

    $Cmd = new Command_WithArgs( Ini::get( Ini::IWORX_BIN, 'support' ) );
    $Cmd->addOption( 'cleanup' )
      ->addOption( 'verbose' )
      ->addOption( 'dry-run' )
      ->exec();

    $this->assertCliOutputContains( 'Deleting SiteWorx Account ' . self::DOMAIN,
                                    $Cmd->getOutput() );
  }

  /**
   * Tests that cleanup removes the siteworx account appropriately.
   *
   * @depends testDryRunFindsSiteworxAccount
   */
  public function testCleaningUpSiteworxAccount() {
    $Cmd = new Command_WithArgs( Ini::get( Ini::IWORX_BIN, 'support' ) );
    $Cmd->addOption( 'cleanup' )
      ->addOption( 'verbose' )
      ->exec();

    $SW = new SiteWorx( self::DOMAIN );

    $this->assertFalse( $SW->exists() );
  }
}
