<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Jonathan Daniel <jdaniel@interworx.com>
 * @date    Mar 11, 2010
 */

/**
 * Tests that the update cron is parsing the yum output properly.
 *
 * @package    InterWorx
 * @subpackage Tests
 * @group      slow-tests
 */
class CLI_UpdateCronTest extends PHPUnit_Framework_TestCase {
  /**
   * Install some old and outdated RPMs so we can let yum do it's thing.
   */
  public function testSetupDowngradeTzdataPackage() {
    $dir      = Ini::get( Ini::IWORX_DIR, 'base' );
    $rpm_path = 'TESTS/_resources/tzdata-2016c-1.el6.noarch.rpm';

    $Cmd = new Command( "yum downgrade -y {$dir}/{$rpm_path}" );
    $Cmd->stderrToStdout();
    $Cmd->exec();

    if( $Cmd->wasSuccessful() === false ) {
      foreach( $Cmd->getOutput() as $line ) {
        IWorxConsole::echoLine( $line );
      }
    }

    $this->assertTrue( $Cmd->wasSuccessful(),
                       'Downgrading the tzdata package failed.' );
  }

  /**
   * Test that the records were properly created.
   *
   * This should help let us know when something breaks from yum changing their
   * output.
   *
   * @depends testSetupDowngradeTzdataPackage
   */
  public function testUpdateRecordCreated() {
    $update = Ini::get( Ini::IWORX_CRON, 'update' );
    $cmd    = "{$update} --check-for-updates";

    IWorxExec::exec( $cmd, $result, $retval );

    $cmd = "{$update} --os --apply";

    IWorxExec::exec( $cmd, $result, $retval );

    $ok = $this->_updateExistsFromCurrentBatch( 'tzdata' );

    $msg = 'Updates are missing! Did yum change their output again?';

    $this->assertTrue( $ok, $msg );
  }

  /**
   * Given a package name, check to see if it exists in the current batch.
   *
   * @param string $name
   * @return boolean
   *
   * @depends testSetupBeforeClass
   */
  private function _updateExistsFromCurrentBatch( $name ) {
    $Payload = Payload_Factory_NW_Updates::recentHistory();
    $data    = $Payload->getData();
    return $data[0]->name === $name;
  }
}
