<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Paul Oehler <poehler@interworx.com>
 * @date       Nov 17, 2008
 */

/**
 * Test class for CLI_VpopmailTest.
 * Generated by PHPUnit on 2008-11-17 at 22:18:27.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class CLI_VpopmailTest extends CLITestCase {

  const DEFAULT_TO   = 'joe@luser.com';
  const DOMAIN       = 'addedbytest.com';
  const ALIAS_DOMAIN = 'addedbytest-alias.com';
  const DROPSCORE    = 15;

  /**
   * Setup.
   */
  public function setup() {
    parent::setUp();
    IWorx_Framework_TestCase::addSWAccountIfNecessary();
  }

  /**
   * Gets the name of the command line script we're testing.
   *
   * @return string
   */
  protected function _getScriptToTest() {
    return Ini::get( Ini::VPOPMAIL_BIN, 'vpopmail' );
  }

  /**
   * Gets the command line parameters for testing.  Is a dataprovider.
   *
   * @return array
   */
  public function getCommandlineTestInfo() {
    $usage = 'Usage: vpopmail.php [ options ]
--debug|-d                            Debug mode on
--nodebug|-n                          Debug mode off (default)
--set-catchall|-H                     Set catch-all e-mail address
--delete-catchall|-E                  Delete catch-all e-mail address
--to|-T <string>                      E-mail address to set catch-all to
--delete-autoresponder                Delete an e-mail autoresponder
--storage|-s                          Get disk space used by an e-mail box
--human|-h                            Get disk space in human readable format
--domain|-o <string>                  Domain name to act on ie domain.com
--user|-u <string>                    User name to act on ie postmaster
--chgrp|-c                            Chgrp mail dirs to appropriate values
--set-autorespond-message|-a <string> File name containing auto-respond message to set
--set-vacation-message|-t <string>    File name containing vacation message to set
--add-alias-domain|-L                 Add an alias domain
--delete-alias-domain|-D              Delete an alias domain
--alias-domain|-A <string>            Alias domain to add ie domain.com
--spamassassin-on|-S <string>         Set spamassassin scanning mode. yes/no
--spamassassin-drop-on <string>       Set the spamassassin drop mode. yes/no
--dropscore <string>                  Spam score required to drop email
--bounce-on|-B <string>               Set bounce messages mode. yes/no
--status <string>                     Get status of mail service. pop3/imap/...
--stop <string>                       Stop a mail service. pop3/imap/...
--start <string>                      Start a mail service. pop3/imap/...
--restart <string>                    Restart a mail service. pop3/imap/...
--list-softlimit <string>             Get the softlimit setting. pop3/imap/...
--list-max-connections <string>       Get the maxconn setting. pop3/imap/...
--set-limits <string>                 Set limit info. comma seperated
--start-on-boot <string>              Set/Get start-on-boot status. service,yes/no/status
--update-sql-file <string>            Update the vpop sql file with new info
--fixPerms <string>                   Fix mail perms for the specified domain
--createFilterDir <string>            Create the filter storage dir for a domain
--help                                Print this help message
--verbose                             Provide verbose output if possible
--no-color                            Suppress colorized output
--force-color                         Forces colorized output.  Overrides --no-color';

    $set_catchall_requires    = '--set-catchall option requires --domain
--set-catchall option requires --to';
    $delete_catchall_requires = '--delete-catchall option requires --domain';

    return( array( array( array( '' ),
                          $usage,
                          CLI::NO_ACTION_FOUND ),
                   array( array( '--set-catchall' ),
                          $set_catchall_requires,
                          CLI::PARAM_REQUIRES_CODE ),
                   array( array( '--delete-catchall' ),
                          $delete_catchall_requires,
                          CLI::PARAM_REQUIRES_CODE ),
                   array( array( '--set-catchall',
                                 '--domain=' . self::DOMAIN,
                                 '--to=' . self::DEFAULT_TO ),
                          '',
                          0,
                          '_checkSetCatchall' ),
                   array( array( '--delete-catchall',
                                 '--domain=' . self::DOMAIN ),
                          '',
                          0,
                          '_checkDeleteCatchall' ),
                   array( array( '--spamassassin-on=yes',
                                 '--domain=' . self::DOMAIN ),
                          '',
                          0,
                          '_checkSpamassassinOnYes' ),
                   array( array( '--spamassassin-on=no',
                                 '--domain=' . self::DOMAIN ),
                          '',
                          0,
                          '_checkSpamassassinOnNo' ),
                   array( array( '--spamassassin-drop-on=yes',
                                 '--domain=' . self::DOMAIN,
                                 '--dropscore=' . self::DROPSCORE ),
                          '',
                          0,
                          '_checkSpamassassinDropOnYes' ),
                   array( array( '--spamassassin-drop-on=no',
                                 '--domain=' . self::DOMAIN ),
                          '',
                          0,
                          '_checkSpamassassinDropOnNo' ),
                   array( array( '--bounce-on=yes',
                                 '--domain=' . self::DOMAIN ),
                          '',
                          0,
                          '_checkBounceOnYes' ),
                   array( array( '--bounce-on=no',
                                 '--domain=' . self::DOMAIN ),
                          '',
                          0,
                          '_checkBounceOnNo' ),
                   // TODO: there's more to do.  see CLI_Vpopmail
                   ) );
  }

  /**
   * Confirms that spamassassin on worked.
   */
  protected function _checkSpamassassinOnYes() {
    $Siteworx = new SiteWorx( self::DOMAIN );
    $maildir  = $Siteworx->getLocalStateDir( SiteWorx::DIR_MAIL );

    $this->assertFileExists( "{$maildir}/.spamon", '
                             spamon file does not exist' );
  }

  /**
   * Confirms that spamassassin off worked.
   */
  protected function _checkSpamassassinOnNo() {
    $Siteworx = new SiteWorx( self::DOMAIN );
    $maildir  = $Siteworx->getLocalStateDir( SiteWorx::DIR_MAIL );

    $this->assertFileNotExists( "{$maildir}/.spamon", '
                             spamon file exist and it should not' );
  }

  /**
   * Confirms that spamassassin drop on actually worked.
   */
  protected function _checkSpamassassinDropOnYes() {
    $Siteworx = new SiteWorx( self::DOMAIN );
    $maildir  = $Siteworx->getLocalStateDir( SiteWorx::DIR_MAIL );

    $this->assertFileExists( "{$maildir}/.dropscore", '
                             dropscore file does not exist' );
    $this->assertEquals( self::DROPSCORE,
                         file_get_contents( "{$maildir}/.dropscore" ),
                         'dropscore file contains bad data' );
  }

  /**
   * Confirms that spamassassin drop off actually worked.
   */
  protected function _checkSpamassassinDropOnNo() {
    $Siteworx = new SiteWorx( self::DOMAIN );
    $maildir  = $Siteworx->getLocalStateDir( SiteWorx::DIR_MAIL );

    $this->assertFileNotExists( "{$maildir}/.dropscore", '
                             dropscore exists and it should not' );
  }

  /**
   * Confirms that adding bounceon actually worked.
   */
  protected function _checkBounceOnYes() {
    $Siteworx = new SiteWorx( self::DOMAIN );
    $maildir  = $Siteworx->getLocalStateDir( SiteWorx::DIR_MAIL );

    $this->assertFileExists( "{$maildir}/.bounceon", '
                             bounceon file does not exist' );
  }

  /**
   * Confirms that delete bounceon actually worked.
   */
  protected function _checkBounceOnNo() {
    $Siteworx = new SiteWorx( self::DOMAIN );
    $maildir  = $Siteworx->getLocalStateDir( SiteWorx::DIR_MAIL );

    $this->assertFileNotExists( "{$maildir}/.bounceon", '
                             bounceon exists and it should not' );
  }

  /**
   * Confirms set catchall actually worked.
   */
  protected function _checkSetCatchall() {
    $Siteworx = new SiteWorx( self::DOMAIN );
    $maildir  = $Siteworx->getLocalStateDir( SiteWorx::DIR_MAIL );

    $this->assertFileExists( "{$maildir}/.catchall", '
                             catchall file does not exist' );

    $this->assertEquals( self::DEFAULT_TO,
                         file_get_contents( "{$maildir}/.catchall" ) );
  }

  /**
   * Confirms delete catchall actually worked.
   */
  protected function _checkDeleteCatchall() {
    $Siteworx = new SiteWorx( self::DOMAIN );
    $maildir  = $Siteworx->getLocalStateDir( SiteWorx::DIR_MAIL );

    $this->assertFileNotExists( "{$maildir}/.catchall",
                                'catchall file exists and it should not' );
  }

  /**
   * Confirms add alias domain actually worked.
   */
  protected function _checkAddAliasDomain() {
    $cmd = Ini::get( Ini::VPOPMAIL_BIN, 'vdominfo' );
    IWorxExec::exec( "$cmd -n " . self::ALIAS_DOMAIN, $result, $retval );
    $this->assertEquals( self::DOMAIN, $result[0], 'add alias domain failed' );
    $this->assertEquals( 0, $retval, 'add alias domain failed' );
  }

  /**
   * Confirms delete alias domain actually worked.
   */
  protected function _checkDeleteAliasDomain() {
    $cmd = Ini::get( Ini::VPOPMAIL_BIN, 'vdominfo' );
    IWorxExec::exec( "$cmd -n " . self::ALIAS_DOMAIN, $result, $retval );
    $this->assertEquals( 'domain ' . self::ALIAS_DOMAIN . ' does not exist',
                         $result[0],
                         'delete alias domain failed' );
    $this->assertEquals( 255, $retval, 'delete alias domain failed' );
  }
}
