<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Tim College <tcollege@interworx.com>
 * @date       Sep 2, 2008
 */

/**
 * DESC.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class CLIErrorBehaviorTest extends PHPUnit_Framework_TestCase {
  /**
   * Test.
   *
   * @group errorHandling
   */
  public function testError() {
    TestDebug::turnDebugOn();
    exec( '/usr/local/interworx/bin/php ' .
          '~iworx/html/test/error_tests.php error 2>/dev/null',
          $command_output );
    $command_output = implode( "\n", $command_output );
    $this->assertContains( 'InterWorx Error Report',
                           $command_output,
                           'Missing error Report' );
    $this->assertContains( 'FILE: /usr/local/interworx/html/test/error_tests.php',
                           $command_output,
                           'FILE was not in error report' );
    $this->assertRegExp( '/LINE: [0-9]+/',
                         $command_output,
                         'LINE number missing' );
    $this->assertContains( 'I triggered an error!',
                           $command_output,
                           'Error Message Missing' );
    TestDebug::restoreDebug();
  }

  /**
   * Test.
   *
   * @group errorHandling
   */
  public function testErrorWithDebugOff() {
    TestDebug::turnDebugOff();
    exec( '/usr/local/interworx/bin/php ' .
          '~iworx/html/test/error_tests.php error 2>/dev/null',
          $command_output );
    $command_output = implode( "\n", $command_output );
    $this->assertNotContains( 'InterWorx Error Report',
                              $command_output,
                              'Error report is in output' );
    $this->assertContains( 'A system error has occurred.',
                           $command_output,
                           'Generic error notification missing' );
    TestDebug::restoreDebug();
  }

  /**
   * Test.
   *
   * @group errorHandling
   */
  public function testErrorFatal() {
    TestDebug::turnDebugOn();
    exec( '/usr/local/interworx/bin/php ' .
          '~iworx/html/test/error_tests.php fatal 2>/dev/null',
          $command_output );
    $command_output = implode( "\n", $command_output );
    $this->assertContains( 'InterWorx Error Report',
                           $command_output,
                           'Missing error Report' );
    $this->assertContains( 'FILE: /usr/local/interworx/html/test/error_tests.php',
                           $command_output,
                           'FILE was not in error report' );
    $this->assertRegExp( '/LINE: [0-9]+/',
                         $command_output,
                         'LINE number missing' );
    try {
      $this->assertContains( 'callUnknownFunction()',
                             $command_output,
                             'Error Message Missing' );
    } catch( PHPUnit_Framework_ExpectationFailedException $e ) {
      $this->assertContains( 'callunknownfunction()',
                             $command_output,
                             'Error Message Missing' );
    }
    TestDebug::restoreDebug();
  }

  /**
   * Test.
   *
   * @group errorHandling
   */
  public function testErrorFatalWithDebugOff() {
    TestDebug::turnDebugOff();
    exec( '/usr/local/interworx/bin/php ' .
          '~iworx/html/test/error_tests.php fatal 2>/dev/null',
          $command_output );
    $command_output = implode( "\n", $command_output );
    $this->assertNotContains( 'InterWorx Error Report',
                              $command_output,
                              'Error report is in output' );
    $this->assertContains( 'A system error has occurred.',
                           $command_output,
                           'Generic error notification missing' );
    TestDebug::restoreDebug();
  }

  /**
   * Test.
   *
   * @group errorHandling
   */
  public function testException() {
    TestDebug::turnDebugOn();
    exec( '/usr/local/interworx/bin/php ' .
          '~iworx/html/test/error_tests.php exception 2>/dev/null',
          $command_output );
    $command_output = implode( "\n", $command_output );
    $this->assertContains( 'InterWorx Error Report',
                           $command_output,
                           'Missing error Report' );
    $this->assertContains( 'FILE: /usr/local/interworx/html/test/error_tests.php',
                           $command_output,
                           'FILE was not in error report' );
    $this->assertRegExp( '/LINE: [0-9]+/',
                         $command_output,
                         'LINE number missing' );
    $this->assertContains( 'Uncaught Exception',
                           $command_output,
                           'Error Message Missing' );
    TestDebug::restoreDebug();
  }

  /**
   * Test.
   *
   * @group errorHandling
   */
  public function testExceptionWithDebugOff() {
    TestDebug::turnDebugOff();
    exec( '/usr/local/interworx/bin/php ' .
          '~iworx/html/test/error_tests.php exception 2>/dev/null',
          $command_output );
    $command_output = implode( "\n", $command_output );
    $this->assertNotContains( 'InterWorx Error Report',
                              $command_output,
                              'Error report is in output' );
    $this->assertContains( 'A system error has occurred.',
                           $command_output,
                           'Generic error notification missing' );
    TestDebug::restoreDebug();
  }
}
