<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Paul Oehler <poehler@interworx.com>
 * @date       Feb 24, 2008
 */

/**
 * Test Suite for Ctrl_Nodeworx_Backup.
 *
 * @package    InterWorx
 * @subpackage Tests
 * @group      slow-tests
 */
class Ctrl_Nodeworx_BackupTest extends IWorx_Framework_TestCase {

  const CTRL = 'Ctrl_Nodeworx_Backup';

  static private $_first_setup = false;

  /**
   * Make sure the SiteWorx account has been deleted.
   */
  public function setUp() {
    if( self::$_first_setup === false ) {
      $this->deleteSWAccountIfNecessary();
      $this->addSWAccountIfNecessary();

      self::$_first_setup = true;
    }//end if
  }

  /**
   * Tests assigned vars of index action.
   */
  public function testIndexActionAssignedVars() {
    $NW    = self::getMasterNodeWorx();
    $input = [];

    $Reply  = IW::FC()->routeFromPHP( self::CTRL, 'index', $input, $NW );
    $result = $Reply->getData();

    $this->assertReplySuccessful( $Reply );

    $this->assertArrayHasKey( 'iw_payload', $result );
    $this->assertArrayHasKey( 'iw_resellers', $result );
    $this->assertArrayHasKey( 'iw_nodeworx_id', $result );
  }

  /**
   * Tests the assigned vars of the manage action.
   */
  public function testManageActionAssignedVars() {
    $NW    = self::getMasterNodeWorx();
    $input = array( 'domain' => self::SW_DOMAIN );

    $Reply  = IW::FC()->routeFromPHP( self::CTRL, 'manage', $input, $NW );
    $result = $Reply->getData();

    $this->assertReplySuccessful( $Reply );

    $this->assertArrayHasKey( 'iw_payload', $result );
    $this->assertArrayHasKey( 'iw_domains', $result );
    $this->assertArrayHasKey( 'iw_current_domain', $result );
  }

  /**
   * Tests the query backups action.
   */
  public function testQueryBackupsAction() {
    $NW    = self::getMasterNodeWorx();
    $input = array( 'domain' => self::SW_DOMAIN );
    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'queryBackups', $input, $NW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Tests the query accounts action.
   */
  public function testQueryAccountsAction() {
    $NW    = self::getMasterNodeWorx();
    $input = array( 'reseller' => 'all' );
    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'queryAccounts', $input, $NW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Tests the full backup action.
   */
  public function testFullBackupAction() {
    $NW    = self::getMasterNodeWorx();
    $input = array( 'domains' => array( self::SW_DOMAIN ), 'block'   => '1' );
    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'fullbackup', $input, $NW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Tests the structure only backup action.
   */
  public function testStructureOnlyBackupAction() {
    $NW    = self::getMasterNodeWorx();
    $input = array( 'domains' => array( self::SW_DOMAIN ), 'block'   => '1' );
    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'structureonly', $input, $NW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Tests restore action (restores the first backup it can find).
   *
   * Assumes there is one to restore.
   */
  public function testRestoreAction() {
    $NW = self::getMasterNodeWorx();

    $i = 0;
    do {
      // give it 1 minute...
      sleep( 1 );
      $filenames = $this->_getCompleteBackupFileNames();
      $first     = $filenames[0];
    } while( $i++ < 60 && $first == '' );

    $input = array( 'file'   => $first,
                    'domain' => self::SW_DOMAIN );

    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'restore', $input, $NW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Tests the restoration of all backups.
   */
  public function testRestoringAllBackups() {
    $NW = self::getMasterNodeWorx();

    $filenames      = $this->_getCompleteBackupFileNames();
    $overall_result = true;

    foreach( $filenames as $first ) {
      $input = array( 'file'   => $first,
                      'domain' => self::SW_DOMAIN );

      $Reply          = IW::FC()->routeFromPHP( self::CTRL, 'restore', $input, $NW );
      $overall_result = $Reply->wasSuccessful() && $overall_result;
    }
    $this->assertTrue( $overall_result );
  }

  /**
   * Tests deleting backups (deletes all it can find).
   */
  public function testDeleteActionReturnsTrue() {
    $NW = self::getMasterNodeWorx();

    $filenames = $this->_getCompleteBackupFileNames();

    $input = array( 'domain'  => self::SW_DOMAIN,
                    'backups' => $filenames );

    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'delete', $input, $NW );
    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Helper to get existing backup file names under the account.
   *
   * @return array
   */
  private function _getCompleteBackupFileNames() {
    $domain     = self::SW_DOMAIN;
    $DataSource = new DataSource_Backup_Files( $domain );

    return $DataSource->getValues();
  }
}
