<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Jon Daniel <jdaniel@interworx.com>
 * @date    Jan 28, 2010
 */

/**
 * Test for nodeworx/cluster/nodes.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class Ctrl_Nodeworx_ClusterNodesTest extends IWorx_Cluster_Framework_TestCase {

  const CTRL = 'Ctrl_Nodeworx_ClusterNodes';

  /**
   * Setup some stuff before the test class.
   */
  static public function setUpBeforeClass() {
    // parent::setUpBeforeClass();  no, we don't want them added
    if( TestConfig::get( 'cluster-tests' ) == '1' ) {
      self::_removeAllNodesFromCluster();
    }
  }

  /**
   * Tears down after the test class.
   */
  static public function tearDownAfterClass() {
    parent::tearDownAfterClass();
    if( TestConfig::get( 'cluster-tests' ) == '1' ) {
      self::_addAllNodesToCluster();
    }
  }


  /**
   * Tests the assigned vars.
   */
  public function testIndexAssignedVars() {
    $NodeWorx = self::getMasterNodeWorx();

    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'index', array(), $NodeWorx );
    $data  = $Reply->getData();
    $Form  = $data['iw_form'];

    $this->assertArrayHasKey( 'iw_node', $data );
    $this->assertArrayHasKey( 'iw_nodes', $data );
    $this->assertArrayHasKey( 'iw_vips', $data );
    $this->assertArrayHasKey( 'iw_quorum_ip', $data );
    $this->assertArrayHasKey( 'iw_init', $data );
    $this->assertArrayHasKey( 'iw_form', $data );

    $this->assertTrue( $Form instanceof Form_NW_ClusterNode_Add );
  }

  /**
   * Test.
   */
  public function testAddNodeActionWithTest() {
    $NodeWorx = self::getMasterNodeWorx();
    $server   = $this->_getFirstNode();

    $input = array( 'hostname' => $server['hostname'],
                    'apikey'   => $server['apikey'],
                    'test'     => '1' );

    $Reply = IW::FC()->routeFromPHP( self::CTRL,
                                     'add',
                                     $input,
                                     $NodeWorx );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Test.
   */
  public function testRealAddNodeAction() {
    $NodeWorx = self::getMasterNodeWorx();
    $server   = $this->_getFirstNode();

    $input = array( 'hostname' => $server['hostname'],
                    'apikey'   => $server['apikey'],
                    'test'     => '0' );

    $Reply = IW::FC()->routeFromPHP( self::CTRL,
                                     'add',
                                     $input,
                                     $NodeWorx );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Test.
   */
  public function testDeleteNodeAction() {
    $NodeWorx = self::getMasterNodeWorx();
    $server   = $this->_getFirstNode();

    $input = array( 'nodes' => array( $server['hostname'] ) );

    $Reply = IW::FC()->routeFromPHP( self::CTRL,
                                     'delete',
                                     $input,
                                     $NodeWorx );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Hack.  Gets the first real server in the cluster.
   *
   * @return array
   */
  private function _getFirstNode() {
    $nodes = TestConfig::get( 'cluster-nodes' );
    return $nodes[0];
  }
}