<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Paul Oehler <poehler@interworx.com>
 * @date       Feb 24, 2008
 */

/**
 * Test Suite for Ctrl_Nodeworx_Firewall.
 *
 * @package    InterWorx
 * @subpackage Tests
 */

class Ctrl_Nodeworx_FirewallTest extends IWorx_Framework_TestCase {

  const CTRL = 'Ctrl_Nodeworx_Firewall';

  /**
   * Test assigned vars of index action.
   */
  public function testIndexActionAssignedVars() {
    $NW     = self::getMasterNodeWorx();
    $Reply  = IW::FC()->routeFromPHP( self::CTRL, 'index', array(), $NW );
    $result = $Reply->getData();
    $this->assertArrayHasKey( 'iw_firewall', $result );
    $this->assertArrayHasKey( 'iw_form', $result );

    $forms = $result['iw_form'];

    $this->assertArrayHasKey( 'sob', $forms );
    $class_name = 'Form_NW_Firewall_StartOnBoot';
    $this->assertTrue( $forms['sob'] instanceof $class_name );

    $this->assertArrayHasKey( 'info', $forms );
    $class_name = 'Form_NW_Firewall_Info';
    $this->assertTrue( $forms['info'] instanceof $class_name );

    $this->assertArrayHasKey( 'allowdeny', $forms );
    $class_name = 'Form_NW_Firewall_AllowDenyIps';
    $this->assertTrue( $forms['allowdeny'] instanceof $class_name );
  }

  /**
   * Test adding a system IP/IP range to the whitelist.
   */
  public function testSystemIpWhitelist() {
    $system_ip       = IWorxIPUtil::getMainExternalIP();
    $system_ip_range = substr( $system_ip, 0, strrpos( $system_ip, '.' ) ) . '.0/24';
    $non_system_ip   = '8.8.8.8';

    $NW     = self::getMasterNodeWorx();

    $input  = array( 'trusted_ips' => $system_ip );
    $Reply  = IW::FC()->routeFromPHP( self::CTRL, 'allowDenyIps', $input, $NW );
    $msg = 'trusted_ips allowed a system ip, that is bad';
    $this->assertReplyNotSuccessful( $Reply, $msg );

    $input  = array( 'trusted_ips' => $system_ip_range );
    $Reply  = IW::FC()->routeFromPHP( self::CTRL, 'allowDenyIps', $input, $NW );
    $msg = 'trusted_ips allowed a system ip range, that is bad';
    $this->assertReplyNotSuccessful( $Reply, $msg );

    $input  = array( 'trusted_ips' => $non_system_ip );
    $Reply  = IW::FC()->routeFromPHP( self::CTRL, 'allowDenyIps', $input, $NW );
    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Test.
   */
  public function testAddPortActionErrorsWithNoInput() {
    $NW     = self::getMasterNodeWorx();
    $Reply  = IW::FC()->routeFromPHP( self::CTRL, 'addPort', array(), $NW );
    $result = $Reply->getData();

    $this->assertInternalType( 'array', $result );

    $msg = '##LG_INPUT_FORM_DID_NOT_VALIDATE##. ' .
      '##LG_INPUT_FORM_SEE_DETAILS_BELOW##.';

    $this->assertEquals( $msg, $result[ 0 ] );
  }

  /**
   * Test.
   */
  public function testStartOnBootActionReturnsTrue() {
    $NW    = self::getMasterNodeWorx();
    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'startOnBoot', array(), $NW );
    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Test.
   */
  public function testAllowDenyIpsActionReturnsTrue() {
    $this->setMaxRunningTime( 30 );
    $NW    = self::getMasterNodeWorx();
    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'allowDenyIps', array(), $NW );
    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Test.
   */
  public function testAddingTrustedIpsWorks() {
    $NW    = self::getMasterNodeWorx();
    $input = array( 'trusted_ips' => '10.10.11.12' );

    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'allowDenyIps', $input, $NW );
    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Dataprovider.
   *
   * @return array
   */
  public function getAdvancedTrustedIpFormats() {
    return array( array( '192.168.5.0/24' ),
                  array( 'tcp:in:d=22:s=192.168.2.1' ),
                  array( 'out:d=23:d=192.168.2.1' ),
                  array( 'd=3306:s=192.168.5.0/24' ),
                  array( 'tcp:in:d=2443:s=208.69.120.120' ) );
  }

  /**
   * Test.
   *
   * @param string $sample
   *
   * @dataProvider getAdvancedTrustedIpFormats
   */
  public function testAdvancedTrustedIpFormat( $sample ) {
    $this->setMaxRunningTime( 30 );
    $NW    = self::getMasterNodeWorx();
    $input = array( 'trusted_ips' => $sample );

    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'allowDenyIps', $input, $NW );
    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Test.
   */
  public function testRemovingTrustedIpsWorks() {
    $this->setMaxRunningTime( 30 );
    $NW    = self::getMasterNodeWorx();
    $input = array( 'trusted_ips' => '' );

    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'allowDenyIps', $input, $NW );
    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Test.
   */
  public function testAddedBlockedIpsWorks() {
    $this->setMaxRunningTime( 30 );
    $NW    = self::getMasterNodeWorx();
    $input = array( 'blocked_ips' => '10.10.11.12' );

    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'allowDenyIps', $input, $NW );
    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Test.
   */
  public function testRemovingBlockedIpsWorks() {
    $this->setMaxRunningTime( 30 );
    $NW    = self::getMasterNodeWorx();
    $input = array( 'blocked_ips' => '' );

    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'allowDenyIps', $input, $NW );
    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Test.
   */
  public function testUpdateConfigActionReturnsTrue() {
    $this->setMaxRunningTime( 30 );
    $NW    = self::getMasterNodeWorx();
    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'updateConfig', array(), $NW );
    $this->assertReplySuccessful( $Reply );
  }
}
