<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Paul Oehler <poehler@interworx.com>
 * @date       Feb 24, 2008
 */

/**
 * Test Suite for Ctrl_Nodeworx_Http.
 *
 * @package    InterWorx
 * @subpackage Tests
 */

class Ctrl_Nodeworx_HttpTest extends IWorx_Framework_TestCase {

  const CTRL = 'Ctrl_Nodeworx_Http';

  /**
   * Test assigned vars of index action.
   */
  public function testIndexActionAssignedVars() {
    $NW     = self::getMasterNodeWorx();
    $Reply  = IW::FC()->routeFromPHP( self::CTRL, 'index', array(), $NW );
    $result = $Reply->getData();
    $this->assertArrayHasKey( 'iw_apache', $result );
    $this->assertArrayHasKey( 'iw_form', $result );
  }

  /**
   * Test assigned vars of index action.
   */
  public function testIndexActionAssignedForms() {
    $NW     = self::getMasterNodeWorx();
    $Reply  = IW::FC()->routeFromPHP( self::CTRL, 'index', array(), $NW );
    $result = $Reply->getData();

    $forms = $result['iw_form'];

    $this->assertArrayHasKey( 'sob', $forms );
    $class_name = 'Form_NW_Http_StartOnBoot';
    $this->assertTrue( $forms['sob'] instanceof $class_name );

    $this->assertArrayHasKey( 'restart', $forms );
    $class_name = 'Form_NW_Http_Autorestart';
    $this->assertTrue( $forms['restart'] instanceof $class_name );

    $this->assertArrayHasKey( 'rrd', $forms );
    $class_name = 'Form_NW_Http_RrdGraph';
    $this->assertTrue( $forms['rrd'] instanceof $class_name );

    $this->assertArrayHasKey( 'options', $forms );
    $class_name = 'Form_NW_Http_ServerOptions';
    $this->assertTrue( $forms['options'] instanceof $class_name );
  }

  /**
   * Tests toggling start on boot action.
   */
  public function testStartOnBootActionToggleWorks() {
    $NW = self::getMasterNodeWorx();

    $sob_status = Driver::isServiceStartOnBoot( 'httpd' );
    $input      = array( 'startonboot' => ($sob_status) ? 0 : 1 );
    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'startOnBoot', $input, $NW );
    $this->assertReplySuccessful( $Reply );
    $this->assertTrue( $sob_status != Driver::isServiceStartOnBoot( 'httpd' ) );

    Form::resetInstances();

    $sob_status = Driver::isServiceStartOnBoot( 'httpd' );
    $input      = array( 'startonboot' => ($sob_status) ? 0 : 1 );
    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'startOnBoot', $input, $NW );
    $this->assertReplySuccessful( $Reply );
    $this->assertTrue( $sob_status != Driver::isServiceStartOnBoot( 'httpd' ) );
  }

  /**
   * Tests toggling update rrd action.
   */
  public function testUpdateRrdActionToggleWorks() {
    $NW = self::getMasterNodeWorx();

    $rrd    = Ini::get( Ini::RRD, 'http' );
    $input  = array( 'rrd' => ($rrd) ? 0 : 1 );
    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'updateRrd', $input, $NW );
    $this->assertReplySuccessful( $Reply );
    $this->assertTrue( $rrd != Ini::get( Ini::RRD, 'http' ) );

    $rrd    = Ini::get( Ini::RRD, 'http' );
    $input  = array( 'rrd' => ($rrd) ? 0 : 1 );
    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'updateRrd', $input, $NW );
    $this->assertReplySuccessful( $Reply );
    $this->assertTrue( $rrd != Ini::get( Ini::RRD, 'http' ) );
  }

  /**
   * Tests toggling autorestart action.
   */
  public function testAutorestartActionToggleWorks() {
    $NW = self::getMasterNodeWorx();

    $sim_info = SIM::getInfo();
    $auto     = ( $sim_info['services']['http'] === true ) ? 1 : 0;
    $input    = array( 'apache_autorestart' => ($auto) ? 0 : 1 );
    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'autorestart', $input, $NW );
    $this->assertReplySuccessful( $Reply );
    $sim_info = SIM::getInfo();
    $auto2    = ( $sim_info['services']['http'] === true ) ? 1 : 0;
    $this->assertTrue( $auto != $auto2 );

    $sim_info = SIM::getInfo();
    $auto     = ( $sim_info['services']['http'] === true ) ? 1 : 0;
    $input    = array( 'apache_autorestart' => ($auto) ? 0 : 1 );
    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'autorestart', $input, $NW );
    $this->assertReplySuccessful( $Reply );
    $sim_info = SIM::getInfo();
    $auto2    = ( $sim_info['services']['http'] === true ) ? 1 : 0;
    $this->assertTrue( $auto != $auto2 );
  }

  /**
   * Tests apache server options action with no input (still works).
   */
  public function testApacheUpdateActionWithNoInput() {
    $NW     = self::getMasterNodeWorx();
    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'apacheUpdate', array(), $NW );
    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Tests that the keepalive setting turns on.
   */
  public function testKeepaliveSettingTurnsOn() {
    $NW     = self::getMasterNodeWorx();
    $input  = array( 'keepalive' => '1' );
    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'apacheUpdate', $input, $NW );
    $this->assertReplySuccessful( $Reply );

    $http      = DriverManager::getDefaultDriver( 'http' )->getServerInfo();
    $keepalive = ($http['keepalive'] === 'off') ? '0' : '1';

    $this->assertEquals( 'on', $http['keepalive'] );
  }

  /**
   * Tests that the keepalive setting turns off.
   */
  public function testKeepaliveSettingTurnsOff() {
    $NW     = self::getMasterNodeWorx();
    $input  = array( 'keepalive' => '0' );
    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'apacheUpdate', $input, $NW );
    $this->assertReplySuccessful( $Reply );

    $http      = DriverManager::getDefaultDriver( 'http' )->getServerInfo();
    $keepalive = ($http['keepalive'] === 'off') ? '0' : '1';

    $this->assertEquals( 'off', $http['keepalive'] );
  }

  /**
   * Tests that the force graceful restart turns on.
   */
  public function testForceGracefulRestartTurnsOn() {
    $NW     = self::getMasterNodeWorx();
    $input  = array( 'force_graceful' => '1' );
    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'apacheUpdate', $input, $NW );
    $this->assertReplySuccessful( $Reply );

    $this->assertEquals( '1', Ini::get( Ini::APACHE, 'force_graceful' ) );
  }

  /**
   * Tests that force graceful restart turns off.
   */
  public function testForceGracefulRestartTurnsOff() {
    $NW     = self::getMasterNodeWorx();
    $input  = array( 'force_graceful' => '0' );
    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'apacheUpdate', $input, $NW );
    $this->assertReplySuccessful( $Reply );

    $this->assertEquals( '0', Ini::get( Ini::APACHE, 'force_graceful' ) );
  }

  /**
   * Tests that switching php mode properly modifies configuration.
   */
  public function testUpdatePhpMode() {
    $NW     = self::getMasterNodeWorx();
    $Apache = new Apache();
    $mod    = PHP::MODE_MOD_PHP;

    Ini::set( Ini::APACHE, 'allow_mod_php', '1' );
    Ini::writeAndResetInstance();

    $input  = array( 'php_mode' => $mod );
    $Reply  = IW::FC()->routeFromPHP( self::CTRL, 'updatePhpMode', $input, $NW );
    $this->assertReplySuccessful( $Reply );

    $this->assertTrue( $Apache->isModPhpInstalled() );
    $this->assertFalse( $Apache->isModPhpAndModSuphpInstalled() );

    $mod = ( $mod === PHP::MODE_MOD_PHP ) ? PHP::MODE_MOD_SUPHP : PHP::MODE_MOD_PHP;

    $input = ['php_mode' => $mod];
    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'updatePhpMode', $input, $NW );
    $this->assertReplySuccessful( $Reply );

    $this->assertTrue( $Apache->isModSuPhpInstalled() );
    $this->assertFalse( $Apache->isModPhpAndModSuphpInstalled() );
  }

  /*
    TODO: do something for these actions

    public function testSyntaxAction() {

    }

    public function testServerStatusAction() {

    }

    public function testServerInfoAction() {

    }

    public function testPhpinfoAction() {

    }
  */
}
