<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Jonathan Daniel <jdaniel@interworx.com>
 * @date    Mar 5, 2009
 */

/**
 * Controller test for Ctrl_Nodeworx_Mysql.
 *
 * @package    InterWorx
 * @subpackage Tests
 *
 */
class Ctrl_Nodeworx_MysqlTest extends IWorx_Framework_TestCase {

  const CTRL = 'Ctrl_Nodeworx_Mysql';

  /**
   * Make sure the database is running in the first place.
   */
  public function setUp() {
    if( IW::DM()->getDb()->isRunning() === false ) {
      IW::DM()->getDb()->start();
    }
  }

  /**
   * Makes sure index has the correct assigned vars.
   */
  public function testIndexAssignedVars() {
    $NodeWorx = self::getMasterNodeWorx();

    $input = [];
    $Reply = IW::FC()->routeFromPHP( self::CTRL,
                                     'index',
                                     $input,
                                     $NodeWorx );

    $result = $Reply->getData();

    $this->assertArrayHasKey( 'iw_form', $result );
    $this->assertArrayHasKey( 'iw_mysql', $result );

    $forms = $result['iw_form'];
    $this->assertArrayHasKey( 'rrd', $forms );
    $this->assertArrayHasKey( 'sob', $forms );
    $this->assertArrayHasKey( 'autorestart', $forms );
    $this->assertArrayHasKey( 'options', $forms );
    $this->assertArrayHasKey( 'rootpw', $forms );
  }

  /**
   * Tests that the root password function returns properly.
   */
  public function testRootpassActionWontAcceptBlank() {
    $NW    = self::getMasterNodeWorx();
    $input = array( 'server' => MySQL::getLocalServer()->getId() );

    $Reply = IW::FC()->routeFromPHP( self::CTRL,
                                     'rootpass',
                                     $input,
                                     $NW );

    // we should have gotten a negative result here
    $this->assertFalse( $Reply->wasSuccessful() );
  }

  /**
   * Tests to make sure the rootpass action won't accept bad passwords.
   */
  public function testRootpassActionDoesntAcceptBadPasswords() {
    $NW    = self::getMasterNodeWorx();
    $input = array( 'server'           => MySQL::getLocalServer()->getId(),
                    'password'         => 'abc123',
                    'confirm_password' => 'thisshouldfail' );

    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'rootpass', $input, $NW );
    $this->assertFalse( $Reply->wasSuccessful() );
  }

  /**
   * Test.
   */
  public function testRrdGraphActionReturnsTrue() {
    $NW    = self::getMasterNodeWorx();
    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'rrdGraph', array(), $NW );
    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Test.
   */
  public function testStartOnBootActionReturnsTrue() {
    $NW    = self::getMasterNodeWorx();
    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'startOnBoot', array(), $NW );
    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Test.
   */
  public function testAutoRestartActionReturnsTrue() {
    $NW    = self::getMasterNodeWorx();
    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'autoRestart', array(), $NW );
    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Test.
   */
  public function testOptionsActionReturnsTrue() {
    $NW    = self::getMasterNodeWorx();
    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'options', array(), $NW );
    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Test the restart issue from #4286.
   *
   * Verify that the issue in the web interface doesn't happen when using the driver
   * class directly.
   */
  public function testBug4286ClientMysqlRestartViaDriver() {
    $MySQL = IW::DM()->getDb();
    $MySQL->restart();
    $this->assertTrue( $MySQL->isRunning() );

    `service iworx restartweb`;

    $this->assertTrue( $MySQL->isRunning() );
  }

  /**
   * Test the restart issue from #4286.
   *
   * The problem manifested itself in restarting the client mysql via InterWorx,
   * specifically in the web interface. After that, when the command
   *
   * service iworx restartweb
   *
   * is called, the client MySQL would go down too! And, of course, not come back.
   */
  public function testBug4286ClientMysqlRestartViaAPI() {
    Cluster::routeToNodeSynchronous( '127.0.0.1',
                                     'Ctrl_Nodeworx_Mysql',
                                     'restartCommit',
                                     [] );

    $MySQL = IW::DM()->getDb();
    $this->assertTrue( $MySQL->isRunning() );

    `service iworx restartweb`;

    $this->assertTrue( $MySQL->isRunning() );
  }
}
