<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Paul Oehler <poehler@interworx.com>
 * @date       Feb 24, 2008
 */

/**
 * Test Suite for Ctrl_Nodeworx_NfsMount.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class Ctrl_Nodeworx_NfsMountTest extends IWorx_Framework_TestCase {

  const CTRL = 'Ctrl_Nodeworx_NfsMount';

  /**
   * Setup Function.
   */
  public function setUp() {
    $NW    = self::getMasterNodeWorx();
    $input = array ( 'directory_to_export' => '/tmp/testdir',
                     'ip_allow_list'       => array ( 0 => '127.0.0.1' ),
                     'read_write_access'   => 'ro',
                     'root_user_access'    => 'no_root_squash',
                     'sync_options'        => 'async' );

    if( !IWorxFileSys::fileExists( '/tmp/testdir' ) ) {
      IWorxFileSys::mkdir( '/tmp/testdir', 0777 );
    }
    if( !IWorxFileSys::fileExists( '/mnt/testdir' ) ) {
      IWorxFileSys::mkdir( '/mnt/testdir', 0777 );
    }
    $Reply = IW::FC()->routeFromPHP( 'Ctrl_Nodeworx_NfsExport',
                                     'add',
                                     $input,
                                     $NW );
    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Tear down function.
   */
  public function tearDown() {
    $NW    = self::getMasterNodeWorx();
    $input = array( 'exports' => array( '/tmp/testdir:127.0.0.1' ) );

    $Reply = IW::FC()->routeFromPHP( 'Ctrl_Nodeworx_NfsExport',
                                     'delete',
                                     $input,
                                     $NW );

    $this->assertReplySuccessful( $Reply );
    IWorxFileSys::rm( '/tmp/testdir', true, true );
    IWorxFileSys::rm( '/mnt/testdir', true, true );
  }

  /**
   * Test assigned vars of index action.
   */
  public function testIndexActionAssignedVars() {
    $NW     = self::getMasterNodeWorx();
    $Reply  = IW::FC()->routeFromPHP( self::CTRL, 'index', array(), $NW );
    $result = $Reply->getData();
    $this->assertArrayHasKey( 'iw_nfs', $result );
    $this->assertArrayHasKey( 'iw_form', $result );

    $class_name = 'Form_NW_NfsMount_Add';
    $this->assertTrue( $result['iw_form'] instanceof $class_name );
  }

  /**
   * Test add action with no input.
   */
  public function testAddActionErrorsWithNoInput() {
    $NW     = self::getMasterNodeWorx();
    $Reply  = IW::FC()->routeFromPHP( self::CTRL, 'add', array(), $NW );
    $result = $Reply->getData();
    $this->assertIsFormErrorArray( $result );
  }

  /**
   * Test add action protocols.
   */
  public function testAddActionProtocols() {
    $NW    = self::getMasterNodeWorx();
    $input = array ( 'host' => '127.0.0.1',
                     'remote_directory' => '/tmp/testdir',
                     'mount_point' => '/mnt/testdir',
                     'protocol' => 'tcp',
                     'mount_type' => 'hard',
                     'remount_options' => 'fg',
                     'interrupt_handling' => 'intr' );

    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'add', $input, $NW );
    $this->assertReplySuccessful( $Reply );

    $mounts = NFS::getMountList();
    $tcp    = array_key_exists( 'tcp', $mounts['/mnt/testdir']['options'] );
    $udp    = array_key_exists( 'udp', $mounts['/mnt/testdir']['options'] );
    $this->assertTrue( $tcp );
    $this->assertFalse( $udp );

    $inputdel = array( 'mounts' => array( '/mnt/testdir' ) );
    $Reply    = IW::FC()->routeFromPHP( self::CTRL, 'delete', $inputdel, $NW );
    $this->assertReplySuccessful( $Reply );

    $input['protocol'] = 'udp';
    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'add', $input, $NW );
    $this->assertReplySuccessful( $Reply );

    $mounts = NFS::getMountList();
    $udp    = array_key_exists( 'udp', $mounts['/mnt/testdir']['options'] );
    $tcp    = array_key_exists( 'tcp', $mounts['/mnt/testdir']['options'] );
    $this->assertTrue( $udp );
    $this->assertFalse( $tcp );

    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'delete', $inputdel, $NW );
    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Test add action.
   */
  public function testAddAction() {
    $NW    = self::getMasterNodeWorx();
    $input = array ( 'host' => '127.0.0.1',
                     'remote_directory' => '/tmp/testdir',
                     'mount_point' => '/mnt/testdir',
                     'protocol' => 'tcp',
                     'mount_type' => 'hard',
                     'remount_options' => 'fg',
                     'interrupt_handling' => 'intr' );

    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'add', $input, $NW );
    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Test delete with no input.
   */
  public function testDeleteActionErrorsWithNoInput() {
    $NW     = self::getMasterNodeWorx();
    $Reply  = IW::FC()->routeFromPHP( self::CTRL, 'delete', array(), $NW );
    $result = $Reply->getData();
    $this->assertIsFormErrorArray( $result );
  }

  /**
   * Test delete action.
   *
   * @depends testAddAction
   */
  public function testDeleteAction() {
    $NW    = self::getMasterNodeWorx();
    $input = array( 'mounts' => array( '/mnt/testdir' ) );

    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'delete', $input, $NW );
    $this->assertReplySuccessful( $Reply );
  }
}
