<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Paul Oehler <poehler@interworx.com>
 * @date       Feb 24, 2008
 */

/**
 * Test Suite for Ctrl_Nodeworx_Packages
 *
 * @package    InterWorx
 * @subpackage Tests
 */

class Ctrl_Nodeworx_ResellerTest extends IWorx_Framework_TestCase {

  const CTRL = 'Ctrl_Nodeworx_Reseller';

  /**
   * Tests the add action.
   */
  public function testAddAction() {
    $this->deleteResellerAccountIfNecessary();
    $NW    = self::getMasterNodeWorx();
    $input = $this->_getResellerInput();
    $Reply = IW::FC()->routeFromPHP( self::CTRL,
                                     'add',
                                     $input,
                                     $NW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Tests add action again (expecting fail).
   *
   * @depends testAddAction
   */
  public function testAddActionWithSameInput() {
    $NW    = self::getMasterNodeWorx();
    $input = $this->_getResellerInput();
    $Reply = IW::FC()->routeFromPHP( self::CTRL,
                                     'add',
                                     $input,
                                     $NW );

    $msg = 'we could add the same user twice?';

    $this->assertReplyNotSuccessful( $Reply, $msg );
  }

  /**
   * Tests edit action.
   *
   * @depends testAddAction
   */
  public function testEditAction() {
    $RSL   = self::getResellerNodeWorx();
    $NW    = self::getMasterNodeWorx();
    $input = array( 'reseller_id' => $RSL->getId() );
    $Reply = IW::FC()->routeFromPHP( self::CTRL,
                                     'edit',
                                     $input,
                                     $NW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Tests changing the user password to prevent regression.
   *
   * @depends testEditAction
   */
  public function testEditChangePassword() {
    $RSL       = self::getResellerNodeWorx();
    $NW        = self::getMasterNodeWorx();
    $rsl_email = $RSL->getWorkingUser()->getEmail();

    $input = array( 'reseller_id'      => $RSL->getId(),
                    'password'         => 'new_password',
                    'confirm_password' => 'new_password' );

    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'edit', $input, $NW );

    $login_ok = $RSL->login( $rsl_email, 'new_password' );

    $this->assertTrue( $login_ok, $RSL->getLastError() );
  }


  /**
   * Tests delete action.
   */
  public function testDeleteAction() {
    $RSL   = self::getResellerNodeWorx();
    $NW    = self::getMasterNodeWorx();
    $input = array( 'reseller_id' => $RSL->getId() );
    $Reply = IW::FC()->routeFromPHP( self::CTRL,
                                     'delete',
                                     $input,
                                     $NW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Tests delete actually deleted the reseller.
   *
   * @depends testDeleteAction
   */
  public function testDeleteActionWorked() {
    $RSL = self::getResellerNodeWorx();
    $this->assertEquals( '-1', $RSL->getId(), 'reseller delete did not delete' );
  }

  /**
   * Tests that the index information has the user list.
   */
  public function testIndexAction() {
    $NW    = $this->getMasterNodeWorx();
    $Reply = IW::FC()->routeFromPHP( self::CTRL,
                                     'index',
                                     array(),
                                     $NW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Test to make sure the list action returns successfully.
   */
  public function testListResellersAction() {
    $NW    = $this->getMasterNodeWorx();
    $Reply = IW::FC()->routeFromPHP( self::CTRL,
                                     'listResellers',
                                     array(),
                                     $NW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Tests that the editable ids ctrl action works.
   */
  public function testListEditableIdsAction() {
    $NW    = $this->getMasterNodeWorx();
    $Reply = IW::FC()->routeFromPHP( self::CTRL,
                                     'listEditableIds',
                                     array(),
                                     $NW );

    $this->assertReplySuccessful( $Reply );

  }

  /**
   * Tests that the deletable ids ctrl action works.
   */
  public function testListDeletableIdsAction() {
    $NW    = $this->getMasterNodeWorx();
    $Reply = IW::FC()->routeFromPHP( self::CTRL,
                                     'listDeletableIds',
                                     array(),
                                     $NW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Tests that the list ids ctrl action works.
   */
  public function testListIdsAction() {
    $NW    = $this->getMasterNodeWorx();
    $Reply = IW::FC()->routeFromPHP( self::CTRL,
                                     'listIds',
                                     array(),
                                     $NW );

    $this->assertReplySuccessful( $Reply );
  }
}
