<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Paul Oehler <poehler@interworx.com>
 * @date       Feb 24, 2008
 */

/**
 * Test Suite for Ctrl_Nodeworx_Siteworx.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class Ctrl_Nodeworx_ShellTest extends IWorx_Framework_TestCase {

  const CTRL = 'Ctrl_Nodeworx_Shell';

  const SHELL        = '/bin/bash';
  const BASH_HISTORY = 'sample text goes here';

  /**
   * Setup the test.
   */
  public function setUp() {
    self::addSWAccountIfNecessary();
  }

  /**
   * Tests that listing the siteworx users works.
   */
  public function testListSiteworxUsersAction() {
    $NW     = self::getMasterNodeWorx();
    $input  = [];
    $action = 'listSiteworxUsers';
    $Reply  = IW::FC()->routeFromPHP( self::CTRL, $action, $input, $NW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Tests that listing the system users works.
   */
  public function testListSystemUsersAction() {
    $NW     = self::getMasterNodeWorx();
    $input  = [];
    $action = 'listSiteworxUsers';
    $Reply  = IW::FC()->routeFromPHP( self::CTRL, $action, $input, $NW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Tests that changing the default shell actually works.
   */
  public function testShellOptionsAction() {
    $NW     = self::getMasterNodeWorx();
    $input  = array( 'shell' => '/bin/bash' );
    $action = 'shellOptions';
    $Reply  = IW::FC()->routeFromPHP( self::CTRL, $action, $input, $NW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Tests that changing a users shell actually works.
   */
  public function testChangeShellAction() {
    $NW     = self::getMasterNodeWorx();
    $input  = array( 'shell' => '/bin/bash',
                     'users' => array( self::SW_UNIQNAME ) );
    $action = 'changeshell';
    $Reply  = IW::FC()->routeFromPHP( self::CTRL, $action, $input, $NW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Tests shell user password changing.
   */
  public function testPasswordActionReturnsTrue() {
    $NW    = self::getMasterNodeWorx();
    $input = array( 'user' => self::SW_UNIQNAME,
                    'password' => self::SW_PASS,
                    'confirm_password' => self::SW_PASS );
    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'password', $input, $NW );
    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Tests result of shell user password changing.
   */
  public function testPasswordActionWorked() {
    $shadow  = $this->_getShadowPassword();
    $hashed_pass = crypt( self::SW_PASS, $shadow );
    $this->assertEquals( $shadow, $hashed_pass, 'Shell password failed to be set' );
  }

  /**
   * Tests that the enable action returns true.
   */
  public function testEnableActionReturnsTrue() {
    $NW     = self::getMasterNodeWorx();
    $input  = array( 'users' => self::SW_UNIQNAME );
    $action = 'enable';
    $Reply  = IW::FC()->routeFromPHP( self::CTRL, $action, $input, $NW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Tests that disable actually worked.
   *
   * @depends testEnableActionReturnsTrue
   */
  public function testEnableActuallyWorked() {
    $passwd = ShellAccount::getCombinedPasswd();
    $user   = self::SW_UNIQNAME;

    $this->assertFalse( $passwd[ $user ]['disabled'] );
  }

  /**
   * Tests that the disable action returns true.
   */
  public function testDisableActionReturnsTrue() {
    $NW     = self::getMasterNodeWorx();
    $input  = array( 'users' => self::SW_UNIQNAME );
    $action = 'disable';
    $Reply  = IW::FC()->routeFromPHP( self::CTRL, $action, $input, $NW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Tests that disable actually worked.
   *
   * @depends testDisableActionReturnsTrue
   */
  public function testDisableActuallyWorked() {
    $passwd = ShellAccount::getCombinedPasswd();
    $user   = self::SW_UNIQNAME;

    $this->assertTrue( $passwd[ $user ]['disabled'] );
  }

  /**
   * Cleans up.
   */
  public function testCleanup() {
    $this->deleteSWAccountIfNecessary();
  }

  /**
   * Write some contents to shell user bash_history file.
   */
  private function _addSomeShellUserHistory() {
    file_put_contents( '/home/addedbyt/.bash_history', self::BASH_HISTORY );
  }

  /**
   * Gets the crypted password from the shadow file.
   *
   * @return string
   */
  private function _getShadowPassword() {
    $shadowfile = file( '/etc/shadow' );
    foreach( $shadowfile as $line ) {
      $line = trim( $line );
      list( $username, $password, $rest ) = explode( ':', $line, 3 );
      if( $username == self::SW_UNIQNAME ) {
        break;
      }
    }
    return $password;
  }

}
