<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Paul Oehler <poehler@interworx.com>
 * @date       Feb 24, 2008
 */

/**
 * Test Suite for Ctrl_Nodeworx_Siteworx
 *
 * @package    InterWorx
 * @subpackage Tests
 * @group      slow-tests
 */
class Ctrl_Nodeworx_SiteworxTest extends IWorx_Framework_TestCase {

  const CTRL = 'Ctrl_Nodeworx_Siteworx';

  const NEW_NICKNAME   = 'New Nickname Here';
  const NEW_BILLINGDAY = '13';

  static private $_first_setup = false;
  static private $_first_teardown = false;

  /**
   * Make sure the SiteWorx account has been deleted.
   */
  public function setUp() {
    if( self::$_first_setup === false ) {
      $this->deleteSWAccountIfNecessary();

      self::$_first_setup = true;
    }
  }

  /**
   * Make sure the SiteWorx account has been deleted.
   */
  public function tearDown() {
    if( self::$_first_teardown === false ) {
      $this->deleteSWAccountIfNecessary();

      self::$_first_teardown = true;
    }
  }

  /**
   * Tests the index action.
   */
  public function testIndexAction() {
    $NW    = self::getMasterNodeWorx();
    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'index', array(), $NW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Tests the list action.
   */
  public function testListAction() {
    $NW    = self::getMasterNodeWorx();
    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'listAccounts', array(), $NW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Tests the search action.
   */
  public function testSearchAction() {
    $NW    = self::getMasterNodeWorx();
    $input = array( 'reseller' => $NW->getId() );
    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'search', $input, $NW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Tests the listMasterDomains action.
   */
  public function testListMasterDomainsAction() {
    $NW    = self::getMasterNodeWorx();
    $Reply = IW::FC()->routeFromPHP( self::CTRL,
                                     'listMasterDomains',
                                     array(),
                                     $NW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Tests the listFreeIps action.
   */
  public function testListFreeIpsAction() {
    $NW    = self::getMasterNodeWorx();
    $Reply = IW::FC()->routeFromPHP( self::CTRL,
                                     'listFreeIps',
                                     array(),
                                     $NW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Tests the listDedicatedFreeIps action.
   */
  public function testListDedicatedFreeIpsAction() {
    $NW    = self::getMasterNodeWorx();
    $Reply = IW::FC()->routeFromPHP( self::CTRL,
                                     'listDedicatedFreeIps',
                                     array(),
                                     $NW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Tests the listSharedFreeIps action.
   */
  public function testListSharedFreeIpsAction() {
    $NW    = self::getMasterNodeWorx();
    $Reply = IW::FC()->routeFromPHP( self::CTRL,
                                     'listSharedFreeIps',
                                     array(),
                                     $NW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Tests the add action.
   */
  public function testAddAction() {
    $NW    = self::getMasterNodeWorx();
    $input = self::_getSWAccountInput();
    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'add', $input, $NW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Tests that the siteworx account was added with all the various params.
   *
   * @depends testAddAction
   */
  public function testAddActionWorkedCorrectly() {
    $SW    = $this->getSWAccount();
    $input = self::_getSWAccountInput();

    extract( $input );

    $this->assertEquals( $master_domain, $SW->getWorkingDomainName() );
    $this->assertEquals( $master_domain_ipv4, $SW->getIP() );
    $this->assertEquals( $uniqname, $SW->getUniqname() );
    $this->assertEquals( $nickname, $SW->getWorkingUser()->getNickName() );
    $this->assertEquals( $email, $SW->getWorkingUser()->getEmail() );
  }

  /**
   * Tests that adding the same account will not work.
   *
   * @depends testAddAction
   */
  public function testAddingSameAccountFails() {
    $NW    = self::getMasterNodeWorx();
    $input = self::_getSWAccountInput();
    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'add', $input, $NW );

    $msg = 'could add the same account twice :(';

    $this->assertReplyNotSuccessful( $Reply, $msg );
  }

  /**
   * Tests the toggle favorite actions.
   *
   * @depends testAddAction
   */
  public function testToggleFavoriteAction() {
    $NW    = self::getMasterNodeWorx();
    $input = array( 'domain' => self::SW_DOMAIN );
    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'toggleFavorite', $input, $NW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Tests to make sure is favorite is false.
   *
   * @depends testToggleFavoriteAction
   */
  public function testFavoriteIsTrue() {
    $user_id = self::getMasterNodeWorx()->getWorkingUser()->getId();
    $domain  = self::SW_DOMAIN;

    $is_favorite = Model_Siteworx::isFavorite( $user_id, $domain );

    $this->assertTrue( $is_favorite );
  }

  /**
   * Tests the toggle favorite actions.
   *
   * @depends testToggleFavoriteAction
   */
  public function testToggleFavoriteAgain() {
    $this->testToggleFavoriteAction();
  }

  /**
   * Tests to make sure is favorite is false.
   *
   * @depends testToggleFavoriteAgain
   */
  public function testFavoriteIsFalse() {
    $user_id = self::getMasterNodeWorx()->getWorkingUser()->getId();
    $domain  = self::SW_DOMAIN;

    $is_favorite = Model_Siteworx::isFavorite( $user_id, $domain );

    $this->assertFalse( $is_favorite );
  }

  /**
   * Tests the edit action.
   *
   * @depends testAddAction
   */
  public function testEditAction() {
    $NW    = self::getMasterNodeWorx();
    $input = array( 'domain' => self::SW_DOMAIN );
    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'edit', $input, $NW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Tests that the empty edit action didn't change anything.
   *
   * @depends testEditAction
   */
  public function testEditActionChangedNothing() {
    Form::resetInstances();
    IW::Env()->addInput( 'domain', self::SW_DOMAIN );

    $Form = Form::getInstance( 'Form_NW_Siteworx_Edit' );
    $this->assertTrue( $Form->validateForm() );

    $ip = self::_getFirstSharedIP();

    $this->assertEquals( self::SW_DOMAIN, $Form->getValue( 'master_domain' ) );
    $this->assertEquals( $ip, $Form->getValue( 'master_domain_ipv4' ) );
    $this->assertEquals( self::SW_UNIQNAME, $Form->getValue( 'uniqname' ) );
    $this->assertEquals( self::SW_NICK, $Form->getValue( 'nickname' ) );
    $this->assertEquals( self::SW_EMAIL, $Form->getValue( 'email' ) );

    $options = Option::getOptionList();

    $unlimited = Ini::get( Ini::CONSTANTS, 'unlimited' );
    foreach( $options as $opt ) {
      if( $opt['displayGroup'] !== '1' ) {
        continue;
      }
      $input_name = substr( $opt['name'], 3 );
      $this->assertEquals( $unlimited, $Form->getValue( $input_name ) );
    }

    foreach( $options as $opt ) {
      $input_name = substr( $opt['name'], 3 );
      if( $opt['displayGroup'] !== '2' ) {
        continue;
      }
      if( $input_name != 'OPT_SUPHP' && $input_name != 'OPT_SSI' ) {
        $this->assertEquals( 1, $Form->getValue( $input_name ) );
      } else {
        $this->assertEquals( 0, $Form->getValue( $input_name ) );
      }
    }
  }

  /**
   * Tests that edit works when the nickname is changed.
   *
   * @depends testEditAction
   */
  public function testEditActionChangeNicknameReturnsTrue() {
    $NW    = self::getMasterNodeWorx();
    $input = array( 'domain'   => self::SW_DOMAIN,
                    'nickname' => self::NEW_NICKNAME );

    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'edit', $input, $NW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Tests edit action, nickname change worked.
   *
   * @depends testEditActionChangeNicknameReturnsTrue
   */
  public function testEditActionChangeNicknameWorked() {
    Form::resetInstances();
    IW::Env()->addInput( 'domain', self::SW_DOMAIN );

    $Form = Form::getInstance( 'Form_NW_Siteworx_Edit' );
    $this->assertTrue( $Form->validateForm() );

    $this->assertEquals( self::NEW_NICKNAME, $Form->getValue( 'nickname' ) );
  }

  /**
   * Tests that edit works when the billing day is changed.
   *
   * @depends testEditAction
   */
  public function testEditActionChangeBillingDayReturnsTrue() {
    $NW    = self::getMasterNodeWorx();
    $input = array( 'domain'      => self::SW_DOMAIN,
                    'billing_day' => self::NEW_BILLINGDAY );

    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'edit', $input, $NW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Tests edit action, billing day change worked.
   *
   * @depends testEditActionChangeBillingDayReturnsTrue
   */
  public function testEditActionChangeBillingDayWorked() {
    Form::resetInstances();
    IW::Env()->addInput( 'domain', self::SW_DOMAIN );

    $Form = Form::getInstance( 'Form_NW_Siteworx_Edit' );
    $this->assertTrue( $Form->validateForm() );

    $this->assertEquals( self::NEW_BILLINGDAY, $Form->getValue( 'billing_day' ) );
  }

  /**
   * Test.
   *
   * @depends testAddAction
   */
  public function testEditActionChangeStorageQuota() {
    $quota_value = '1000';
    $NW    = self::getMasterNodeWorx();
    $input = array( 'domain'   => self::SW_DOMAIN,
                    'OPT_STORAGE' => $quota_value );

    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'edit', $input, $NW );

    $this->assertReplySuccessful( $Reply );
    $ov = Ini::get( Ini::QUOTA, 'quota-overflow-pct' );
    $soft = IWorxFileSys::getQuota( self::SW_UNIQNAME, 'soft' );
    $hard = IWorxFileSys::getQuota( self::SW_UNIQNAME, 'hard' );

    $quota = $quota_value * 1024;
    $this->assertEquals( $quota, $soft );
    $this->assertEquals( $quota + ($quota * ($ov/100)), $hard );
  }

  /**
   * Tests bulk deactivate action.
   *
   * @depends testAddAction
   */
  public function testBulkdeactivateAction() {
    $NW    = self::getMasterNodeWorx();
    $input = array( 'domains' => array( self::SW_DOMAIN ) );
    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'bulkdeactivate', $input, $NW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Tests results of bulk deactivate action.
   *
   * @depends testBulkdeactivateAction
   */
  public function testBulkdeactivateActionWorked() {
    $NW = self::getMasterNodeWorx();
    $SW = new SiteWorx( self::SW_DOMAIN );

    $this->assertEquals( 'inactive', $SW->getStatus() );
  }

  /**
   * Tests bulk activate action.
   *
   * @depends testAddAction
   * @depends testBulkdeactivateAction
   */
  public function testBulkactivateAction() {
    $NW    = self::getMasterNodeWorx();
    $input = array( 'domains' => array( self::SW_DOMAIN ) );
    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'bulkactivate', $input, $NW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Tests results of bulk activate action.
   *
   * @depends testBulkactivateAction
   */
  public function testBulkactivateActionWorked() {
    $NW = self::getMasterNodeWorx();
    $SW = new SiteWorx( self::SW_DOMAIN );

    $this->assertEquals( 'active', $SW->getStatus() );
  }

  /**
   * Tests bulk delete action.
   *
   * @depends testAddAction
   */
  public function testBulkdeleteAction() {
    $NW    = self::getMasterNodeWorx();
    $input = array( 'domains' => array( self::SW_DOMAIN ) );
    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'bulkdelete', $input, $NW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Tests result of bulk delete action.
   *
   * @depends testAddAction
   * @depends testBulkdeleteAction
   */
  public function testBulkdeleteActionWorked() {
    $NW = self::getMasterNodeWorx();
    $SW = new SiteWorx( self::SW_DOMAIN );

    $this->assertFalse( $SW->exists() );
  }

  /**
   * Tests delete action.
   */
  public function testDeleteAction() {
    self::addSWAccountIfNecessary();

    $NW    = self::getMasterNodeWorx();
    $input = array( 'domain' => self::SW_DOMAIN );
    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'delete', $input, $NW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Tests results of delete action.
   *
   * @depends testDeleteAction
   */
  public function testDeleteActionWorked() {
    $NW = self::getMasterNodeWorx();
    $SW = new SiteWorx( self::SW_DOMAIN );

    $this->assertFalse( $SW->exists() );
  }

  /**
   * Tests that the delete action removes databases.
   */
  public function testDeleteActionRemovesDatabases() {
    self::addSWAccountIfNecessary();

    $input = array( 'name' => 'sampledb' );
    $SW    = $this->getSWAccount();
    $Reply = IW::FC()->routeFromPHP( 'Ctrl_Siteworx_MysqlDb',
                                     'add',
                                     $input,
                                     $SW );

    $this->assertReplySuccessful( $Reply );

    $NW    = self::getMasterNodeWorx();
    $input = array( 'domain' => self::SW_DOMAIN );
    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'delete', $input, $NW );

    $this->assertReplySuccessful( $Reply );
    $this->assertFalse( is_dir( '/var/lib/mysql/addedbyt_sampledb' ) );
  }

  /**
   * Test.
   */
  public function testAddOnDedicatedIPMakesIPUnavailable() {
    $NW    = self::getMasterNodeWorx();
    $input = self::_getSWAccountInput();

    $input['master_domain_ipv4'] = self::_getFirstAvailableDedicatedIP();

    IW::FC()->routeFromPHP( self::CTRL, 'add', $input, $NW );

    $available_ips = $NW->getUnusedIPList();

    $this->assertFalse( isset( $available_ips[ $input['master_domain_ipv4'] ] ),
                        'Dedicated IP used but still shown as available!' );
  }

  /**
   * Test.
   */
  public function testEditIPAffectsPointerDomains() {
    $NW    = self::getMasterNodeWorx();
    $SW    = $this->getSWAccount();
    $input = array( 'domain' => self::SW_POINTER_DOMAIN,
                    'points_to' => self::SW_DOMAIN );

    $Reply = IW::FC()->routeFromPHP( 'Ctrl_Siteworx_DomainsPointer',
                                     'add',
                                     $input,
                                     $SW );

    $this->assertReplySuccessful( $Reply );
    $pointers = $SW->getMasterDomain()->getChildDomains();

    $ip1 = $SW->getMasterDomain()->getIPv4Address();
    $ip2 = $pointers[0]->getIPv4Address();

    $this->assertEquals( $ip1, $ip2 );

    $new_ip   = self::_getFirstSharedIP();
    $IWorx_IP = IWorx_IP::factoryFromDatabase( $new_ip );

    $SW->getMasterDomain()->changeIPv4AddressTo( $IWorx_IP );
    $pointers = $SW->getMasterDomain()->getChildDomains();

    $ip1 = $SW->getMasterDomain()->getIPv4Address();
    $ip2 = $pointers[0]->getIPv4Address();

    $this->assertEquals( $ip1, $ip2 );
  }

  /**
   * Test changing to an IPv6 subpool.
   */
  public function testChangeToIPv6Subpool() {
    self::deleteSWAccountIfNecessary();

    $SW = $this->getSWAccount();
    $NW = new NodeWorx( NodeWorx::MASTER_ID );

    $this->assertEquals( '0', $SW->getIPv6Pool() );

    $Pool = $this->_setupIPv6Pool();

    $parent_pool = $Pool->getRangeStartWithCIDR();

    $input = array( 'domain' => $SW->getMasterDomainName(),
                    'ipv6_pool' => $parent_pool );

    $Reply = IW::FC()->routeFromPHP( 'Ctrl_Nodeworx_Siteworx',
                                     'editCommit',
                                     $input,
                                     $NW );

    $this->assertReplySuccessful( $Reply );
    $this->assertContains( '/112', $SW->getIPv6Pool() );
  }

  /**
   * Sets up the IPv6 Pool.
   *
   * @return IPv6_Pool
   */
  private function _setupIPv6Pool() {
    $Pool = IPv6_Pool::factoryFromAddress( '2001:db8::/96' );

    $Pool->setNickname( 'testrange' );
    $Pool->setDistributionPolicy( IPv6_Pool::POLICY_SEQUENTIAL );
    $Pool->setNetmaskCIDR( '128' );
    $Pool->setDevice( 'eth0' );
    $Pool->setSubpoolSize( 112 );

    $Pool->save();

    return $Pool;
  }
}
