<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Paul Oehler <poehler@interworx.com>
 * @date       Feb 24, 2008
 */

/**
 * Test Suite for Ctrl_Nodeworx_Sshd.
 *
 * @package    InterWorx
 * @subpackage Tests
 */

class Ctrl_Nodeworx_SshdTest extends IWorx_Framework_TestCase {

  const CTRL = 'Ctrl_Nodeworx_Sshd';

  /**
   * Test assigned vars of index action.
   */
  public function testIndexActionAssignedVars() {
    $NW     = self::getMasterNodeWorx();
    $Reply  = IW::FC()->routeFromPHP( self::CTRL, 'index', array(), $NW );
    $result = $Reply->getData();
    $this->assertArrayHasKey( 'iw_sshd', $result );
    $this->assertArrayHasKey( 'iw_shell', $result );
    $this->assertArrayHasKey( 'iw_form', $result );

    $forms = $result['iw_form'];

    $this->assertArrayHasKey( 'autorestart', $forms );
    $class_name = 'Form_NW_Sshd_Autorestart';
    $this->assertTrue( $forms['autorestart'] instanceof $class_name );

    $this->assertArrayHasKey( 'rrd', $forms );
    $class_name = 'Form_NW_Sshd_RrdGraph';
    $this->assertTrue( $forms['rrd'] instanceof $class_name );

    $this->assertArrayHasKey( 'startonboot', $forms );
    $class_name = 'Form_NW_Sshd_StartOnBoot';
    $this->assertTrue( $forms['startonboot'] instanceof $class_name );

    $this->assertArrayHasKey( 'options', $forms );
    $class_name = 'Form_NW_Sshd_Options';
    $this->assertTrue( $forms['options'] instanceof $class_name );
  }

  /**
   * Test.
   */
  public function testTerminateActionErrorsWithNoInput() {
    $NW     = self::getMasterNodeWorx();
    $Reply  = IW::FC()->routeFromPHP( self::CTRL, 'terminate', array(), $NW );
    $result = $Reply->getData();
    $this->assertInternalType( 'array', $result );
    $this->assertEquals( '##LG_INPUT_FORM_DID_NOT_VALIDATE##. ' .
                         '##LG_INPUT_FORM_SEE_DETAILS_BELOW##.', $result[ 0 ] );
  }

  /**
   * Test.
   */
  public function testStartOnBootActionErrorsWithNoInput() {
    $NW     = self::getMasterNodeWorx();
    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'startOnBoot', array(), $NW );
    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Test.
   */
  public function testUpdateRrdActionErrorsWithNoInput() {
    $NW     = self::getMasterNodeWorx();
    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'updateRrd', array(), $NW );
    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Test.
   */
  public function testUpdateConfigActionReturnsTrueWithNoInput() {
    $NW     = self::getMasterNodeWorx();
    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'updateConfig', array(), $NW );
    $this->assertReplySuccessful( $Reply );
  }


}
