<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Paul Oehler <poehler@interworx.com>
 * @date       Feb 24, 2008
 */

/**
 * Test Suite for Ctrl_Nodeworx_Backup
 *
 * @package    InterWorx
 * @subpackage Tests
 */

class Ctrl_Nodeworx_UsersTest extends IWorx_Framework_TestCase {

  const CTRL = 'Ctrl_Nodeworx_Users';

  const SW_SECONDARY_NICK_CHANGE = 'Changed Nickname';

  /**
   * Tests that the index information has the user list
   */
  public function testIndexAction() {
    $NW    = $this->getMasterNodeWorx();
    $Reply = IW::FC()->routeFromPHP( self::CTRL,
                                     'index',
                                     array(),
                                     $NW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Test to make sure the list action returns successfully.
   */
  public function testListAction() {
    $NW    = $this->getMasterNodeWorx();
    $Reply = IW::FC()->routeFromPHP( self::CTRL,
                                     'list',
                                     array(),
                                     $NW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Test to make sure the list action returns successfully.
   */
  public function testListEditableAction() {
    $NW    = $this->getMasterNodeWorx();
    $Reply = IW::FC()->routeFromPHP( self::CTRL,
                                     'listEditable',
                                     array(),
                                     $NW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Test to make sure the list action returns successfully.
   */
  public function testlistDeletableAction() {
    $NW    = $this->getMasterNodeWorx();
    $Reply = IW::FC()->routeFromPHP( self::CTRL,
                                     'listDeletable',
                                     array(),
                                     $NW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Test to make sure the list action returns successfully.
   */
  public function testListMasterUserAction() {
    $NW    = $this->getMasterNodeWorx();
    $Reply = IW::FC()->routeFromPHP( self::CTRL,
                                     'listWorkingUser',
                                     array(),
                                     $NW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Test to make sure the list action returns successfully.
   */
  public function testListWorkingUserAction() {
    $NW    = $this->getMasterNodeWorx();
    $Reply = IW::FC()->routeFromPHP( self::CTRL,
                                     'listMasterUser',
                                     array(),
                                     $NW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Tests the isReseller action.
   */
  public function testIsResellerAction() {
    $NW    = $this->getMasterNodeWorx();
    $Reply = IW::FC()->routeFromPHP( self::CTRL,
                                     'isResellerCommit',
                                     array(),
                                     $NW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Tests adding a secondary user
   */
  public function testAddActionReturnsTrue() {
    $NW = $this->getMasterNodeWorx();

    $input = array( 'nickname'         => self::SW_SECONDARY_NICK,
                    'email'            => self::SW_SECONDARY_EMAIL,
                    'password'         => self::SW_SECONDARY_PASS,
                    'confirm_password' => self::SW_SECONDARY_PASS,
                    'perms'            => array( 'LOGIN' ) );

    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'add', $input, $NW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Tests adding a secondary user again
   */
  public function testAddActionFailsWithDuplicate() {
    $NW = $this->getMasterNodeWorx();

    $input = array( 'nickname'         => self::SW_SECONDARY_NICK,
                    'email'            => self::SW_SECONDARY_EMAIL,
                    'password'         => self::SW_SECONDARY_PASS,
                    'confirm_password' => self::SW_SECONDARY_PASS,
                    'perms'            => array( 'LOGIN' ) );

    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'add', $input, $NW );

    $this->assertFalse( $Reply->wasSuccessful() );
  }

  /**
   * Tests that the edit action returns true
   */
  public function testEditActionReturnsTrue() {
    $NW    = $this->getMasterNodeWorx();
    $input = array( 'user'     => self::SW_SECONDARY_EMAIL,
                    'nickname' => self::SW_SECONDARY_NICK_CHANGE );

    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'edit', $input, $NW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Tests that the deactivate action returns true
   */
  public function testDeactivateActionReturnsTrue() {
    $NW      = $this->getMasterNodeWorx();
    $where   = 'email = ' . q( self::SW_SECONDARY_EMAIL );
    $user    = $NW->getUserInfo( $where );
    $user_id = $user['userid'];

    $input = array( 'user' => self::SW_SECONDARY_EMAIL );
    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'deactivate', $input, $NW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Tests that the activate action returns true
   */
  public function testActivateActionReturnsTrue() {
    $NW      = $this->getMasterNodeWorx();
    $where   = 'email = ' . q( self::SW_SECONDARY_EMAIL );
    $user    = $NW->getUserInfo( $where );
    $user_id = $user['userid'];

    $input = array( 'user' => self::SW_SECONDARY_EMAIL );
    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'activate', $input, $NW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Tests deleting the user
   */
  public function testDeleteActionReturnsTrue() {
    $NW      = $this->getMasterNodeWorx();
    $where   = 'email = ' . q( self::SW_SECONDARY_EMAIL );
    $user    = $NW->getUserInfo( $where );
    $user_id = $user['userid'];

    $input = array( 'user' => self::SW_SECONDARY_EMAIL );
    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'delete', $input, $NW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Verifies that the user was actually deleted
   */
  public function testDeleteActionHappened() {
    $user_id = self::getNWUserId( self::SW_SECONDARY_EMAIL );
    $this->assertFalse( $user_id, 'User still exists!' );
  }

  /**
   * Gets the array of information assigned to the index action
   *
   * @return array
   */
  private function _getIndexInfo() {
    $NW = $this->getMasterNodeWorx();
    return IW::FC()->routeFromPHP( self::CTRL, 'index', array(), $NW )->getData();
  }

}
