<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Tim College <tcollege@interworx.com>
 * @date       Feb 26, 2009
 */

/**
 * Tests for Ctrl_Siteworx_DomainsSub.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class Ctrl_Siteworx_DomainsSubTest extends IWorx_Framework_TestCase {
  const CTRL = 'Ctrl_Siteworx_DomainsSub';

  const PREFIX = 'subdomain';

  /**
   * Removes the account if necessary.
   */
  public function testSetup() {
    $this->deleteSWAccountIfNecessary();
    $this->addSWAccount();
  }

  /**
   * Tests the output of the index action.
   */
  public function testIndexActionReturnsArray() {
    $output = $this->getSWIndexData( self::CTRL );

    $this->assertInternalType( 'array', $output );
  }

  /**
   * Verifies initial count of subdomains.
   */
  public function testInitialSubdomainCount() {
    $data = $this->getSWIndexData( self::CTRL );

    $this->assertCount( 0, $data['iw_payload']->getData(),
                        'Subdomains already exist' );
  }

  /**
   * Tests that addSubdomainAction fails without input.
   */
  public function testAddFailsWithoutInput() {
    $SW = $this->getSWAccount();

    $Reply  = IW::FC()->routeFromPHP( self::CTRL, 'add', array(), $SW );
    $result = $Reply->getData();

    $this->assertIsFormErrorArray( $result );
  }

  /**
   * Tests that addSubdomainAction returns true.
   */
  public function testAddReturnsTrue() {
    $SW = $this->getSWAccount();

    $input = array( 'prefix' => self::PREFIX );

    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'add', $input, $SW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Tests that the add action worked.
   */
  public function testAddActionWorked() {
    $data = $this->getSWIndexData( self::CTRL );

    $this->assertCount( 1, $data['iw_payload']->getData(),
                        'Subdomains were not added' );
  }

  /**
   * Test that the directory was actually created.
   *
   * @depends testAddReturnsTrue
   */
  public function testAddActionCreatedDirectory() {
    $SW = $this->getSWAccount();
    $dir = $SW->getWebRoot() . '/' . self::PREFIX;

    $this->assertTrue( IWorxFileSys::fileExists( $dir ) );
  }

  /**
   * Tests that addSubdomainAction returns true.
   */
  public function testNumericAddReturnsTrue() {
    $SW = $this->getSWAccount();

    $input = array( 'prefix' => '12345' );

    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'add', $input, $SW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Test that the directory was actually created.
   *
   * @depends testNumericAddReturnsTrue
   */
  public function testAddNumericCreatedDirectory() {
    $SW = $this->getSWAccount();
    $dir = $SW->getWebRoot() . '/' . '12345';

    $this->assertTrue( IWorxFileSys::fileExists( $dir ) );
  }

  /**
   * Tests that deleteAction fails without input.
   */
  public function testDeleteFailsWithoutInput() {
    $SW = $this->getSWAccount();

    $Reply  = IW::FC()->routeFromPHP( self::CTRL, 'delete', array(), $SW );
    $result = $Reply->getData();

    $this->assertIsFormErrorArray( $result );
  }

  /**
   * Tests that deleteAction returns true.
   */
  public function testDeleteReturnsTrue() {
    $SW = $this->getSWAccount();

    $input = array( 'prefix'     => self::PREFIX,
                    'delete_dir' => true );

    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'delete', $input, $SW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Test that the directory was actually deleted.
   *
   * @depends testDeleteReturnsTrue
   */
  public function testDeleteActionDeletedDirectory() {
    $SW  = $this->getSWAccount();
    $dir = $SW->getWebRoot() . '/' . self::PREFIX;

    $this->assertFalse( IWorxFileSys::fileExists( $dir ) );
  }

  /**
   * Tests that deleteAction returns true.
   *
   * @depends testNumericAddReturnsTrue
   */
  public function testDeleteNumericReturnsTrue() {
    $SW = $this->getSWAccount();

    $input = array( 'prefix' => '12345' );

    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'delete', $input, $SW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Test that the directory was not deleted.
   *
   * @depends testDeleteNumericReturnsTrue
   */
  public function testDeleteNumericDeletedDirectory() {
    $SW  = $this->getSWAccount();
    $dir = $SW->getWebRoot() . '/12345';

    $this->assertTrue( IWorxFileSys::fileExists( $dir ) );
  }

  /**
   * Test that the delete action seemed to work.
   */
  public function testDeleteActionWorked() {
    $data = $this->getSWIndexData( self::CTRL );

    $this->assertCount( 0, $data['iw_payload']->getData() );
  }
}
