<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Tim College <tcollege@interworx.com>
 * @date    Feb 27, 2009
 */

/**
 * Tests for the EmailAlias controller.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class Ctrl_Siteworx_EmailAliasTest extends IWorx_Framework_TestCase {
  const CTRL  = 'Ctrl_Siteworx_EmailAlias';
  const ALIAS = 'alias';

  /**
   * Only deletes the account if there are already aliases.
   */
  public function testSetup() {
    $data = $this->getSWIndexData( self::CTRL );

    if( count( $data['iw_payload']->getData() ) !== 0 ) {
      $this->deleteSWAccountIfNecessary();
    }
  }

  /**
   * Verifies that the index data has the aliases varialbe.
   */
  public function testIwAliasesIsAssigned() {
    $data = $this->getSWIndexData( self::CTRL );

    $this->assertArrayHasKey( 'iw_payload', $data );
  }

  /**
   * Makes sure that the iw_form is a Form.
   */
  public function testIwFormPresence() {
    $data = $this->getSWIndexData( self::CTRL );

    $this->assertInstanceOf( 'Form', $data['iw_form'] );
  }

  /**
   * Checks that there are no Aliases initially.
   */
  public function testInitialAliasCount() {
    $data = $this->getSWIndexData( self::CTRL );

    $this->assertCount( 0,
                        $data['iw_payload']->getData(),
                        'Aliases are already present!' );
  }

  /**
   * Checks that the unknownadd action returns true.
   */
  public function testAddReturnsTrue() {
    $SW = $this->getSWAccount();

    $input = array( 'username' => self::ALIAS,
                    'forwardsto' => self::SW_EMAIL );

    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'add', $input, $SW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Checks that the alias was added.
   */
  public function testNewAliasCount() {
    $data = $this->getSWIndexData( self::CTRL );

    $this->assertCount( 1,
                        $data['iw_payload']->getData(),
                        'Alias does not seem to have been added' );
  }

  /**
   * Tests that the edit action returns true.
   */
  public function testEditActionReturnsTrue() {
    $SW = $this->getSWAccount();

    $input = array( 'username'   => self::ALIAS,
                    'forwardsto' => self::SW_SECONDARY_EMAIL );

    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'edit', $input, $SW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Verifies the changed value.
   */
  public function testEditActionValueChanged() {
    $data = $this->getSWIndexData( self::CTRL );

    $users = $data['iw_payload']->getData();

    $user = $users[0];

    $this->assertEquals( self::SW_SECONDARY_EMAIL, $user->dest );
  }

  /**
   * Tests that the delete action returned true.
   */
  public function testDeleteActionReturnsTrue() {
    $SW = $this->getSWAccount();

    $input = array( 'username' => self::ALIAS );

    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'delete', $input, $SW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Checks that the alias is gone.
   */
  public function testAliasCountAfterDelete() {
    $data = $this->getSWIndexData( self::CTRL );

    $this->assertCount( 0,
                        $data['iw_payload']->getData(),
                        'Aliases are still present!' );
  }
}
