<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Tim College <tcollege@interworx.com>
 * @date       Feb 26, 2009
 */

/**
 * Tests for Ctrl_Siteworx_MysqlDb.
 *
 * @package    InterWorx
 * @subpackage Tests
 * @group      slow-tests
 */
class Ctrl_Siteworx_MysqlDbTest extends IWorx_Framework_TestCase {
  const CTRL = 'Ctrl_Siteworx_MysqlDb';

  const DBNAME   = 'dbname';
  const DBUSER   = 'dbuser';
  const DBUSERPW = '1234%11';

  /**
   * Clears the account if needed.
   */
  public function testSetup() {
    $data = $this->getSWIndexData( self::CTRL );

    if( count( $data['iw_payload']->getData() ) !== 0 ) {
      $this->deleteSWAccountIfNecessary();
    }
  }

  /**
   * Tests the output of the index action.
   */
  public function testIndexActionReturnsArray() {
    $output = $this->getSWIndexData( self::CTRL );

    $this->assertInternalType( 'array', $output );
  }

  /**
   * Tests that there are no databases initially.
   */
  public function testInitialDatabaseCount() {
    $data = $this->getSWIndexData( self::CTRL );

    $this->assertCount( 0,
                        $data['iw_payload']->getData(),
                        'Databases already exist!' );
  }

  /**
   * Tests that addAction fails without input.
   */
  public function testAddFailsWithoutInput() {
    $SW = $this->getSWAccount();

    $Reply  = IW::FC()->routeFromPHP( self::CTRL, 'add', array(), $SW );
    $result = $Reply->getData();

    $this->assertIsFormErrorArray( $result );
  }

  /**
   * Tests that addAction returns true.
   */
  public function testAddReturnsTrue() {
    $SW = $this->getSWAccount();

    $input = array( 'name' => self::DBNAME );

    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'add', $input, $SW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Test that the add action seemed to work.
   */
  public function testAddActionWorked() {
    $data = $this->getSWIndexData( self::CTRL );

    $this->assertCount( 1, $data['iw_payload']->getData() );
  }

  /**
   * Tests that deleteAction fails without input.
   */
  public function testDeleteFailsWithoutInput() {
    $SW = $this->getSWAccount();

    $Reply  = IW::FC()->routeFromPHP( self::CTRL, 'delete', array(), $SW );
    $result = $Reply->getData();

    $this->assertIsFormErrorArray( $result );
  }

  /**
   * Tests that deleteAction returns true.
   */
  public function testDeleteReturnsTrue() {
    $SW = $this->getSWAccount();

    $input = array( 'name' => self::DBNAME );

    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'delete', $input, $SW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Test that the delete action seemed to work.
   */
  public function testDeleteActionWorked() {
    $data = $this->getSWIndexData( self::CTRL );

    $this->assertCount( 0, $data['iw_payload']->getData() );
  }

  /**
   * Test the add action with the create user option returns true.
   */
  public function testAddActionWithUser() {
    $SW = $this->getSWAccount();

    $input = array( 'name'        => self::DBNAME,
                    'create_user' => '1',
                    'user'        => self::DBUSER,
                    'password'    => self::DBUSERPW,
                    'confirm_password' => self::DBUSERPW );

    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'add', $input, $SW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Test that the add action with the create option checked actually worked.
   */
  public function testAddActionWithUserWorked() {
    $SW     = $this->getSWAccount();
    $Driver = $SW->getMySQLDriver();

    $user_ok = $Driver->userExists( self::DBUSER );
    $this->assertTrue( $user_ok );

    $perms_ok = $Driver->permissionsExist( self::DBUSER, self::DBNAME, '%' );
    $this->assertTrue( $perms_ok );
  }
}
