<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Tim College <tcollege@interworx.com>
 * @date    Feb 24, 2009
 */

/**
 * Tests for siteworx users controller.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class Ctrl_Siteworx_UsersTest extends IWorx_Framework_TestCase {

  const CTRL = 'Ctrl_Siteworx_Users';

  const SW_SECONDARY_NICK_CHANGE = 'Changed Nickname';

  static private $_first_setup = false;
  static private $_first_teardown = false;

  /**
   * Make sure the SiteWorx account has been deleted.
   */
  public function setUp() {
    if( self::$_first_setup === false ) {
      $this->deleteSWAccountIfNecessary();

      self::$_first_setup = true;
    }
  }

  /**
   * Make sure the SiteWorx account has been deleted.
   */
  public function tearDown() {
    if( self::$_first_teardown === false ) {
      $this->deleteSWAccountIfNecessary();

      self::$_first_teardown = true;
    }
  }

  /**
   * Tests the index action executes successfully.
   */
  public function testIndexAction() {
    $SW    = $this->getSWAccount();
    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'index', array(), $SW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Tests the list action (which has been depricated).
   */
  public function testListAction() {
    $SW    = $this->getSWAccount();
    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'list', array(), $SW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Tests the list users action.
   */
  public function testListUsersAction() {
    $SW    = $this->getSWAccount();
    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'listUsers', array(), $SW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Tests the list editable action.
   */
  public function testListEditableAction() {
    $SW    = $this->getSWAccount();
    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'listEditable', array(), $SW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Tests the list deletable action.
   */
  public function testListDeletableAction() {
    $SW    = $this->getSWAccount();
    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'listDeletable', array(), $SW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Tests the list working user action.
   */
  public function testListWorkingUserAction() {
    $SW    = $this->getSWAccount();
    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'listWorkingUser', array(), $SW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Tests the list master user action.
   */
  public function testListMasterUserAction() {
    $SW    = $this->getSWAccount();
    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'listMasterUser', array(), $SW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Tests adding a secondary user.
   */
  public function testAddAction() {
    $SW    = $this->getSWAccount();
    $input = array( 'nickname'         => self::SW_SECONDARY_NICK,
                    'email'            => self::SW_SECONDARY_EMAIL,
                    'password'         => self::SW_SECONDARY_PASS,
                    'confirm_password' => self::SW_SECONDARY_PASS,
                    'perms'            => array( 'LOGIN' ) );

    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'add', $input, $SW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Tests that adding a user actually worked.
   *
   * @depends testAddAction
   */
  public function testAddActionWorked() {
    $Model_User = Model_Siteworx_User::getByEmail( self::SW_SECONDARY_EMAIL );

    $this->assertTrue( !empty( $Model_User ) );
    $this->assertTrue( $Model_User instanceof Model_Siteworx_User );
  }

  /**
   * Tests adding a secondary user again.
   */
  public function testAddActionFailsWithDuplicate() {
    $SW = $this->getSWAccount();

    $input = array( 'nickname'         => self::SW_SECONDARY_NICK,
                    'email'            => self::SW_SECONDARY_EMAIL,
                    'password'         => self::SW_SECONDARY_PASS,
                    'confirm_password' => self::SW_SECONDARY_PASS );

    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'add', $input, $SW );

    $this->assertFalse( $Reply->wasSuccessful() );
  }

  /**
   * Tests that the edit action returns true.
   */
  public function testEditAction() {
    $SW    = $this->getSWAccount();
    $input = array( 'user'     => self::SW_SECONDARY_EMAIL,
                    'nickname' => self::SW_SECONDARY_NICK_CHANGE );

    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'edit', $input, $SW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Tests that adding a user actually worked.
   *
   * @depends testEditAction
   */
  public function testEditActionWorked() {
    $Model_User = Model_Siteworx_User::getByEmail( self::SW_SECONDARY_EMAIL );

    $this->assertTrue( !empty( $Model_User ) );
    $this->assertTrue( $Model_User instanceof Model_Siteworx_User );
    $this->assertEquals( self::SW_SECONDARY_NICK_CHANGE, $Model_User->nickname );
  }

  /**
   * Tests that the deactivate action returns true.
   */
  public function testDeactivateAction() {
    $SW    = $this->getSWAccount();
    $input = array( 'user' => self::SW_SECONDARY_EMAIL );

    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'deactivate', $input, $SW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Tests that the activate action returns true.
   */
  public function testActivateAction() {
    $SW    = $this->getSWAccount();
    $input = array( 'user' => self::SW_SECONDARY_EMAIL );

    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'activate', $input, $SW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Tests deleting the user.
   */
  public function testDeleteActionReturnsTrue() {
    $SW    = $this->getSWAccount();
    $input = array( 'user' => self::SW_SECONDARY_EMAIL );
    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'delete', $input, $SW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Verifies that the user was actually deleted.
   */
  public function testDeleteActionHappened() {
    $user_id = self::getSWUserId( self::SW_DOMAIN, self::SW_SECONDARY_EMAIL );
    $this->assertFalse( $user_id, 'User still exists!' );
  }

  /**
   * Test that add/edit/delete of users with no permissions works.
   */
  public function testUsersWithNoPermissions() {
    $SW = $this->getSWAccount();

    //add
    $input = array( 'nickname'         => self::SW_SECONDARY_NICK,
                    'email'            => self::SW_SECONDARY_EMAIL,
                    'password'         => self::SW_SECONDARY_PASS,
                    'confirm_password' => self::SW_SECONDARY_PASS,
                    'perms'            => array() );
    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'add', $input, $SW );
    $this->assertReplySuccessful( $Reply );

    //edit
    $input = array( 'user' => self::SW_SECONDARY_EMAIL );
    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'edit', $input, $SW );
    $this->assertReplySuccessful( $Reply );

    //delete
    $input = array( 'user' => self::SW_SECONDARY_EMAIL );
    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'delete', $input, $SW );
    $this->assertReplySuccessful( $Reply );
  }
}