<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Matt Parmelee <mparmelee@interworx.com>
 * @date       Nov 12, 2014
 */

/**
 * Test class for Quotas.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class QuotaTest extends IWorx_Framework_TestCase {

  /**
   * Setup.
   */
  public function setUp() {
    $cmd = 'useradd quotatest';
    $Command = new Command( $cmd );
    $Command->noOutput();
    $Command->exec();
  }

  /**
   * Teardown.
   */
  public function tearDown() {
    $cmd = 'userdel -rf quotatest';
    $Command = new Command( $cmd );
    $Command->noOutput();
    $Command->exec();
  }

  /**
   * Checks if quotas are enabled.
   */
  public function testQuotasEnabled() {
    $partition = Ini::get( Ini::QUOTA, 'partition' );
    $cmd       = "mount | grep ' $partition ' | grep noquota";
    $Command   = new Command( $cmd );
    $Command->allowPipe();
    $Command->exec();

    $retval = $Command->getRetval();
    $this->assertNotEquals( 0, $retval );
  }

  /**
   * Checks if quotas can be edited.
   */
  public function testQuotaEdit() {
    $mb     = 1337;
    $result = IWorxFileSys::setQuota( 'quotatest', $mb );
    $this->assertTrue( $result );

    $bytes = IWorxFileSys::getQuota( 'quotatest' );
    $mb2   = round( $bytes / 1024, 2 );
    $this->assertEquals( $mb, $mb2 );
  }

  /**
   * Checks if quotas can be deleted.
   */
  public function testQuotaDelete() {
    $unlim  = Ini::get( Ini::CONSTANTS, 'unlimited' );
    $result = IWorxFileSys::setQuota( 'quotatest', $unlim );
    $this->assertTrue( $result );

    $bytes = IWorxFileSys::getQuota( 'quotatest' );
    $this->assertEquals( $bytes, 0 );
  }

  /**
   * Checks if quotas are enforced.
   */
  public function testQuotasEnforced() {
    $mb     = 10;
    $result = IWorxFileSys::setQuota( 'quotatest', $mb );
    $this->assertTrue( $result );

    $file = '/home/quotatest/quotatest.txt';
    $over = $mb + 2;
    $cmd  = "dd if=/dev/urandom of={$file} bs={$over}M count=1";
    $Command = new Command( $cmd );
    $Command->setUser( 'quotatest' );
    $Command->noOutput();
    $Command->exec();

    $this->assertFileExists( $file );
    $this->assertNotEquals( $over, filesize( $file ) );

    IWorxFileSys::rm( $file );
  }
}
