<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2014 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Tim College <tcollege@interworx.com>
 * @date       Feb 13, 2014
 */

require_once '/usr/local/interworx/TESTS/integration/Service/ClientTestCase.php';

/**
 * Test for status of client DNS.
 *
 * @package    InterWorx
 * @subpackage Tests
 * @retryLimit 2
 */
class Service_Client_DnsTest extends Service_ClientTestCase {
  /**
   * Run before each test.
   */
  public function setUp() {
    $NW = new NodeWorx( NodeWorx::MASTER_ID );
    $this->_setService( $NW->getDNSDriver() );
  }

  /**
   * Basic test to see if the service responds on the port it uses.
   */
  public function testSocketConnection() {
    parent::testSocketConnection();
  }

  /**
   * Test that the service isRunning function thinks it's running.
   */
  public function testIsRunning() {
    parent::testIsRunning();
  }

  /**
   * Run service-specific tests to verify running status one more way.
   */
  public function testConfirmIsRunning() {
    parent::testConfirmIsRunning();
  }

  /**
   * Test iterative (local) DNS lookups.
   */
  public function testDnsLookupIterative() {
    self::addSWAccountIfNecessary();
    IWorxExec::exec( '/home/interworx/cron/iworx.pex --fively' );
    $ip = $this->_getFirstSharedIP();

    $cmd = "dig @{$ip} addedbytest.com mx";

    $result = [];
    $retval = -1;

    IWorxExec::exec( $cmd, $result, $retval );

    $actual = implode( "\n", $result );

    $this->assertContains( 'mail.addedbytest.com', $actual );
  }

  /**
   * Test the recursive (external) DNS lookups.
   */
  public function testDnsLookupRecursive() {
    $cmd = 'dig @127.0.0.1 interworx.com mx';

    $result = [];
    $retval = -1;

    IWorxExec::exec( $cmd, $result, $retval );

    $actual = implode( "\n", $result );

    $this->assertContains( 'turboflow.interworx.com', $actual );
  }

  /**
   * Tests that stopping the function returns true.
   *
   * @depends testIsRunning
   */
  public function testStop() {
    parent::testStop();
  }

  /**
   * Pretty stupid test, checks whether the service is in ps auxf.
   *
   * @depends testStop
   */
  public function testStoppedWithGrepPsauxf() {
    parent::testStoppedWithGrepPsauxf();
  }

  /**
   * Confirms that the service is actually stopped.
   *
   * @depends testStop
   */
  public function testConfirmStopped() {
    parent::testConfirmStopped();
  }

  /**
   * Tests that stopping the function returns true.
   *
   * @depends testIsRunning
   * @depends testStop
   */
  public function testStart() {
    parent::testStart();
  }

  /**
   * Pretty stupid test, checks whether the service is in ps auxf.
   *
   * @depends testStart
   */
  public function testStartedWithGrepPsauxf() {
    parent::testStartedWithGrepPsauxf();
  }

  /**
   * Confirms the service was started.
   *
   * @depends testStart
   */
  public function testConfirmStarted() {
    parent::testConfirmStarted();
  }

  /**
   * Confirm the service is running.
   *
   * @return boolean
   */
  protected function _confirmIsRunning() {
    return true;
  }

  /**
   * Confirm that mysql is stopped.
   *
   * @return boolean
   */
  protected function _confirmStopped() {
    return true;
  }

  /**
   * Confirm that mysql is started.
   *
   * @return boolean
   */
  protected function _confirmStarted() {
    return $this->_confirmIsRunning();
  }

  /**
   * Get the string to look for in ps auxf.
   *
   * @return string
   */
  protected function _getGrepProcessString() {
    return '/usr/bin/axfrdns';
  }
}
