<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2014 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Service
 * @author     Tim College <tcollege@interworx.com>
 * @date       Feb 6, 2014
 */

require_once '/usr/local/interworx/TESTS/integration/Service/ClientTestCase.php';

/**
 * Test for status of client MySQL.
 *
 * @package    InterWorx
 * @subpackage Service
 */
class Service_Client_MySQLTest extends Service_ClientTestCase {
  /**
   * Run before each test.
   */
  public function setUp() {
    $NW = new NodeWorx( NodeWorx::MASTER_ID );
    $this->_setService( $NW->getMySQLDriver() );
  }

  /**
   * Basic test to see if the service responds on the port it uses.
   */
  public function testSocketConnection() {
    parent::testSocketConnection();
  }

  /**
   * Test that the service isRunning function thinks it's running.
   */
  public function testIsRunning() {
    parent::testIsRunning();
  }

  /**
   * Run service-specific tests to verify running status one more way.
   */
  public function testConfirmIsRunning() {
    parent::testConfirmIsRunning();
  }

  /**
   * Test that we can initialize a connection to the database.
   *
   * @depends testIsRunning
   */
  public function testConnection() {
    try {
      $DB = IWorxDbUtil::db_connect( Ini::get( Ini::MYSQL, 'rootdsn' ) );
      // Success is the absence of failure, in this case.
      $this->assertTrue( true );
    } catch( Exception $e ) {
      $this->fail( $e->getMessage() );
    }
  }

  /**
   * Test inserting.
   *
   * @depends testConnection
   */
  public function testInsert() {
    try {
      $DB = IWorxDbUtil::db_connect( Ini::get( Ini::MYSQL, 'rootdsn' ) );
      $sql = 'INSERT INTO user SET User = "Fake User"';
      $this->assertEquals( 1, IWorxDbUtil::db_query( $DB, $sql ) );
    } catch( Exception $e ) {
      $this->fail( $e->getMessage() );
    }
  }

  /**
   * Test a basic select.
   *
   * @depends testConnection
   */
  public function testSelect() {
    try {
      $DB = IWorxDbUtil::db_connect( Ini::get( Ini::MYSQL, 'rootdsn' ) );

      $sql = 'SELECT * FROM user WHERE User = "Fake User"';
      $users = IWorxDbUtil::db_getAll( $DB, $sql );
      $this->assertNotEmpty( $users );
    } catch( Exception $e ) {
      $this->fail( $e->getMessage() );
    }
  }

  /**
   * Test deleting.
   *
   * @depends testConnection
   */
  public function testDelete() {
    try {
      $DB = IWorxDbUtil::db_connect( Ini::get( Ini::MYSQL, 'rootdsn' ) );
      $sql = 'DELETE FROM user WHERE User = "Fake User"';
      $this->assertEquals( 1, IWorxDbUtil::db_query( $DB, $sql ) );
    } catch( Exception $e ) {
      $this->fail( $e->getMessage() );
    }
  }

  /**
   * Tests that stopping the function returns true.
   *
   * @depends testIsRunning
   */
  public function testStop() {
    parent::testStop();
  }

  /**
   * Pretty stupid test, checks whether the service is in ps auxf.
   *
   * @depends testStop
   */
  public function testStoppedWithGrepPsauxf() {
    parent::testStoppedWithGrepPsauxf();
  }

  /**
   * Confirms that the service is actually stopped.
   *
   * @depends testStop
   */
  public function testConfirmStopped() {
    parent::testConfirmStopped();
  }

  /**
   * Tests that stopping the function returns true.
   *
   * @depends testIsRunning
   * @depends testStop
   */
  public function testStart() {
    parent::testStart();
  }

  /**
   * Pretty stupid test, checks whether the service is in ps auxf.
   *
   * @depends testStart
   */
  public function testStartedWithGrepPsauxf() {
    parent::testStartedWithGrepPsauxf();
  }

  /**
   * Confirms the service was started.
   *
   * @depends testStart
   */
  public function testConfirmStarted() {
    parent::testConfirmStarted();
  }

  /**
   * Confirm the service is running.
   *
   * @return boolean
   */
  protected function _confirmIsRunning() {
    return $this->_getService()->isRunningLocal();
  }

  /**
   * Confirm that mysql is stopped.
   *
   * @return boolean
   */
  protected function _confirmStopped() {
    return $this->_confirmIsRunning() === false;
  }

  /**
   * Confirm that mysql is started.
   *
   * @return boolean
   */
  protected function _confirmStarted() {
    return $this->_confirmIsRunning();
  }

  /**
   * Get the string to look for in ps auxf.
   *
   * @return string
   */
  protected function _getGrepProcessString() {
    return 'datadir=/var/lib/mysql';
  }
}
