<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2014 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Paul Oehler <poehler@interworx.com>
 * @date       Apr 09, 2014
 */

/**
 * Test class.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class Bug3493IworxBackupDirRootAttackTest extends IWorx_Framework_TestCase {
  /**
   * Trac: #3493.
   */
  public function testIworxBackupDirRootAttack() {
    IWorxFileSys::rm( '/etc/iworx-fake-shadow', true, true );

    // same default perms/owner as /etc/shadow
    file_put_contents( '/etc/iworx-fake-shadow', 'top secret info here' );
    chmod( '/etc/iworx-fake-shadow', 000 );
    chown( '/etc/iworx-fake-shadow', 'root' );
    chgrp( '/etc/iworx-fake-shadow', 'root' );

    $perms1 = $this->getFilePermsAsString( fileperms( '/etc/iworx-fake-shadow' ) );
    $owner1 = posix_getpwuid( fileowner( '/etc/iworx-fake-shadow' ) );
    $owner1 = $owner1['name'];
    $group1 = posix_getgrgid( filegroup( '/etc/iworx-fake-shadow' ) );
    $group1 = $group1['name'];
    $this->assertEquals( 'root:root|----------', "{$owner1}:{$group1}|{$perms1}" );

    $this->deleteSWAccountIfNecessary();
    $this->addSWAccount();

    // prep attack
    exec( 'su -m addedbyt -c "cd /home/addedbyt/addedbytest.com/
                       rm -rf iworx-backup
                       mkdir iworx-backup
                       ln /etc/iworx-fake-shadow iworx-backup/iworx-fake-shadow"' );

    // trigger backup creation
    $ctrl = 'Ctrl_Nodeworx_Backup';
    $action = 'fullbackup';
    $input = array( 'domains' => 'addedbytest.com' );
    $NW = $this->getMasterNodeWorx();

    $Reply = IW::FC()->routeFromPHP( $ctrl, $action, $input, $NW );
    $this->assertReplySuccessful( $Reply );

    $perms2 = $this->getFilePermsAsString( fileperms( '/etc/iworx-fake-shadow' ) );
    $owner2 = posix_getpwuid( fileowner( '/etc/iworx-fake-shadow' ) );
    $owner2 = $owner2['name'];
    $group2 = posix_getgrgid( filegroup( '/etc/iworx-fake-shadow' ) );
    $group2 = $group2['name'];
    $this->assertEquals( 'root:root|----------', "{$owner2}:{$group2}|{$perms2}" );

    $this->deleteSWAccountIfNecessary();
    $this->addSWAccount();

    // prep attack
    exec( 'su -m addedbyt -c "cd /home/addedbyt/addedbytest.com/
                       rm -rf iworx-backup
                       ln /etc/iworx-fake-shadow iworx-backup"' );

    // trigger backup creation
    $ctrl = 'Ctrl_Nodeworx_Backup';
    $action = 'fullbackup';
    $input = array( 'domains' => 'addedbytest.com' );
    $NW = $this->getMasterNodeWorx();

    $Reply = IW::FC()->routeFromPHP( $ctrl, $action, $input, $NW );
    $this->assertReplySuccessful( $Reply );

    $perms2 = $this->getFilePermsAsString( fileperms( '/etc/iworx-fake-shadow' ) );
    $owner2 = posix_getpwuid( fileowner( '/etc/iworx-fake-shadow' ) );
    $owner2 = $owner2['name'];
    $group2 = posix_getgrgid( filegroup( '/etc/iworx-fake-shadow' ) );
    $group2 = $group2['name'];
    $this->assertEquals( 'root:root|----------', "{$owner2}:{$group2}|{$perms2}" );
  }
}
