<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2014 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Paul Oehler <poehler@interworx.com>
 * @date       Apr 09, 2014
 */

/**
 * Test class.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class Bug3497CrossResellerBackupRestoreAttackTest extends IWorx_Framework_TestCase {
  /**
   * Trac: #3497.
   */
  public function testCrossResellerBackupRestoreAttack() {
    $this->deleteResellerSwAccountIfNecessary();
    $this->deleteResellerAccountIfNecessary();

    $this->addResellerIfNecessary();

    $this->deleteSWAccount();
    $this->addSWAccount();

    $input = array( 'domain' => self::SW_DOMAIN,
                    'domains' => array( self::SW_DOMAIN ),
                    'ipv4s' => array( self::_getFirstSharedResellerIP() ),
                    'ipv6s' => array( 0 ) );
    $Reply = IW::FC()->routeFromPHP( 'Ctrl_Nodeworx_Siteworx',
                                     'editIpsCommit',
                                     $input,
                                     self::getMasterNodeWorx() );

    $this->assertReplySuccessful( $Reply );

    $input = array( 'domains' => array( self::SW_DOMAIN ), 'block' => '1' );
    $Reply = IW::FC()->routeFromPHP( 'Ctrl_Nodeworx_Backup',
                                     'structureonly',
                                     $input,
                                     self::getMasterNodeWorx() );

    $this->assertReplySuccessful( $Reply );

    $input = array( 'reseller_id' => self::getResellerId(),
                    'ips' => array( self::_getFirstSharedResellerIP(),
                                    self::_getFirstSharedIP() ) );

    $Reply = IW::FC()->routeFromPHP( 'Ctrl_Nodeworx_Reseller',
                                     'edit',
                                     $input,
                                     self::getMasterNodeWorx() );

    $backups = $this->_getCompleteBackupFileNames();
    $backup = '/home/addedbyt/addedbytest.com/iworx-backup/' . $backups[0];

    $ctrl = 'Ctrl_Nodeworx_Import';
    $action = 'importCommit';
    $input = array( 'reseller_id' => self::getResellerId(),
                    'archive' => $backup,
                    'cp' => 'siteworx',
                    'ipaddr' => self::_getFirstSharedResellerIP(),
                    'dbserver' => 'localhost' );

    $Reply = IW::FC()->routeFromPHP( $ctrl,
                                     $action,
                                     $input,
                                     self::getResellerNodeWorx() );

    $this->assertReplyNotSuccessful( $Reply, 'Imported account across reseller??' );

    $this->assertContains( '##LG_CANNOT_TRIGGER_IMPORT_CROSS_RESELLER##',
                           $Reply->getErrorsAsString() );

    exec( 'tail -n500 ~iworx/var/log/iworx.log | ' .
          "grep 'import.php --archive={$backup}'",
          $result,
          $retval );

    $msg = 'Got too far trying to import a backup from a different reseller. ' .
      'We should prevent this from happening much sooner, like in form validation.';
    $this->assertCount( 0, $result, $msg );
  }

  /**
   * Helper to get existing backup file names under the account.
   *
   * @return array
   */
  private function _getCompleteBackupFileNames() {
    $domain     = self::SW_DOMAIN;
    $DataSource = new DataSource_Backup_Files( $domain );

    return $DataSource->getValues();
  }

}
