<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2014 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Paul Oehler <poehler@interworx.com>
 * @date       Apr 09, 2014
 */

/**
 * Test class.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class HtaccessSecurityTest extends IWorx_Framework_TestCase {

  /**
   * Trac: #3509, #3510.
   */
  public function testHtaccessHardlinkAttack() {
    IWorxFileSys::rm( '/etc/iworx-fake-shadow', true, true );

    // same default perms/owner as /etc/shadow
    file_put_contents( '/etc/iworx-fake-shadow', 'top secret info here' );
    chmod( '/etc/iworx-fake-shadow', 000 );
    chown( '/etc/iworx-fake-shadow', 'root' );
    chgrp( '/etc/iworx-fake-shadow', 'root' );

    $this->deleteSWAccountIfNecessary();
    $this->addSWAccount();

    $old_md5 = md5_file( '/etc/iworx-fake-shadow' );

    // prep attack
    exec( 'su -m addedbyt -c "cd /home/addedbyt/addedbytest.com/html
                              rm -f .htaccess
                              ln /etc/iworx-fake-shadow .htaccess.off"' );

    $input = array( 'input' => array( 'new_dir' =>
                                      '/home/addedbyt/addedbytest.com/html' ) );

    $Reply = IW::FC()->routeFromPHP( 'Ctrl_Siteworx_Htaccess',
                                     'add',
                                     $input,
                                     $this->getSWAccount() );

    $msg = 'Critical system file modified!';
    $this->assertEquals( $old_md5, md5_file( '/etc/iworx-fake-shadow' ), $msg );
  }

  /**
   * Trac: #3509, #3510.
   */
  public function testHtaccessSymlinkAttack() {
    IWorxFileSys::rm( '/etc/iworx-fake-shadow', true, true );

    // same default perms/owner as /etc/shadow
    file_put_contents( '/etc/iworx-fake-shadow', 'top secret info here' );
    chmod( '/etc/iworx-fake-shadow', 000 );
    chown( '/etc/iworx-fake-shadow', 'root' );
    chgrp( '/etc/iworx-fake-shadow', 'root' );

    $this->deleteSWAccountIfNecessary();
    $this->addSWAccount();

    $old_md5 = md5_file( '/etc/iworx-fake-shadow' );

    // prep attack
    exec( 'su -m addedbyt -c "cd /home/addedbyt/addedbytest.com/html
                              rm -f .htaccess
                              rm -f .htaccess.off
                              ln -s /etc/iworx-fake-shadow .htaccess.off"' );

    $input = array( 'input' => array( 'new_dir' =>
                                      '/home/addedbyt/addedbytest.com/html' ) );

    $Reply = IW::FC()->routeFromPHP( 'Ctrl_Siteworx_Htaccess',
                                     'add',
                                     $input,
                                     $this->getSWAccount() );

    $msg = 'Critical system file modified!';
    $this->assertEquals( $old_md5, md5_file( '/etc/iworx-fake-shadow' ), $msg );
  }
}
