<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2015 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Upgrade
 * @author     Tim College <tcollege@interworx.com>
 * @date       Feb 05, 2015
 */

/**
 * Class Command_WithArgsTest
 *
 * @package    InterWorx
 * @subpackage TESTS
 */
class Command_WithArgsTest extends IWorx_Framework_TestCase {
  /**
   * Tests that omitting the base class raises and exception.
   *
   * @expectedException IWorx_Exception_ExecNoCommand
   */
  public function testNoBaseClassRaisesException() {
    $Cmd = new Command_WithArgs();
    $Cmd->exec();
  }

  /**
   * Tests that assembling the command works as expected.
   */
  public function testCommandOptionComposition() {
    $Cmd = new Command_WithArgs( 'command' );
    $Cmd->addOption( 'option', 'foobar' );

    $expected = 'command --option "foobar"';

    $this->assertEquals( $expected, $Cmd->getCmd() );
  }

  /**
   * Tests that assembling the command works as expected.
   */
  public function testCommandArgumentComposition() {
    $Cmd = new Command_WithArgs( 'command' );
    $Cmd->addArgument( 'argument' );

    $expected = 'command argument';

    $this->assertEquals( $expected, $Cmd->getCmd() );
  }

  /**
   * Tests that assembling the command works as expected.
   */
  public function testCommandOptionAndArgumentComposition() {
    $Cmd = new Command_WithArgs( 'command' );
    $Cmd->addOption( 'option', 'foobar' );
    $Cmd->addArgument( 'argument' );

    $expected = 'command --option "foobar" argument';

    $this->assertEquals( $expected, $Cmd->getCmd() );
  }

  /**
   * Tests that assembling the command works as expected.
   */
  public function testCommandArgumentWithSpaces() {
    $Cmd = new Command_WithArgs( 'command' );
    $Cmd->addArgument( 'This is the argument' );

    $expected = 'command "This is the argument"';

    $this->assertEquals( $expected, $Cmd->getCmd() );
  }

  /**
   * Tests that assembling the command works as expected.
   */
  public function testCommandArgumentWithDoubleQuotes() {
    $Cmd = new Command_WithArgs( 'command' );
    $Cmd->addArgument( '"This is the argument"' );

    $expected = 'command "This is the argument"';

    $this->assertEquals( $expected, $Cmd->getCmd() );
  }

  /**
   * Tests that assembling the command works as expected.
   */
  public function testCommandArgumentWithSingleQuotes() {
    $Cmd = new Command_WithArgs( 'command' );
    $Cmd->addArgument( "'This is the argument'" );

    $expected = "command 'This is the argument'";

    $this->assertEquals( $expected, $Cmd->getCmd() );
  }
}
