<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Tim College <tcollege@interworx.com>
 * @date       Oct 30, 2008
 */

/**
 * Tests the behavior of the DataSource_Boolean class.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class DataSource_BooleanTest extends PHPUnit_Framework_TestCase {
  /**
   * Tests behavior of isValidValue()
   */
  public function testIsValidValue() {
    $DS = new DataSource_Boolean();

    $this->assertTrue( $DS->isValidValue( '0' ) );
    $this->assertTrue( $DS->isValidValue( '1' ) );

    $this->assertFalse( $DS->isValidValue( 0 ) );
    $this->assertFalse( $DS->isValidValue( 1 ) );
    $this->assertFalse( $DS->isValidValue( 2 ) );
    $this->assertFalse( $DS->isValidValue( '2' ) );

    $this->assertFalse( $DS->isValidValue( '##LG_YES##' ) );
  }

  /**
   * Tests behavior of getDisplayValue()
   */
  public function testGetDisplayValue() {
    $DS = new DataSource_Boolean();

    $this->assertEquals( IWorxTranslate::tr( '##LG_YES##' ),
                         $DS->getDisplayValue( '1' ) );
    $this->assertEquals( IWorxTranslate::tr( '##LG_NO##' ),
                         $DS->getDisplayValue( '0' ) );
  }

  /**
   * Tests behavior of factory() with default options.
   */
  public function testFactoryDefault() {
    $DS = DataSource_Boolean::factory();

    $this->assertEquals( IWorxTranslate::tr( '##LG_YES##' ),
                         $DS->getDisplayValue( '1' ) );
    $this->assertEquals( IWorxTranslate::tr( '##LG_NO##' ),
                         $DS->getDisplayValue( '0' ) );
  }

  /**
   * Tests behavior of factory() with ENABLED_DISABLED.
   */
  public function testFactoryEnabledDisabled() {
    $DS = DataSource_Boolean::factory( DataSource_Boolean::ENABLED_DISABLED );

    $this->assertEquals( IWorxTranslate::tr( '##LG_ENABLED##' ),
                         $DS->getDisplayValue( '1' ) );
    $this->assertEquals( IWorxTranslate::tr( '##LG_DISABLED##' ),
                         $DS->getDisplayValue( '0' ) );
  }

  /**
   * Tests behavior of factory() with ON_OFF.
   */
  public function testFactoryOnOff() {
    $DS = DataSource_Boolean::factory( DataSource_Boolean::ON_OFF );

    $this->assertEquals( IWorxTranslate::tr( '##LG_ON##' ),
                         $DS->getDisplayValue( '1' ) );
    $this->assertEquals( IWorxTranslate::tr( '##LG_OFF##' ),
                         $DS->getDisplayValue( '0' ) );
  }

  /**
   * Tests behavior of factory() with OPEN_CLOSED.
   */
  public function testFactoryOpenClosed() {
    $DS = DataSource_Boolean::factory( DataSource_Boolean::OPEN_CLOSED );

    $this->assertEquals( IWorxTranslate::tr( '##LG_OPEN##' ),
                         $DS->getDisplayValue( '1' ) );
    $this->assertEquals( IWorxTranslate::tr( '##LG_CLOSED##' ),
                         $DS->getDisplayValue( '0' ) );
  }
}
