<?php
/**
 * InterWorx Hosting Control Panel
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2015 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Upgrade
 * @author     Tim College <tcollege@interworx.com>
 * @date       Jul 28, 2015
 */

/**
 * Class DataSource_LogCustomTest
 *
 * @package    InterWorx
 * @subpackage TESTS
 */
class DataSource_LogCustomTest extends IWorx_Framework_TestCase {
  const NW_CUSTOM_FILE = '/home/interworx/etc/logs.d/nodeworx/unit-test-files.yaml';
  const SW_CUSTOM_FILE = '/home/interworx/etc/logs.d/siteworx/unit-test-files.yaml';

  /**
   * Blow away any leftovers.
   */
  public function setUp() {
    IWorxFileSys::rm( self::NW_CUSTOM_FILE );
    IWorxFileSys::rm( self::SW_CUSTOM_FILE );
    IW::Env()->resetSession();
  }

  /**
   * Check that adding to an existing group works.
   */
  public function testCustomFileExistingGroupNodeworx() {
    $this->_makeNodeworxCustomLogConfig();
    $DS = new DataSource_Log_NW_Http();
    $this->assertContains( '/var/log/what/ever.log', $DS->getValues() );
  }

  /**
   * Check that replacing the filename works as expected.
   */
  public function testCustomFileFilenameReplacement() {
    $this->_makeNodeworxCustomLogConfig();
    $DS = new DataSource_Log_NW_Http();
    $this->assertContains( 'Test log ever.log', $DS->getDisplayValues() );
  }

  /**
   * Confirm that the custom group is noticed.
   */
  public function testCustomGroupExistance() {
    $this->_makeNodeworxCustomLogConfig();
    $DS = new DataSource_Log_NW_Groups();
    $this->assertContains( 'Unit Test Custom Logs', $DS->getValues() );
  }

  /**
   * Confirm that the custom group can load.
   *
   * @depends testCustomGroupExistance
   */
  public function testCustomGroupFiles() {
    $this->_makeNodeworxCustomLogConfig();
    IW::Env()->addInput( 'iw_sess_hint', 'nodeworx' );
    $DS = new DataSource_Log_Custom( 'Unit Test Custom Logs' );
    $this->assertContains( '/var/log/still/fake.log', $DS->getValues() );
  }

  /**
   * Confirm that translation happens in the name field.
   *
   * @depends testCustomGroupExistance
   */
  public function testNameLangEntries() {
    $this->_makeNodeworxCustomLogConfig();
    $DS = new DataSource_Log_Custom( 'Unit Test Custom Logs' );
    $this->assertContains( 'Test Log Custom', $DS->getDisplayValues() );
  }

  /**
   * Confirm that the custom SiteWorx group exists.
   */
  public function testSWCustomGroupExistance() {
    $this->_makeSiteworxCustomLogConfig();
    $DS = new DataSource_Log_SW_Groups();
    $this->assertContains( 'unittest-sw', $DS->getValues() );
  }

  /**
   * Tests the {$user} variable.
   */
  public function testUserVariableReplacement() {
    $this->_makeSiteworxCustomLogConfig();
    $this->_loginThisSiteworx( $this->getSWAccount() );

    $DS     = new DataSource_Log_Custom( 'unittest-sw' );
    $values = $DS->getValues();
    $this->assertContains( SW_UNIQNAME, $values[0] );
  }

  /**
   * Tests the {$domain} variable.
   */
  public function testDomainVariableReplacement() {
    $this->_makeSiteworxCustomLogConfig();
    $this->_loginThisSiteworx( $this->getSWAccount() );

    $DS     = new DataSource_Log_Custom( 'unittest-sw' );
    $values = $DS->getValues();
    $this->assertContains( SW_DOMAIN, $values[0] );
  }

  /**
   * Test that wildcard expansion resolved to a real file.
   */
  public function testWildcardExpensionSW() {
    $this->_makeSiteworxCustomLogConfig();
    $SW = $this->getSWAccount();
    $this->_loginThisSiteworx( $SW );

    $DS     = new DataSource_Log_Custom( 'unittest-sw' );
    $values = $DS->getValues();
    $path   = "/home/{$SW->getUniqname()}/var/{$SW->getWorkingDomainName()}" .
      '/logs/error.log';
    $this->assertContains( $path, $values );
  }

  /**
   * Makes the custom log config for NodeWorx.
   */
  private function _makeNodeworxCustomLogConfig() {
    $file    = self::NW_CUSTOM_FILE;
    $content = 'test-log-http:
  name: "Test log {$filename}"
  path: "/var/log/what/ever.log"
  group: "http"
test-log-custom:
  name: "Test Log ##LG_CUSTOM##"
  path: "/var/log/still/fake.log"
  group: "Unit Test Custom Logs"
  ';

    IWorxFileSys::smartBarf( $file, $content );
  }

  /**
   * Makes the custom log config for SiteWorx.
   */
  private function _makeSiteworxCustomLogConfig() {
    $file    = self::SW_CUSTOM_FILE;
    $content = 'test-log-http:
  name: "Test log {$filename}"
  path: "/home/{$user}/var/{$domain}/logs/*.log"
  group: "unittest-sw"
  ';

    IWorxFileSys::smartBarf( $file, $content );
  }
}
