<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Tim College <tcollege@interworx.com>
 * @date    Jun 11, 2009
 */

/**
 * Generic test related to Environment classes.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class EnvTest extends IWorx_Framework_TestCase {
  /**
   * Test.
   */
  public function testDefaultEnvironmentIsPhpUnit() {
    $this->assertTrue( IW::Env() instanceof Env_PHPUnit );
  }

  /**
   * Test.
   */
  public function testSwitchEnvironmentTo() {
    Env_Factory::switchEnvironmentTo( new Env_PHP() );

    $this->assertTrue( IW::Env() instanceof Env_PHP );
  }

  /**
   * Test.
   */
  public function testSwitchEnvironmentToAgain() {
    Env_Factory::switchEnvironmentTo( new Env_HTML() );

    $this->assertTrue( IW::Env() instanceof Env_HTML );
  }

  /**
   * Test.
   */
  public function testSwitchEnvironmentBack() {
    Env_Factory::switchEnvironmentBack();

    $this->assertTrue( IW::Env() instanceof Env_PHP );
  }

  /**
   * Test.
   */
  public function testSwitchEnvironmentBackAgain() {
    Env_Factory::switchEnvironmentBack();

    $this->assertTrue( IW::Env() instanceof Env_PHPUnit );
  }

  /**
   * Test.
   */
  public function testEnvSwitchAroundRouteFromPHP() {
    $NW = new NodeWorx( 1 );

    IW::FC()->routeFromPHP( 'Ctrl_Nodeworx_Siteworx',
                            'listMasterDomains',
                            array(),
                            $NW );

    $this->assertTrue( IW::Env() instanceof Env_PHPUnit );
  }

  /**
   * Test.
   */
  public function testPhpUnitGetSession() {
    $Session = IW::Env()->getNodeWorxSession();

    $this->assertTrue( $Session instanceof Session_NodeWorx );
  }

  /**
   * Test.
   */
  public function testEnvSwitchAroundRouteFromPHPToSiteWorx() {
    self::addSWAccountIfNecessary();
    $SW = new SiteWorx( self::SW_DOMAIN );

    IW::FC()->routeFromPHP( 'Ctrl_Siteworx_EmailBox', 'list', array(), $SW );

    $Session = IW::Env()->getNodeWorxSession();

    $this->assertTrue( $Session instanceof Session_NodeWorx );
  }

  /**
   * Checks that a non-existent flag returns false.
   */
  public function testNonExistantEnvFlag() {
    $this->assertFalse( IW::Env()->getEnvFlag( 'doesnt-exist' ) );
  }

  /**
   * Test setting a flag.
   */
  public function testSetEnvFlag() {
    $this->assertTrue( IW::Env()->setEnvFlag( 'test-flag' ) );
  }

  /**
   * Test that the set flag is gettable.
   *
   * @depends testSetEnvFlag
   */
  public function testSetEnvFlagPersists() {
    $this->assertTrue( IW::Env()->getEnvFlag( 'test-flag' ) );
  }

  /**
   * Tests clearing an env flag.
   *
   * @depends testSetEnvFlagPersists
   */
  public function testClearEnvFlag() {
    $this->assertTrue( IW::Env()->clearEnvFlag( 'test-flag' ) );

    $file = '/home/interworx/etc/env/test-flag';

    $this->assertFalse( IWorxFileSys::isFile( $file ) );
  }

  /**
   * Test setting a flag.
   */
  public function testSetEnvFlagWithContent() {
    $this->assertTrue( IW::Env()->setEnvFlag( 'test-flag2', 'content' ) );
  }

  /**
   * Test that the set flag is gettable.
   *
   * @depends testSetEnvFlagWithContent
   */
  public function testSetEnvFlagPersistsWithContent() {
    $this->assertEquals( 'content', IW::Env()->getEnvFlag( 'test-flag2' ) );
  }

  /**
   * Tests overriding session flag.
   */
  public function testSetEnvFlagForSession() {
    $this->assertTrue( IW::Env()->setEnvFlagForSession( 'test-flag2', 'c' ) );
  }

  /**
   * Test that the set flag is gettable.
   *
   * @depends testSetEnvFlagForSession
   */
  public function testSetEnvFlagForSessionPersists() {
    $this->assertEquals( 'c', IW::Env()->getEnvFlag( 'test-flag2' ) );
  }

  /**
   * Tests clearing an env flag with content.
   *
   * @depends testSetEnvFlagPersists
   */
  public function testClearEnvFlagWithContent() {
    $this->assertTrue( IW::Env()->clearEnvFlag( 'test-flag2' ) );

    $file = '/home/interworx/etc/env/test-flag2';

    $this->assertFalse( IWorxFileSys::isFile( $file ) );
  }

  /**
   * Tests clearing an env flag with content.
   *
   * @depends testSetEnvFlagForSessionPersists
   */
  public function testClearEnvFlagFromSession() {
    $this->assertTrue( IW::Env()->clearEnvFlagFromSession( 'test-flag2' ) );
    $this->assertFalse( IW::Env()->getEnvFlag( 'test-flag2' ) );
  }

  /**
   * Test clearing out all session variables.
   */
  public function testClearAllSessionVariables() {
    IW::Env()->setEnvFlagForSession( 'test-flag-3' );

    $this->assertTrue( IW::Env()->getEnvFlag( 'test-flag-3' ) );
    $this->assertTrue( IW::Env()->clearAllSessionEnvFlags() );
    $this->assertFalse( IW::Env()->getEnvFlag( 'test-flag-3' ) );
  }
}
