<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2011 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage TESTS
 * @author     Tim College <tcollege@interworx.com>
 * @date       Aug 23, 2011
 */

/**
 * Unit tests for the IPv6 utility class.
 *
 * @package    InterWorx
 * @subpackage TESTS
 */
class IPv6Test extends IWorx_Framework_TestCase {
  /**
   * Data Provider.
   *
   * @return array
   */
  public function dataProviderIPv6Expander() {
    return
      array(
            array( 'dead::beef', 'dead:0000:0000:0000:0000:0000:0000:beef' ),
            array( 'dead::beef::1', 'dead::beef::1' ),
            array( '::27', '0000:0000:0000:0000:0000:0000:0000:0027' ),
            array( '0000:0000:0000:0000:0000:0000:0000:0027',
                   '0000:0000:0000:0000:0000:0000:0000:0027' ),
            array( 'dead:::beef', 'dead:::beef' ),
            array( 'dead::::beef', 'dead::::beef' ),
            array( '5::34:3:192.168.34.32',
                   '0005:0000:0000:0000:0034:0003:c0a8:2220' ),
            array( 'dead::beef/24', 'dead:0000:0000:0000:0000:0000:0000:beef/24' ),
            array( 'dead::/::beef/32', 'dead::/::beef/32' ),
            );
  }

  /**
   * Data provider.
   *
   * @return array
   */
  public function dataProviderIPv6Compressor() {
    return
      array(
            array( 'dead::beef', 'dead:0000:0000:0000:0000:0000:0000:beef' ),
            array( 'dead::beef::1', 'dead::beef::1' ),
            array( '::27', '0000:0000:0000:0000:0000:0000:0000:0027' ),
            array( 'dead:::beef', 'dead:::beef' ),
            array( 'dead::::beef', 'dead::::beef' ),
            array( '5::34:3:c0a8:2220', '0005:0000:0000:0000:0034:0003:c0a8:2220' ),
            array( 'dead::beef/24', 'dead:0000:0000:0000:0000:0000:0000:beef/24' ),
            array( '255.255.255.0', '255.255.255.0' ),
            array( '127.0.0.1', '127.0.0.1' ),
            array( '2607:1:1:afe0:d045:7f00:0:ac', '2607:1:1:afe0:d045:7f00:0:ac'),
            array( '2607:1:1:2:d045:7f00:0:ac', '2607:1:1:2:d045:7f00:0000:ac'),
            array( '2607:f7c0:1:afe0:d045::ac', '2607:f7c0:1:afe0:d045:0:0:ac'),
            );
  }

  /**
   * Data provider.
   *
   * @return array
   */
  public function dataProviderFromDecimal() {
    return array(
                 array( '338620831926207318622244848606417780736', 'fec0::' ),
                 array( '14678624090587745565677611462493732864', 'b0b:b:0:b::' ),
                 array( '14678624090587745565677611462493732865', 'b0b:b:0:b::1' ),
                 );
  }

  /**
   * Data provider.
   *
   * @return array
   */
  public function dataProviderToDecimal() {
    return array(
                 array( 'fec0::', '338620831926207318622244848606417780736' ),
                 array( 'fec0:0000:0000:0000:0000:0000:0000:0001',
                        '338620831926207318622244848606417780737' ),
                 array( 'b0b:b:0:b::', '14678624090587745565677611462493732864' ),
                 array( 'b0b:b:0:b::1', '14678624090587745565677611462493732865' ),
                 array( '127.0.0.1', '2130706433' ),
                 array( '192.168.5.26', '3232236826' ),
                 array( '208.69.127.223', '3494215647' ),
                 );
  }

  /**
   * Data provider.
   *
   * @return array
   */
  public function dataProviderVersions() {
    return array(
                 array( '1.1.1.1', false ),
                 array( 'dead::beef', true ),
                 );
  }

  /**
   * Tests the expander.
   *
   * @param string $input
   * @param string $expected
   * @dataProvider dataProviderIPv6Expander
   */
  public function testExpander( $input, $expected ) {
    $this->assertEquals( $expected, IPv6::expand( $input ) );
  }

  /**
   * Tests the compressor.
   *
   * @param string $expected
   * @param string $input
   * @dataProvider dataProviderIPv6Compressor
   */
  public function testCompressor( $expected, $input ) {
    $this->assertEquals( $expected, IPv6::compress( $input ) );
  }

  /**
   * Tests converting from decimal to IPv6 notation.
   *
   * @param string $decimal
   * @param string $expected
   * @dataProvider dataProviderFromDecimal
   */
  public function testFromDecimal( $decimal, $expected ) {
    $this->assertEquals( $expected, IPv6::fromDecimal( $decimal ) );
  }

  /**
   * Tests converting to decimal from IPv6 notation.
   *
   * @param string $ipv6
   * @param string $expected
   * @dataProvider dataProviderToDecimal
   */
  public function testToDecimal( $ipv6, $expected ) {
    $this->assertEquals( $expected, IPv6::toDecimal( $ipv6 ) );
  }

  /**
   * Tests isIPv6().
   *
   * @param string $ip
   * @param string $expected
   * @dataProvider dataProviderVersions
   */
  public function testIsIPv6( $ip, $expected ) {
    $this->assertEquals( $expected, IPv6::isIPv6( $ip ) );
  }
}
