<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Tim College <tcollege@interworx.com>
 * @date    May 12, 2009
 */

/**
 * Tests for IW.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class IWTest extends IWorx_Framework_TestCase {
  /**
   * Tests that the current environment matches CLI.
   */
  public function testMatchesCLI() {
    $this->assertTrue( IW::Env()->matches( IW::CLI ) );
  }

  /**
   * Tests that the current environment matches CLI_INTERACTIVE.
   */
  public function testMatchesCLIInteractive() {
    $this->assertTrue( IW::Env()->matches( IW::CLI_INTERACTIVE ) );
  }

  /**
   * Tests that the current environment does not match CLI_NONINTERACTIVE.
   */
  public function testDoesNotMatchCLINonInteractive() {
    $this->assertFalse( IW::Env()->matches( IW::CLI_NONINTERACTIVE ) );
  }

  /**
   * Tests that the current environment matches ALL.
   */
  public function testMatchesAll() {
    $this->assertTrue( IW::Env()->matches( IW::ALL ) );
  }

  /**
   * Tests that the current environment does not match WEB.
   */
  public function testMatchesNotWeb() {
    $this->assertTrue( IW::Env()->matches( ~IW::WEB ) );
  }

  /**
   * Tests that the current environment does not match ~CLI.
   */
  public function testNoMatchesNotCLI() {
    $this->assertFalse( IW::Env()->matches( ~IW::CLI ) );
  }

  /**
   * Tests that the IW::tr function accepts 1 additional argument correctly.
   */
  public function testIWTrWorks() {
    $string   = '##LG_TESTING_LANGUAGE_SUBSTITUTION|40##';
    $expected = 'The number here should be 40.';

    $this->assertEquals( $expected, IW::tr( $string ) );
  }

  /**
   * Tests that the IW::tr function accepts multiple additional arguments.
   * correctly.
   */
  public function testIWTrWorksWithMultipleArgs() {
    $string = '##LG_TESTING_LANGUAGE_SUBSTITUTION_2|testing|test|123##';

    $expected = 'This is a test, only a test.  testing testing 123.';

    $this->assertEquals( $expected, IW::tr( $string ) );
  }
}
