<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Test
 * @author     Paul Oehler <poehler@interworx.com>
 * @date       Nov 16, 2015
 */

/**
 * Tests IWorx_Formatter_Link.
 *
 * @package    InterWorx
 * @subpackage Test
 */
class IWorx_Formatter_LinkTest extends IWorx_Framework_TestCase {

  /**
   * Returns some sample payload data to use with formatter testing.
   *
   * @return multitype:StdClass
   */
  private function _getSamplePayloadData() {
    return [ (object)['id' => '1', 'fname' => 'Paul', 'lname' => 'Oehler'],
             (object)['id' => '2', 'fname' => 'Tim',  'lname' => 'College'],
             (object)['id' => '3', 'fname' => 'Matt', 'lname' => 'Parmelee'] ];
  }
  /**
   * Test basic Link formatting.
   */
  public function testBasic() {
    $Data   = $this->_getSamplePayloadData();
    $Obj    = new IWorx_Formatter_Link( 'http://iwx.io/?uid=%%id%%' );
    $result = $Obj->format( $Data[0]->fname, $Data[0], $Data );
    $this->assertEquals( "<a href='http://iwx.io/?uid=1'>Paul</a>", $result );
  }

  /**
   * Test XSS attack in href.
   */
  public function testEntityInHref() {
    $Data   = $this->_getSamplePayloadData();
    $Data[0]->id = "'><script>alert('h4x0rd');</script>";
    $Obj    = new IWorx_Formatter_Link( 'http://iwx.io/?uid=%%id%%' );
    $result = $Obj->format( $Data[0]->fname, $Data[0], $Data );
    $clean_uid = IWorxSecUtil::html_entity_encode( $Data[0]->id );
    $this->assertEquals( "<a href='http://iwx.io/?uid={$clean_uid}'>Paul</a>",
                         $result,
                         'Link not properly escaped!' );
  }

  /**
   * Test XSS attack in value.
   */
  public function testEntityInValue() {
    $Data   = $this->_getSamplePayloadData();
    $Data[0]->fname = "><script>alert('h4x0rd');</script>";
    $Obj    = new IWorx_Formatter_Link( 'http://iwx.io/?uid=%%id%%' );
    $result = $Obj->format( $Data[0]->fname, $Data[0], $Data );
    $clean_fname = IWorxSecUtil::html_entity_encode( $Data[0]->fname );
    $this->assertEquals( "<a href='http://iwx.io/?uid=1'>{$clean_fname}</a>",
                         $result,
                         'Link not properly escaped!' );
  }

  /**
   * Test XSS attack in class.
   */
  public function testEntityInClass() {
    $Data = $this->_getSamplePayloadData();
    $Obj  = new IWorx_Formatter_Link( 'http://iwx.io/?uid=%%id%%' );
    $Obj->setClass( "'><script>alert('h4x0rd');</script>" );
    $result  = $Obj->format( $Data[0]->fname, $Data[0], $Data );
    $c = IWorxSecUtil::html_entity_encode( "'><script>alert('h4x0rd');</script>" );
    $this->assertEquals( "<a href='http://iwx.io/?uid=1' class='{$c}'>Paul</a>",
                         $result,
                         'Class not properly escaped!' );
  }

  /**
   * Test XSS attack in title.
   */
  public function testEntityInTitle() {
    $Data = $this->_getSamplePayloadData();
    $Obj  = new IWorx_Formatter_Link( 'http://iwx.io/?uid=%%id%%' );
    $Obj->setTitle( "'><script>alert('h4x0rd');</script>" );
    $result  = $Obj->format( $Data[0]->fname, $Data[0], $Data );
    $t = IWorxSecUtil::html_entity_encode( "'><script>alert('h4x0rd');</script>" );
    $this->assertEquals( "<a href='http://iwx.io/?uid=1' title='{$t}'>Paul</a>",
                         $result,
                         'Class not properly escaped!' );
  }
}