<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Tim College <tcollege@interworx.com>
 * @date    Apr 2, 2009
 */

/**
 * All tests for validators.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class IWorx_Validate_AllValidatorsGenericTest extends IWorx_Framework_TestCase {
  /**
   * Gets an array of all the validator classes.
   *
   * @return array
   */
  public function dataProviderValidateClasses() {
    $files = IWorxFileSys::rglob( '*.php', 0,
                                  '/usr/local/interworx/include/IWorx/Validate' );

    $output = [];

    foreach( $files as $file ) {
      $parts = explode( '/', $file );
      array_shift( $parts ); // remove /
      array_shift( $parts ); // remove usr/
      array_shift( $parts ); // remove local/
      array_shift( $parts ); // remove interworx/
      array_shift( $parts ); // remove include/
      $class = implode( '_', $parts );
      $class = substr( $class, 0, -4 );

      if( $this->_isStandardConstructor( $class ) ) {
        $output[] = array( $class );
      }
    }

    //sort( $output );

    return $output;
  }

  /**
   * Gets an array of all the validators and several data to test for each.
   *
   * @return array
   */
  public function dataProviderValidateClassesAndData() {
    $output = [];

    foreach( $this->dataProviderValidateClasses() as $array ) {
      $output[] = array( $array[0], 1 );
      $output[] = array( $array[0], '234' );
      $output[] = array( $array[0], 'a string' );
      $output[] = array( $array[0], 'INTERWORX_TIM_DEV' );
      $output[] = array( $array[0], 'email@email.com' );
      $output[] = array( $array[0], 'interworx.com' );
      $output[] = array( $array[0], array( '192.168.1.141' ) );
      $output[] = array( $array[0], array( 1, 2, 3 ) );
      //$output[] = array( $array[0], new DataSource_Array( array( 1, 2, 3, 4 ) ) );
    }

    return $output;
  }

  /**
   * Uses reflection to check if it's a standard constructor.
   *
   * @param string $class
   * @return boolean
   */
  private function _isStandardConstructor( $class ) {
    $RC = new ReflectionClass( $class );

    if( $RC->isAbstract() ) {
      return false;
    }

    $Constructor = $RC->getConstructor();

    if( is_object( $Constructor ) ) {
      return false;
    }

    return true;
  }

  /**
   * Instantiates a validator.
   *
   * @param string $class
   * @return Zend_Validate_Abstract
   */
  private function _getValidator( $class ) {
    return new $class();
  }

  /**
   * Some tests here require the dummy siteworx accounts.
   */
  public function testAddSwAccount() {
    self::addSWAccountIfNecessary();
  }

  /**
   * Tests the general behavior of isValid.
   *
   * @param string $class
   * @param mixed  $data
   *
   * @dataProvider dataProviderValidateClassesAndData
   */
  public function testIsValidBehavior( $class, $data ) {
    $Validator = $this->_getValidator( $class );

    $result = $Validator->isValid( $data );

    if( $result === true ) {
      $this->assertCount( 0, $Validator->getMessages() );
    } else if( $result === false ) {
      $count = count( $Validator->getMessages() );
      $this->assertGreaterThan( 0, $count );
    } else {
      $this->fail( 'Validator did not return boolean value' );
    }
  }
}
