<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2011 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Test
 * @author     Tim College <tcollege@interworx.com>
 * @date       Feb 21, 2011
 */

/**
 * Tests for IWorx_Validate_Contains
 *
 * @package    InterWorx
 * @subpackage Test
 */
class IWorx_Validate_ContainsTest extends IWorx_Framework_TestCase {
  /**
   * Tests good data for MODE_ANYWHERE.
   */
  public function testContainsAnywhere() {
    $V = new IWorx_Validate_Contains( 'contains', 'Test' );

    $this->assertTrue( $V->isValid( 'This contains it' ) );
  }

  /**
   * Tests bad data for MODE_ANYWHERE.
   */
  public function testContainsAnywhereFail() {
    $V = new IWorx_Validate_Contains( 'contains', 'Test' );

    $this->assertFalse( $V->isValid( 'This does not have it' ) );
  }

  /**
   * Tests good data for MODE_STARTS_WITH.
   */
  public function testStartsWith() {
    $V = new IWorx_Validate_Contains( 'Starts with',
                                      'Test',
                                      IWorx_Validate_Contains::MODE_STARTS_WITH );

    $this->assertTrue( $V->isValid( 'Starts with the right thing' ) );
  }

  /**
   * Tests bad data for MODE_STARTS_WITH.
   */
  public function testStartsWithFailOnWrongPosition() {
    $V = new IWorx_Validate_Contains( 'Starts with',
                                      'Test',
                                      IWorx_Validate_Contains::MODE_STARTS_WITH );

    $this->assertFalse( $V->isValid( 'Contains Starts with in wrong location' ) );
  }

  /**
   * Tests bad data for MODE_STARTS_WITH.
   */
  public function testStartsWithFailOnMissingEntirely() {
    $V = new IWorx_Validate_Contains( 'Starts with',
                                      'Test',
                                      IWorx_Validate_Contains::MODE_STARTS_WITH );

    $this->assertFalse( $V->isValid( 'Does not contain the test string' ) );
  }

  /**
   * Tests good data for MODE_ENDS_WITH.
   */
  public function testEndsWith() {
    $V = new IWorx_Validate_Contains( 'ends with this.',
                                      'Test',
                                      IWorx_Validate_Contains::MODE_ENDS_WITH );

    $this->assertTrue( $V->isValid( 'Make sure it ends with this.' ) );
  }

  /**
   * Tests good data for MODE_ENDS_WITH.
   */
  public function testEndsWithWhenThatsTheWholeString() {
    $V = new IWorx_Validate_Contains( 'ends with this.',
                                      'Test',
                                      IWorx_Validate_Contains::MODE_ENDS_WITH );

    $this->assertTrue( $V->isValid( 'ends with this.' ) );
  }

  /**
   * Tests bad data for MODE_ENDS_WITH.
   */
  public function testEndsWithFailOnWrongPosition() {
    $V = new IWorx_Validate_Contains( 'ends with this.',
                                      'Test',
                                      IWorx_Validate_Contains::MODE_ENDS_WITH );

    $this->assertFalse( $V->isValid( 'ends with this.  NOT!' ) );
  }

  /**
   * Tests bad data for MODE_ENDS_WITH.
   */
  public function testEndsWithFailOnMissingEntirely() {
    $V = new IWorx_Validate_Contains( 'ends with this.',
                                      'Test',
                                      IWorx_Validate_Contains::MODE_ENDS_WITH );

    $this->assertFalse( $V->isValid( 'Totally Missing' ) );
  }
}