<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Tim College <tcollege@interworx.com>
 * @date       Aug 26, 2008
 */

/**
 * Unit tests for IWorxDbUtil.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class IWorxDbUtilTest extends PHPUnit_Framework_TestCase {
  /**
   * Provider of test data for db_quote.
   *
   * @return array
   */
  public function dbQuoteData() {
    return array(
                 array( "Tim's Input", "'Tim\'s Input'" ),
                 array( 0, "'0'" ),
                 array( 2000, "'2000'" ),
                 array( '0', "'0'" ),
                 array( '0123string', "'0123string'" ),
                 array( '0A34BCDEF', "'0A34BCDEF'" ),
                 array( '1e72', "'1e72'" ),
                 array( md5( 'paul' ), "'6c63212ab48e8401eaf6b59b95d816a9'" ),
                 );
  }

  /**
   * Tests behavior of db_quote.
   *
   * @param mixed $input
   * @param mixed $expected_output
   * @dataProvider dbQuoteData
   * @group bug_972
   */
  public function testDbQuoteBehavior( $input, $expected_output ) {
    $db = IW::DB();
    $this->assertEquals( $expected_output, IWorxDbUtil::db_quote( $db, $input ) );
  }

  /**
   * Test that iworx_spam is not checked in iworxdb_tuneup.pex.
   */
  public function testGetTuneupTablesDoesNotContainIworxSpam() {
    $msg = 'NO NO NO.  Auto repairing of the iworx_spam database ' .
      'has caused big problems due to the apparently unbounded growth ' .
      'of the iworx_spam.awl and iworx_spam.bayes* database tables. ' .
      'Do NOT do this, unless you can guarantee that the iworx_spam ' .
      'database will not eventually contain gigabytes of data, which ' .
      'is harder than you think it is.  ';
    $this->assertNotContains( 'iworx_spam', IWorxDbUtil::getTuneupTables(), $msg );
  }
}
