<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Jonathan Daniel <jdaniel@interworx.com>
 * @date    Jun 16, 2009
 */

/**
 * Gzip Test.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class IWorxGzipTest extends PHPUnit_Framework_TestCase {

  const TEST_FILE_NAME = 'test.txt';
  const FILE_CONTENTS  = 'this is a test';

  /**
   * Test.
   *
   * Tests the GzipCompression helper function and ensures the source file.
   * is being removed successfully
   */
  public function testGzipCompression() {
    $tmp_dir           = Ini::get( Ini::DIR, 'tmp' );
    $testfile          = self::TEST_FILE_NAME;
    $uncompressed_path = "$tmp_dir/$testfile";

    $success = IWorxFileSys::smartBarf( $uncompressed_path,
                                        self::FILE_CONTENTS );

    $this->assertTrue( $success, 'Smart barf failed' );

    $gzip_path = IWorxGzip::compress( $uncompressed_path );

    $uncomporessed_exists = IWorxFileSys::fileExists( $uncompressed_path );
    $this->assertFalse( $uncomporessed_exists );

    $compressed_exists = IWorxFileSys::fileExists( $gzip_path );
    $this->assertTrue( $compressed_exists, "$gzip_path doesn't exist" );
  }

  /**
   * Test.
   *
   * Tests the GzipDecompression helper function and ensures the archive file.
   * is being removed successfully
   */
  public function testGzipDecompression() {
    $tmp_dir         = Ini::get( Ini::DIR, 'tmp' );
    $testfile        = self::TEST_FILE_NAME;
    $compressed_path = "$tmp_dir/$testfile.gz";

    $compressed_exists = IWorxFileSys::fileExists( $compressed_path );
    $this->assertTrue( $compressed_exists );

    $expected_uncompressed_path = "$tmp_dir/$testfile";

    $uncompressed_path = IWorxGzip::decompress( $compressed_path );

    $this->assertEquals( $expected_uncompressed_path, $uncompressed_path );

    $uncomporessed_exists = IWorxFileSys::fileExists( $uncompressed_path );
    $this->assertTrue( $uncomporessed_exists,
                       "$uncompressed_path should exist, but it doesn't" );

    $compressed_exists = IWorxFileSys::fileExists( $compressed_path );
    $this->assertFalse( $compressed_exists );

    $contents = file_get_contents( $uncompressed_path );
    $this->assertEquals( self::FILE_CONTENTS, $contents );

    //delete the test file
    IworxFileSys::rm( $uncompressed_path );
  }
}
