<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2013 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Paul Oehler <poehler@interworx.com>
 * @date    Nov 25, 2013
 */

/**
 * Tests for IWorxIPUtil.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class IWorxIPUtilTest extends PHPUnit_Framework_TestCase {
  /**
   * Data Provider.
   *
   * @return array
   */
  static public function dataProviderGetIPv4Range() {
    return array(
                 array( '127.0.0.1', 32, true,
                        array( 'start' => '127.0.0.1',
                               'end'   => '127.0.0.1' ) ),
                 array( '127.0.0.1', 31, true,
                        array( 'start' => '127.0.0.0',
                               'end'   => '127.0.0.1' ) ),
                 array( '127.0.0.1', 30, true,
                        array( 'start' => '127.0.0.0',
                               'end'   => '127.0.0.3' ) ),
                 array( '127.0.0.1', 29, true,
                        array( 'start' => '127.0.0.0',
                               'end'   => '127.0.0.7' ) ),
                 array( '127.0.0.1', 28, true,
                        array( 'start' => '127.0.0.0',
                               'end'   => '127.0.0.15' ) ),
                 array( '127.0.0.1', 27, true,
                        array( 'start' => '127.0.0.0',
                               'end'   => '127.0.0.31' ) ),
                 array( '127.0.0.1', 26, true,
                        array( 'start' => '127.0.0.0',
                               'end'   => '127.0.0.63' ) ),
                 array( '127.0.0.1', 25, true,
                        array( 'start' => '127.0.0.0',
                               'end'   => '127.0.0.127' ) ),
                 array( '127.0.0.1', 24, true,
                        array( 'start' => '127.0.0.0',
                               'end'   => '127.0.0.255' ) ),
                 array( '127.0.0.1', 23, true,
                        array( 'start' => '127.0.0.0',
                               'end'   => '127.0.1.255' ) ),
                 array( '127.0.0.1', 22, true,
                        array( 'start' => '127.0.0.0',
                               'end'   => '127.0.3.255' ) ),
                 array( '127.0.0.1', 21, true,
                        array( 'start' => '127.0.0.0',
                               'end'   => '127.0.7.255' ) ),
                 array( '127.0.0.1', 20, true,
                        array( 'start' => '127.0.0.0',
                               'end'   => '127.0.15.255' ) ),
                 array( '127.0.0.1', 19, true,
                        array( 'start' => '127.0.0.0',
                               'end'   => '127.0.31.255' ) ),
                 array( '127.0.0.1', 18, true,
                        array( 'start' => '127.0.0.0',
                               'end'   => '127.0.63.255' ) ),
                 array( '127.0.0.1', 17, true,
                        array( 'start' => '127.0.0.0',
                               'end'   => '127.0.127.255' ) ),
                 array( '127.0.0.1', 16, true,
                        array( 'start' => '127.0.0.0',
                               'end'   => '127.0.255.255' ) ),
                 array( '127.0.0.1', 15, true,
                        array( 'start' => '127.0.0.0',
                               'end'   => '127.1.255.255' ) ),
                 array( '127.0.0.1', 14, true,
                        array( 'start' => '127.0.0.0',
                               'end'   => '127.3.255.255' ) ),
                 array( '127.0.0.1', 13, true,
                        array( 'start' => '127.0.0.0',
                               'end'   => '127.7.255.255' ) ),
                 array( '127.0.0.1', 12, true,
                        array( 'start' => '127.0.0.0',
                               'end'   => '127.15.255.255' ) ),
                 array( '127.0.0.1', 11, true,
                        array( 'start' => '127.0.0.0',
                               'end'   => '127.31.255.255' ) ),
                 array( '127.0.0.1', 10, true,
                        array( 'start' => '127.0.0.0',
                               'end'   => '127.63.255.255' ) ),
                 array( '127.0.0.1', 9, true,
                        array( 'start' => '127.0.0.0',
                               'end'   => '127.127.255.255' ) ),
                 array( '127.0.0.1', 8, true,
                        array( 'start' => '127.0.0.0',
                               'end'   => '127.255.255.255' ) ),

                 array( '208.69.120.208', 28, true,
                        array( 'start' => '208.69.120.208',
                               'end'   => '208.69.120.223' ) ),

                 array( '208.69.120.1', 28, true,
                        array( 'start' => '208.69.120.0',
                               'end'   => '208.69.120.15' ) ),
                 array( '208.69.120.16', 28, true,
                        array( 'start' => '208.69.120.16',
                               'end'   => '208.69.120.31' ) ),
                 array( '208.69.120.32', 28, true,
                        array( 'start' => '208.69.120.32',
                               'end'   => '208.69.120.47' ) ),
                 array( '208.69.120.48', 28, true,
                        array( 'start' => '208.69.120.48',
                               'end'   => '208.69.120.63' ) ),
                 array( '208.69.120.64', 28, true,
                        array( 'start' => '208.69.120.64',
                               'end'   => '208.69.120.79' ) ),
                 array( '208.69.120.80', 28, true,
                        array( 'start' => '208.69.120.80',
                               'end'   => '208.69.120.95' ) ),
                 array( '208.69.120.96', 28, true,
                        array( 'start' => '208.69.120.96',
                               'end'   => '208.69.120.111' ) ),
                 array( '208.69.120.112', 28, true,
                        array( 'start' => '208.69.120.112',
                               'end'   => '208.69.120.127' ) ),
                 array( '208.69.120.128', 28, true,
                        array( 'start' => '208.69.120.128',
                               'end'   => '208.69.120.143' ) ),
                 array( '208.69.120.144', 28, true,
                        array( 'start' => '208.69.120.144',
                               'end'   => '208.69.120.159' ) ),
                 array( '208.69.120.160', 28, true,
                        array( 'start' => '208.69.120.160',
                               'end'   => '208.69.120.175' ) ),
                 array( '208.69.120.176', 28, true,
                        array( 'start' => '208.69.120.176',
                               'end'   => '208.69.120.191' ) ),
                 array( '208.69.120.192', 28, true,
                        array( 'start' => '208.69.120.192',
                               'end'   => '208.69.120.207' ) ),
                 array( '208.69.120.208', 28, true,
                        array( 'start' => '208.69.120.208',
                               'end'   => '208.69.120.223' ) ),
                 array( '208.69.120.224', 28, true,
                        array( 'start' => '208.69.120.224',
                               'end'   => '208.69.120.239' ) ),
                 array( '208.69.120.240', 28, true,
                        array( 'start' => '208.69.120.240',
                               'end'   => '208.69.120.255' ) ),

                 array( '8.8.8.8', '255.255.255.255', false,
                        array( 'start' => '8.8.8.8',
                               'end'   => '8.8.8.8' ) ),
                 array( '8.8.8.8', '255.255.255.0', false,
                        array( 'start' => '8.8.8.0',
                               'end'   => '8.8.8.255' ) ),
                 array( '8.8.8.8', '255.255.255.252', false,
                        array( 'start' => '8.8.8.8',
                               'end'   => '8.8.8.11' ) ),
                 array( '8.8.8.9', '255.255.255.252', false,
                        array( 'start' => '8.8.8.8',
                               'end'   => '8.8.8.11' ) ),
                 array( '8.8.8.0', '255.255.255.128', false,
                        array( 'start' => '8.8.8.0',
                               'end'   => '8.8.8.127' ) ),

                 array( '222.222.222.222', '255.255.0.0', false,
                        array( 'start' => '222.222.0.0',
                               'end'   => '222.222.255.255' ) ),
                 );
  }

  /**
   * Data Provider.
   *
   * @return array
   */
  static public function badIsIPv4InRange() {
    return array( array( '208.69.126.71', '208.69.120.208/28' ),
                  array( '127.0.0.1',     '8.8.8.0/8' ),
                  array( '192.168.1.100', '192.168.2.0/24' ),
                  array( '192.168.1.255', '192.168.1.0/25' ),
                  array( '192.168.55.55', '192.168.55.100/28' ),
                  );
  }

  /**
   * Data Provider.
   *
   * @return array
   */
  static public function goodIsIPv4InRange() {
    return array( array( '208.69.126.71', '208.69.126.71/28' ),
                  array( '8.100.100.100',  '8.8.8.0/8' ),
                  array( '192.168.1.100', '192.168.1.0/24' ),
                  array( '192.168.1.255', '192.168.1.0/24' ),
                  array( '192.168.55.55', '192.168.55.40/25' ),
                  );
  }

  /**
   * Test.
   *
   * @param string $ip
   * @param string $range
   * @dataProvider badIsIPv4InRange
   */
  public function testIsIPv4InRangeBad( $ip, $range ) {
    $this->assertFalse( IWorxIPUtil::isIPv4InRange( $ip, $range ) );
  }

  /**
   * Test.
   *
   * @param string $ip
   * @param string $range
   * @dataProvider goodIsIPv4InRange
   */
  public function testIsIPv4InRangeGood( $ip, $range ) {
    $this->assertTrue( IWorxIPUtil::isIPv4InRange( $ip, $range ) );
  }

  /**
   * Test.
   *
   * @param string $ip
   * @param string $mask
   * @param string $cidr
   * @param array  $expected
   * @dataProvider dataProviderGetIPv4Range
   */
  public function testGetIpv4Range( $ip, $mask, $cidr, array $expected ) {
    $this->assertEquals( $expected, IWorxIPUtil::getIPv4Range( $ip, $mask, $cidr ) );
  }

}
