<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Tim College <tcollege@interworx.com>
 * @date       Oct 30, 2008
 */

/**
 * Tests the base Input_Input class.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class Input_InputTest extends PHPUnit_Framework_TestCase {
  const INPUT_NAME  = 'test';
  const INPUT_VALUE = 'initial value';

  /**
   * Builds an input for tests.
   *
   * @param string $name
   * @return Input_Input
   */
  private function _getInput( $name = self::INPUT_NAME ) {
    return new Input_Input( $name, self::INPUT_VALUE );
  }

  /**
   * Tests behavior of getCurrentValue()
   */
  public function testGetCurrentValue() {
    $I = $this->_getInput();

    $this->assertEquals( self::INPUT_VALUE, $I->getInitialValue(),
                         'The initial value function is busted' );
    $this->assertEquals( self::INPUT_VALUE, $I->getCurrentValue(),
                         'The current value did not return the initial value' );

    $I->setCurrentValue( 'current' );

    $this->assertEquals( 'current', $I->getCurrentValue(),
                         'The current value is not what was expected' );
    $this->assertEquals( self::INPUT_VALUE, $I->getInitialValue(),
                         'The initial value has changed, which is stupid' );
  }

  /**
   * Tests behavior of getLabel()
   */
  public function testGetLabel() {
    $I = $this->_getInput();

    $this->assertEquals( '##LG_TEST##', $I->getLabel(),
                         'getLabel() did not return a lang\'ed guess' );

    $I->setLabel( 'DISPLAYNAME' );

    $this->assertEquals( 'DISPLAYNAME', $I->getLabel(),
                         'getLabel() did not return user-set name' );
  }

  /**
   * Tests the generation of the quickhelp.
   */
  public function testGetQuickHelp() {
    $I = $this->_getInput( 'HTACCESS_HOTLINK_EXTENSIONS' );

    $this->assertEquals( 'LG_QH_HTACCESS_HOTLINK_EXTENSIONS',
                         $I->getQuickHelp() );
  }

  /**
   * Tests the behavior of validate()
   */
  public function testValidateDefault() {
    $I = $this->_getInput();

    $this->assertTrue( $I->validate(),
                       'validate() returned false with good input' );
  }

  /**
   * Tests behavior of validate when required and no input is given.
   */
  public function testValidateWithBlankInput() {
    $I = $this->_getInput();

    $I->setCurrentValue( '' );

    $this->assertFalse( $I->validate(),
                        'validate() returned true with bad input' );

    $this->assertInternalType( 'array', $I->getErrors(),
                               'Array of errors was not returned' );
    $this->assertTrue( $I->hasErrors() );
  }

  /**
   * Tests behevior of validate when not required and no input is given.
   */
  public function testValidateWithBlankInputWhenNotRequired() {
    $I = $this->_getInput();

    $I->setRequired( false );

    $this->assertTrue( $I->validate(),
                       'validate() returned false with good input' );
  }
}
