<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Jonathan Daniel <jdaniel@interworx.com>
 * @date    Mar 7, 2011
 */

/**
 * Tests for the Siteworx_User Model.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class Model_Siteworx_UserTest extends IWorx_Framework_TestCase {

  const TEST_EMAIL = 'unit.test.email@test.example.com';

  /**
   * Delete all the test data.
   */
  static private function _deleteTestData() {
    $q_test_email = q( self::TEST_EMAIL );

    $sql   = "email = {$q_test_email}";
    $users = Model_Siteworx_User::find( $sql );

    foreach( $users as $User ) {
      $User->delete();
    }
  }

  /**
   * Set up before class.
   */
  static public function setUpBeforeClass() {
    self::_deleteTestData();
  }

  /**
   * Tear down after class.
   */
  static public function tearDownAfterClass() {
    self::_deleteTestData();
  }

  /**
   * Tests the create function.
   */
  public function testCreate() {
    $User = Model_Siteworx_User::create();

    $this->assertTrue( $User instanceof Model_Siteworx_User );
  }

  /**
   * Gets a test package.
   *
   * @return Package
   */
  private function _getTestPackage() {
    $SiteWorx = $this->getSWAccount();
    return $SiteWorx->getPackage();
  }

  /**
   * Tests building the user.
   */
  public function testBuildingUser() {
    $User = Model_Siteworx_User::create();

    $nickname   = 'UNIT TEST USER';
    $email      = self::TEST_EMAIL;
    $password   = IWorxUtil::hashWithSalt( 'abc123' );
    $global_uid = Model_User::PENDING;
    $status     = Model_User::INACTIVE;
    $language   = 'en-us';
    $theme      = 'interworx';
    $type       = Model_User::MASTER;
    $package_id = $this->_getTestPackage()->getId();
    $menu_style = Model_User::MENU_SMALL;

    $User->nickname   = $nickname;
    $User->global_uid = $global_uid;
    $User->email      = $email;
    $User->password   = $password;
    $User->status     = $status;
    $User->language   = $language;
    $User->theme      = $theme;
    $User->type       = $type;
    $User->package_id = $package_id;
    $User->menu_style = $menu_style;

    $User->save();

    $this->assertEquals( $nickname,   $User->nickname );
    $this->assertEquals( $email,      $User->email );
    $this->assertEquals( $password,   $User->password );
    $this->assertEquals( $theme,      $User->theme );
    $this->assertEquals( $language,   $User->language );
    $this->assertEquals( $status,     $User->status );
    $this->assertEquals( $type,       $User->type );
    $this->assertEquals( $menu_style, $User->menu_style );
    $this->assertEquals( $package_id, $User->package_id );

    $this->assertNotEquals( $global_uid, $User->global_uid );
    $this->assertEquals( "sw{$User->getPrimaryId()}", $User->global_uid );
  }

  /**
   * Tests that the initial permissions are empty.
   */
  public function testInitialPermissionsAreEmpty() {
    $package_id = $this->_getTestPackage()->getId();
    $User       = Model_Siteworx_User::getByEmailAndPackageId( self::TEST_EMAIL,
                                                               $package_id );
    $perms = $User->getPermissions();

    $this->assertTrue( empty( $perms ) );
  }

  /**
   * Tests setting the user's permissions.
   */
  public function testSettingPermissions() {
    $package_id = $this->_getTestPackage()->getId();
    $User       = Model_Siteworx_User::getByEmailAndPackageId( self::TEST_EMAIL,
                                                               $package_id );
    $perm_names = array( Session_SiteWorx::PERM_MASTER );

    $User->setPermissions( $perm_names );
    $User->save();

    $permissions = $User->getPermissions();

    $this->assertArrayHasKey( Session_SiteWorx::PERM_MASTER, $permissions );
  }

  /**
   * Tests that loading the user's permissions worked.
   */
  public function testLoadingPermissionsWorked() {
    $package_id = $this->_getTestPackage()->getId();
    $User       = Model_Siteworx_User::getByEmailAndPackageId( self::TEST_EMAIL,
                                                               $package_id );
    $permissions = $User->getPermissions();

    $this->assertArrayHasKey( Session_SiteWorx::PERM_MASTER, $permissions );
  }
}