<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Test
 * @author     Jon Daniel <jdaniel@interworx.com>
 */

/**
 * Test suite for Ldirectord tests.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class LdirectordTest extends IWorx_Cluster_Framework_TestCase {
  /**
   * Setup.
   */
  public function setUp() {
    if( IW::Env()->isManager() === false ) {
      $this->markTestSkipped( 'Not a clustered environment' );
    }
  }

  /**
   * Tests to make sure sync is working correctly.
   */
  public function testSync() {
    $Ldirectord = new Ldirectord();

    $this->assertTrue( $Ldirectord->sync() );
  }

  /**
   * Tests that the getServiceFromPort command is working properly.
   */
  public function testGetServiceFromPort() {
    $ftp = Ldirectord::getServiceFromPort( '21' );
    $this->assertEquals( 'ftp', $ftp );

    $http = Ldirectord::getServiceFromPort( '80' );
    $this->assertEquals( 'http', $http );

    $none = Ldirectord::getServiceFromPort( '65000' );
    $this->assertEquals( 'none', $none );
  }

  /**
   * Tests the ldirectord start function.
   */
  public function testStart() {
    $Ldirectord = new Ldirectord();

    $this->assertTrue( $Ldirectord->start() );
    sleep( 2 );
  }

  /**
   * Tests the ldirectord stop function.
   */
  public function testStop() {
    $Ldirectord = new Ldirectord();

    $this->assertTrue( $Ldirectord->stop() );
    sleep( 2 );
  }

  /**
   * Tests the restart function.
   */
  public function testRestart() {
    $Ldirectord = new Ldirectord();

    $this->assertTrue( $Ldirectord->restart() );
    sleep( 2 );
  }

  /**
   * Tests the startOnBoot function.
   */
  public function testStartOnBoot() {
    $Ldirectord = new Ldirectord();

    $is_sob = $Ldirectord->isStartedOnBoot();

    $Ldirectord->startOnBoot( true );
    $this->assertTrue( $Ldirectord->isStartedOnBoot() );

    $Ldirectord->startOnBoot( false );
    $this->assertFalse( $Ldirectord->isStartedOnBoot() );

    $Ldirectord->startOnBoot( $is_sob );
  }

  /**
   * Tests the isRunning function.
   */
  public function testIsRunning() {
    $Ldirectord = new Ldirectord();

    $Ldirectord->start();
    $this->assertTrue( $Ldirectord->isRunning() );
  }

  /**
   * Tests that the persistent workaround is functioning correctly.
   */
  public function testPersistentWorkaround() {
    $echo = Ini::get( Ini::BIN, 'echo' );
    $cmd  = "{$echo} 0 > /proc/sys/net/ipv4/vs/expire_quiescent_template";
    IWorxExec::exec( $cmd, $result, $retval, IWorxExec::ALLOW_REDIR_OUT );

    $Ldirectord = new Ldirectord();
    $Ldirectord->enablePersistentWorkaround();

    $cat = Ini::get( Ini::BIN, 'cat' );
    $cmd = "{$cat} /proc/sys/net/ipv4/vs/expire_quiescent_template";
    IWorxExec::exec( $cmd, $result, $retval );

    $this->assertEquals( '1', $result[0] );
  }
}
