<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2014 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage TESTS
 * @author     Tim College <tcollege@interworx.com>
 * @date       May 28, 2014
 */

/**
 * Unit test for Trac #3498
 *
 * @package    InterWorx
 * @subpackage TESTS
 */
class Bug3498PhpMyAdminSessionTest extends IWorx_Framework_TestCase {
  /**
   * The phpMyAdmin item is hidden in the CLI, but we need it for this test.
   *
   * @beforeClass
   */
  public function unhidePMAInMenu() {
    $path = '/home/interworx/etc/siteworx-menu.ini';
    $menu = parse_ini_file( $path, true );
    $pma  = $menu['iw-menu-features-mysql-pma'];
    $pma['hide'] = '';

    $Manager = new IWorxMenuManager( $menu, 'etc/siteworx-menu.ini' );
    $Manager->updateMenuItem( 'iw-menu-features-mysql-pma', $pma );
    $Manager->save();
  }

  /**
   * Rehide the menu item.
   *
   * @afterClass
   */
  public function hidePMAInMenu() {
    $path = '/home/interworx/etc/siteworx-menu.ini';
    $menu = parse_ini_file( $path, true );
    $pma  = $menu['iw-menu-features-mysql-pma'];
    $pma['hide'] = '~WEB';

    $Manager = new IWorxMenuManager( $menu, 'etc/siteworx-menu.ini' );
    $Manager->updateMenuItem( 'iw-menu-features-mysql-pma', $pma );
    $Manager->save();
  }

  /**
   * Tests exploit from #3498.
   */
  public function testPMASessionExploit() {
    $target = '/etc/bug-3498-target';
    IWorxFileSys::rm( $target, true, true );
    IWorxFileSys::rm( '/tmp/sess_naughty' );

    // same default perms/owner as /etc/shadow
    $expected = 'top secret info here';
    file_put_contents( $target, $expected );
    chmod( $target, 000 );
    chown( $target, 'root' );
    chgrp( $target, 'root' );

    `ln -s {$target} /tmp/sess_naughty`;

    $SW = $this->getSWAccount();

    $Driver   = $SW->getMySQLDriver();

    if( $Driver->databaseExists( 'db3498' ) === false ) {
      $Driver->addDatabase( 'db3498' );
    }

    $command = 'siteworx -u -n --login_domain ' . SW_DOMAIN .
      ' -c MysqlPhpmyadmin -a signon --phpMyAdmin naughty';

    $Cmd = new Command();
    $Cmd->setCmd( $command );
    $Cmd->stderrToStdout();
    $Cmd->exec();

    /*
     * This is a little misleading - the actual login really fails, but because of
     * the redirect, there's no non-zero return value. We're checking that it
     * "succeeds" because at one point in the testing, it was actually stacktracing
     * here.
     */
    $this->assertTrue( $Cmd->wasSuccessful() );

    $actual = file_get_contents( $target );

    $this->assertEquals( $expected, $actual );
  }

}
