<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2014 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage TESTS
 * @author     Tim College <tcollege@interworx.com>
 * @date       Apr 29, 2014
 */

/**
 * Test for a race condition in IWorxFileSys::mkdir().
 *
 * @package    InterWorx
 * @subpackage TESTS
 */
class Bug3573IWorxFileSysMkdirRaceConditionTest extends IWorx_Framework_TestCase {
  const TICK_FAIL_ENV = 'bug-3573-tick-fail';
  const TARGET_DIR    = '/tmp/bug3573/what/ever/';

  /**
   * Checks if the folder was created with bad permissions at some point.
   */
  public function tickHandlerCheckExploitableOwnership() {
    // Because caching sucks, that's why.
    clearstatcache();

    if( IWorxFileSys::fileExists( self::TARGET_DIR ) ) {
      if( IW::Env()->getEnvFlag( self::TICK_FAIL_ENV ) ) {
        // Without this return, it takes FOR FUCKING EVER to run this.
        return;
      }
      $stats = IWorxFileSys::stat( self::TARGET_DIR );
      if( $stats['uid'] == 0 ) {
        $msg = 'Folder was created with incorrect permissions';
        IW::Env()->setEnvFlagForRequest( self::TICK_FAIL_ENV, $msg );
      }
    }
  }

  /**
   * Test a race condition that existed in IWorxFileSys::mkdir.
   */
  public function testMkdirRace() {
    IWorxFileSys::rm( '/tmp/bug3573', true );
    IW::Env()->clearEnvFlagFromRequest( self::TICK_FAIL_ENV );

    declare( ticks = 1 );
    $function = 'tickHandlerCheckExploitableOwnership';

    register_tick_function( array( $this, $function ) );
    IWorxFileSys::mkdir( self::TARGET_DIR, 0600, true, 'iworx' );
    unregister_tick_function( array( $this, $function ) );

    $stats = IWorxFileSys::stat( self::TARGET_DIR );
    $this->assertEquals( 250, $stats['uid'] );

    if( IW::Env()->getEnvFlag( self::TICK_FAIL_ENV ) ) {
      $msg = IW::Env()->getEnvFlag( self::TICK_FAIL_ENV );
      IW::Env()->clearEnvFlagFromRequest( self::TICK_FAIL_ENV );
      $this->fail( $msg );
    }
  }
}
